package jp.co.kayo.android.localplayer;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 *      This program is free software; you can redistribute it and/or modify it under
 *      the terms of the GNU General Public License as published by the Free Software
 *      Foundation; either version 2 of the License, or (at your option) any later
 *      version.
 *      
 *      This program is distributed in the hope that it will be useful, but WITHOUT
 *      ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *      FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 *      details.
 *      
 *      You should have received a copy of the GNU General Public License along with
 *      this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *      Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.util.ArrayList;
import java.util.List;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.TabsAdapter.TabInfo;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.core.ContextMenuFragment;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.provider.DeviceContentProvider;
import jp.co.kayo.android.localplayer.service.IMediaPlayerService;
import jp.co.kayo.android.localplayer.service.IMediaPlayerServiceCallback;
import jp.co.kayo.android.localplayer.service.MediaPlayerService;
import jp.co.kayo.android.localplayer.ui.AlbumGridViewFragment;
import jp.co.kayo.android.localplayer.ui.AlbumListViewFragment;
import jp.co.kayo.android.localplayer.ui.ArtistListViewFragment;
import jp.co.kayo.android.localplayer.ui.ControlFragment;
import jp.co.kayo.android.localplayer.ui.FileListViewFragment;
import jp.co.kayo.android.localplayer.ui.GenresListViewFragment;
import jp.co.kayo.android.localplayer.ui.IProgressView;
import jp.co.kayo.android.localplayer.ui.MediaListViewFragment;
import jp.co.kayo.android.localplayer.ui.PlayOrderListViewFragment;
import jp.co.kayo.android.localplayer.ui.VideoListViewFragment;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.StrictHelper;
import jp.co.kayo.android.localplayer.util.ThemeHelper;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.NfcHelper.BeamMessage;
import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.AlertDialog;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.pm.PackageManager;
import android.content.pm.ProviderInfo;
import android.database.ContentObserver;
import android.database.Cursor;
import android.media.AudioManager;
import android.net.Uri;
import android.nfc.NfcAdapter;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.preference.PreferenceManager;
import android.support.v4.view.ViewPager;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager.LayoutParams;

public class MainActivity extends BaseActivity {
    SharedPreferences mPref;
    ViewPager mViewPager;
    private BillingService mBillingService = null;
    DungeonsPurchaseObserver mDungeonsPurchaseObserver;
    private ViewCache mViewCache;
    TabsAdapter mTabsAdapter;
    Handler mHandler = new Handler();
    private Intent oneIntent;

    public PlayOrderListViewFragment mPlayList;

    IMediaPlayerServiceCallback.Stub mCallback = new IMediaPlayerServiceCallback.Stub() {
        ControlFragment control;

        ControlFragment getControll() {
            if (control == null) {
                FragmentManager m = getSupportFragmentManager();
                control = (ControlFragment) m
                        .findFragmentByTag(SystemConsts.TAG_CONTROL);
            }
            return control;
        }

        @Override
        public void updateView(final boolean updatelist) throws RemoteException {
            Logger.d("updateView=" + updatelist);
            // もし、コントロール部分が表示されているならリストを更新してあげて
            if (mHandler != null) {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        // AlbumList,ArtistList,Song等
                        ControlFragment control = getControll();
                        // 再生中の曲の更新
                        if (mBinder != null) {
                            try {
                                long media_id = mBinder.getMediaId();
                                int pos = mBinder.getPosition();
                                long pref_id = mBinder.getPrefetchId();
                                Logger.i("media_id=" + media_id + " " + "pos="
                                        + pos + " " + "pref_id=" + pref_id);
                                mViewCache.setPosition(pos, media_id, pref_id);
                            } catch (RemoteException e) {
                            }
                        }

                        // 更新処理
                        if (control != null) {
                            control.updateView();
                        }

                        // 変更を通知
                        for (final TabInfo tabinfo : mTabsAdapter.getTabs()) {
                            if (tabinfo.fragment_tag != null) {
                                ContentManager cntmgr = (ContentManager) getSupportFragmentManager()
                                        .findFragmentByTag(tabinfo.fragment_tag);
                                if (cntmgr != null) {
                                    if ((cntmgr instanceof PlayOrderListViewFragment) != true) {
                                        cntmgr.changedMedia();
                                    } else {
                                        if (updatelist) {
                                            cntmgr.reload();
                                        } else {
                                            cntmgr.changedMedia();
                                        }
                                    }
                                }
                            }
                        }
                    }
                });
            }
        }

        @Override
        public void updateList() throws RemoteException {
            // もし、再生中のフラグメントが表示されているならリストを更新してあげて
        }

        @Override
        public void onBufferingUpdate(int percent) throws RemoteException {
            // バッファリング中のプログレスバーですよぉ
            ControlFragment f = getControll();
            if (f != null) {
                f.onBufferingUpdate(percent);
            }
        }

        @Override
        public void startProgress(long max) throws RemoteException {
            long media_id = mBinder.getMediaId();
            int pos = mBinder.getPosition();
            long pref_id = mBinder.getPrefetchId();
            mViewCache.setPosition(pos, media_id, pref_id);
            Fragment current = getCurrentFragment();
            if (current != null && current instanceof IProgressView) {
                ((IProgressView) current).startProgress(max);
            }
        }

        @Override
        public void stopProgress() throws RemoteException {
            Fragment current = getCurrentFragment();
            if (current != null && current instanceof IProgressView) {
                ((IProgressView) current).stopProgress();
            }
        }

        @Override
        public void progress(long pos, long max) throws RemoteException {
            Fragment current = getCurrentFragment();
            if (current != null && current instanceof IProgressView) {
                ((IProgressView) current).progress(pos, max);
            }
        }
    };

    ContentObserver mAlbumContentObserver = new ContentObserver(new Handler()) {
        @Override
        public void onChange(boolean selfChange) {
            super.onChange(selfChange);
            // データベースに変更があったときに呼び出したい処理を書く
            for (final TabInfo tabinfo : mTabsAdapter.getTabs()) {
                if (tabinfo.fragment_tag != null) {
                    ContentManager cntmgr = (ContentManager) getSupportFragmentManager()
                            .findFragmentByTag(tabinfo.fragment_tag);
                    if (cntmgr != null) {
                        if ((cntmgr instanceof AlbumListViewFragment)
                                || (cntmgr instanceof AlbumGridViewFragment)) {
                            cntmgr.reload();
                        }
                    }
                }
            }
        }
    };

    ContentObserver mArtistContentObserver = new ContentObserver(new Handler()) {
        @Override
        public void onChange(boolean selfChange) {
            super.onChange(selfChange);
            // データベースに変更があったときに呼び出したい処理を書く
            for (final TabInfo tabinfo : mTabsAdapter.getTabs()) {
                if (tabinfo.fragment_tag != null) {
                    ContentManager cntmgr = (ContentManager) getSupportFragmentManager()
                            .findFragmentByTag(tabinfo.fragment_tag);
                    if (cntmgr != null) {
                        if ((cntmgr instanceof ArtistListViewFragment)) {
                            cntmgr.reload();
                        }
                    }
                }
            }
        }
    };

    ContentObserver mMediaContentObserver = new ContentObserver(new Handler()) {
        @Override
        public void onChange(boolean selfChange) {
            super.onChange(selfChange);
            // データベースに変更があったときに呼び出したい処理を書く
            for (final TabInfo tabinfo : mTabsAdapter.getTabs()) {
                if (tabinfo.fragment_tag != null) {
                    ContentManager cntmgr = (ContentManager) getSupportFragmentManager()
                            .findFragmentByTag(tabinfo.fragment_tag);
                    if (cntmgr != null) {
                        if ((cntmgr instanceof MediaListViewFragment)) {
                            cntmgr.reload();
                        } else if ((cntmgr instanceof PlayOrderListViewFragment)) {
                            cntmgr.reload();
                        }
                    }
                }
            }
        }
    };
    
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent arg2) {
        super.onActivityResult(requestCode, resultCode, arg2);
        if(requestCode == SystemConsts.REQUEST_TAGEDIT){
            // 変更を通知
            for (final TabInfo tabinfo : mTabsAdapter.getTabs()) {
                if (tabinfo.fragment_tag != null) {
                    ContentManager cntmgr = (ContentManager) getSupportFragmentManager()
                            .findFragmentByTag(tabinfo.fragment_tag);
                    if (cntmgr != null) {
                        cntmgr.reload();
                    }
                }
            }
        }
    }
    

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        new ThemeHelper().selectTheme(this);
        StrictHelper.registStrictMode();
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        super.onCreate(savedInstanceState);

        mPref = PreferenceManager.getDefaultSharedPreferences(this);

        setContentView(R.layout.main_tab);

        hideProgressBar();
        getSupportActionBar().setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
        getSupportActionBar().setSubtitle(getString(R.string.hello));

        getWindow().setSoftInputMode(
                LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);

        // SDCardChec
        Funcs.checkSDCard(this);

        // Audio Volume
        setVolumeControlStream(AudioManager.STREAM_MUSIC);

        // ViewCache
        mViewCache = (ViewCache) getSupportFragmentManager().findFragmentByTag(
                SystemConsts.TAG_CACHE);

        // setup
        mViewPager = (ViewPager) findViewById(R.id.pager);

        mTabsAdapter = new TabsAdapter(this, getSupportActionBar(), mViewPager);

        if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_ALBUM, true)) {
            String type = mPref.getString("key.type_album", "0");
            if ("0".equals(type)) {
                ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                        getString(R.string.lb_tab_albums));
                tab.setTag(SystemConsts.TAG_ALBUM);
                mTabsAdapter.addTab(tab, AlbumGridViewFragment.class, null);
            } else {
                ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                        getString(R.string.lb_tab_albums));
                tab.setTag(SystemConsts.TAG_ALBUM);
                mTabsAdapter.addTab(tab, AlbumListViewFragment.class, null);
            }
        }
        if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_ARTIST, true)) {
            ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                    getString(R.string.lb_tab_artist));
            tab.setTag(SystemConsts.TAG_ARTIST);
            mTabsAdapter.addTab(tab, ArtistListViewFragment.class, null);
        }
        if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_MEDIA, true)) {
            ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                    getString(R.string.lb_tab_media));
            tab.setTag(SystemConsts.TAG_MEDIA);
            mTabsAdapter.addTab(tab, MediaListViewFragment.class, null);
        }
        if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_GENRES, true)) {
            ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                    getString(R.string.lb_tab_genres));
            tab.setTag(SystemConsts.TAG_GENRES);
            mTabsAdapter.addTab(tab, GenresListViewFragment.class, null);
        }
        if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_FOLDER, true)) {
            ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                    getString(R.string.lb_tab_folder));
            tab.setTag(SystemConsts.TAG_FOLDER);
            mTabsAdapter.addTab(tab, FileListViewFragment.class, null);
        }
        if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_VIDEO, false)) {
            ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                    getString(R.string.lb_tab_videos));
            tab.setTag(SystemConsts.TAG_VIDEO);
            mTabsAdapter.addTab(tab, VideoListViewFragment.class, null);
        }
        // if (mPref.getBoolean(SystemConsts.VISIBLE_TAG_ORDER, true)) {
        ActionBar.Tab tab = getSupportActionBar().newTab().setText(
                getString(R.string.lb_tab_order));
        tab.setTag(SystemConsts.TAG_ORDER);
        mTabsAdapter.addTab(tab, PlayOrderListViewFragment.class, null);
        // }
        if (mTabsAdapter.getCount() < 2) {
            findViewById(android.R.id.tabs).setVisibility(View.GONE);
        }

        // ContentResolver
        getContentResolver().registerContentObserver(
                MediaConsts.ALBUM_CONTENT_URI, true, mAlbumContentObserver);
        getContentResolver().registerContentObserver(
                MediaConsts.ARTIST_CONTENT_URI, true, mArtistContentObserver);
        getContentResolver().registerContentObserver(
                MediaConsts.MEDIA_CONTENT_URI, true, mMediaContentObserver);

        // 音楽サービスとの接続
        Intent service = new Intent(this, MediaPlayerService.class);
        startService(service);
        bindService(service, mConnection, BIND_AUTO_CREATE);

        Intent intent = getIntent();
        if (intent == null
                || intent.getAction() == null
                || !intent.getAction()
                        .equals(SystemConsts.MAIN_ACITON_SHOWHOMW)) {
            // 初回起動か確認
            boolean isFirst = mPref.getBoolean("is_first", true);
            if (isFirst) {
                Editor editor = mPref.edit();
                editor.putBoolean("is_first", false);
                editor.commit();
                // まず、既にアプリを購入済みか確認する
                mBillingService = new BillingService();
                mDungeonsPurchaseObserver = new DungeonsPurchaseObserver(this,
                        mBillingService, new Handler());
                ResponseHandler.register(mDungeonsPurchaseObserver);
                mBillingService.setContext(this);
                mBillingService.restoreTransactions();
            }
        }

        if (savedInstanceState != null) {
            int index = savedInstanceState.getInt("index");
            if (getSupportActionBar().getTabCount() > index) {
                getSupportActionBar().setSelectedNavigationItem(index);
            }
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        Intent intent = getIntent();
        if (procIntent(intent) == false) {
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        procIntent(intent);
    }

    @Override
    public void onServiceConnected(IMediaPlayerService binder) {
        if (oneIntent != null) {
            revieveNdef(oneIntent);
        }
    }

    private boolean procIntent(Intent intent) {
        if (intent != null) {
            if (NfcAdapter.ACTION_NDEF_DISCOVERED.equals(intent.getAction())) {
                if (revieveNdef(intent)) {
                    getSupportActionBar().setSelectedNavigationItem(
                            getSupportActionBar().getTabCount() - 1);
                    return true;
                }
            } else if (SystemConsts.MAIN_ACITON_SHOWPLAYLIST.equals(intent
                    .getAction())) {
                getSupportActionBar().setSelectedNavigationItem(
                        getSupportActionBar().getTabCount() - 1);
                return true;
            }
        }
        return false;
    }

    public boolean revieveNdef(Intent intent) {
        if (getNFCHelper() != null) {
            IMediaPlayerService binder = getBinder();
            if (binder != null) {
                oneIntent = null;
                try {
                    ArrayList<BeamMessage> list = getNFCHelper()
                            .getNdefMessage(intent);
                    if (list != null) {
                        try {
                            String cntkey = SystemConsts.CONTENTSKEY_PLAYLIST
                                    + System.currentTimeMillis();
                            try {
                                binder.setContentsKey(cntkey);
                                binder.lockUpdateToPlay();
                                binder.clearcut();
                                for (int i = 0; i < list.size(); i++) {
                                    BeamMessage msg = list.get(i);
                                    try {
                                        binder.addMediaD(0, msg.duration,
                                                msg.title, msg.album,
                                                msg.artist, msg.url);
                                    } catch (RemoteException e) {
                                    }
                                }
                            } finally {
                                binder.play();
                            }
                        } catch (RemoteException e) {
                        }
                        return true;
                    }
                } finally {
                }
            }
        }
        oneIntent = intent;
        return false;
    }

    public void reloadAllFragment() {
        for (final TabInfo tabinfo : mTabsAdapter.getTabs()) {
            if (tabinfo.fragment_tag != null) {
                ContentManager cntmgr = (ContentManager) getSupportFragmentManager()
                        .findFragmentByTag(tabinfo.fragment_tag);
                if (cntmgr != null) {
                    cntmgr.reload();
                }
            }
        }
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("index", getSupportActionBar()
                .getSelectedNavigationIndex());
    }

    public Fragment getCurrentFragment() {
        Tab tab = getSupportActionBar().getSelectedTab();
        TabInfo info = (TabInfo) tab.getTag();
        if (info != null) {
            return getSupportFragmentManager().findFragmentByTag(
                    info.fragment_tag);
        }
        return null;
    }

    @Override
    protected void onResume() {
        super.onResume();

        // 音楽サービスとの接続
        /*
         * Intent intent = new Intent(this, MediaPlayerService.class);
         * startService(intent); bindService(intent, mConnection,
         * BIND_AUTO_CREATE);
         */

        // Sessionの復帰
        /*
         * if (!ContentsUtils.isSDCard(mPref)) {
         * getContentResolver().query(MediaConsts.CLEAR_CONTENT_URI, null, null,
         * null, null); }
         */
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        if (mBinder != null) {
            try {
                mBinder.release();
                mBinder.unregisterCallback(getCallBack());
                mBinder = null;
            } catch (RemoteException e) {
            }
        }
        if (mConnection != null) {
            unbindService(mConnection);
        }

        getContentResolver().unregisterContentObserver(mAlbumContentObserver);
        getContentResolver().unregisterContentObserver(mArtistContentObserver);
        getContentResolver().unregisterContentObserver(mMediaContentObserver);

        if (mDungeonsPurchaseObserver != null) {
            ResponseHandler.unregister(mDungeonsPurchaseObserver);
        }
        if (mBillingService != null) {
            mBillingService.unbind();
        }

        ViewCache.clearImage();
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_DOWN) {
            if (event.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                Fragment fragment = getCurrentFragment();
                if (fragment != null) {
                    if (fragment instanceof ContextMenuFragment) {
                        ContextMenuFragment f = (ContextMenuFragment) fragment;
                        if (f.onBackPressed()) {
                            return true;
                        }
                    }
                }

                SharedPreferences pref = PreferenceManager
                        .getDefaultSharedPreferences(this);
                boolean exitcheck = pref.getBoolean("key.exitcheck", false);
                if (exitcheck) {
                    // ダイアログの表示
                    AlertDialog.Builder ad = new AlertDialog.Builder(this);
                    ad.setMessage(getString(R.string.alert_exit_msg));
                    ad.setPositiveButton(getString(R.string.lb_yes),
                            new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog,
                                        int whichButton) {
                                    // OKならActivity終了
                                    finish();
                                }
                            });
                    ad.setNegativeButton(getString(R.string.lb_no),
                            new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog,
                                        int whichButton) {
                                }
                            });
                    ad.create();
                    ad.show();
                    return true;
                } else {
                    return super.dispatchKeyEvent(event);
                }
            }
        }
        return super.dispatchKeyEvent(event);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getSupportMenuInflater();
        inflater.inflate(R.menu.main_menu_items, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        if (menu == null) {
            return super.onPrepareOptionsMenu(menu);
        }
        MenuItem item = menu.findItem(R.id.mnu_equalizer);
        if (item != null) {
            item.setVisible(Build.VERSION.SDK_INT >= 9).setEnabled(
                    Build.VERSION.SDK_INT >= 9);
        }

        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        try {
            switch (item.getItemId()) {
            case R.id.mnu_ds: {

                // データソース選択画面
                selectSource();
            }
                break;
            case R.id.mnu_config: {
                // 設定画面
                Intent intent = new Intent(this, ConfigActivity.class);
                startActivityForResult(intent, SystemConsts.REQUEST_CONFIG);
            }
                break;
            case R.id.mnu_equalizer: {
                // イコライザー画面
                Intent intent = new Intent(this, EqualizerActivity.class);
                startActivity(intent);
            }
                break;
            case R.id.mnu_search: {
                onSearchRequested();
            }
                break;
            case R.id.mnu_playlist: {
                getSupportActionBar().setSelectedNavigationItem(
                        getSupportActionBar().getTabCount() - 1);
            }
                break;
            case R.id.mnu_sort: {
                Fragment f = getCurrentFragment();
                if (f instanceof ContextMenuFragment) {
                    ((ContextMenuFragment) f).selectSort();
                }
                return true;
            }
            case R.id.mnu_share: {
                ControlFragment control = (ControlFragment) getSupportFragmentManager()
                        .findFragmentByTag(SystemConsts.TAG_CONTROL);
                if (control != null) {
                    control.share();
                }
                return true;
            }
            default: {
            }
            }
            return super.onOptionsItemSelected(item);
        } finally {
            // Google Analyticsにトラックイベントを送信
        }
    }

    private void selectSource() {
        final String contenturi = mPref.getString(SystemConsts.PREF_CONTENTURI,
                DeviceContentProvider.MEDIA_AUTHORITY);

        final List<ProviderInfo> dslist = ContentsUtils.getDataSouce(this);
        PackageManager pm = this.getPackageManager();
        final ArrayList<String> items = new ArrayList<String>();
        if (contenturi.equals(DeviceContentProvider.MEDIA_AUTHORITY)) {
            items.add(getString(R.string.lb_sdcard) + " *");
        } else {
            items.add(getString(R.string.lb_sdcard));
        }
        for (ProviderInfo inf : dslist) {
            if (inf.authority.equals(contenturi)) {
                items.add(inf.loadLabel(pm) + " *");
            } else {
                items.add(inf.loadLabel(pm).toString());
            }
        }
        items.add(getString(R.string.lb_conf_add_ds_title));
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getString(R.string.lb_srcdialog));
        builder.setItems(items.toArray(new String[items.size()]),
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int item) {
                        String selecturl = null;
                        String item_name = items.get(item);
                        if (item_name.indexOf(getString(R.string.lb_sdcard)) != -1) {
                            // SDCard
                            selecturl = DeviceContentProvider.MEDIA_AUTHORITY;
                        } else if (item_name
                                .equals(getString(R.string.lb_conf_add_ds_title))) {
                            Uri uri = Uri
                                    .parse("market://search?q=jp.co.kayo.android.localplayer.ds");
                            Intent it = new Intent(Intent.ACTION_VIEW, uri);
                            startActivity(it);
                        } else {
                            // Ampcaheまたはその他
                            selecturl = dslist.get(item - 1).authority;

                            boolean checkSetting = false;
                            if (selecturl.equals(contenturi)) {
                                // 同じURLなら設定画面を呼び出す
                                checkSetting = true;
                            } else {
                                // PINGを飛ばしてだめなら設定画面を呼び出す
                                Cursor cursor = null;
                                try {
                                    Uri uri = Uri.parse("content://"
                                            + selecturl + "/config/ping");
                                    cursor = getContentResolver().query(uri,
                                            null, null, null, null);
                                    if (cursor == null) {
                                        checkSetting = true;
                                    }
                                } finally {
                                    if (cursor != null) {
                                        cursor.close();
                                    }
                                }
                            }
                            if (checkSetting) {
                                Intent it = new Intent();
                                it.setClassName(
                                        dslist.get(item - 1).packageName,
                                        dslist.get(item - 1).packageName
                                                + ".MainActivity");
                                startActivity(it);
                            }
                        }

                        if (selecturl != null && !selecturl.equals(contenturi)) {
                            Editor editor = mPref.edit();
                            editor.putString(SystemConsts.PREF_CONTENTURI,
                                    selecturl);
                            editor.commit();

                            try {
                                IMediaPlayerService binder = getBinder();
                                if (binder != null) {
                                    binder.setContentsKey(null);
                                    binder.clear();
                                }
                            } catch (RemoteException e) {
                            }

                            reloadAllFragment();
                        }
                    }
                });
        AlertDialog dlg = builder.create();
        dlg.show();
    }

    @Override
    IMediaPlayerServiceCallback getCallBack() {
        return mCallback;
    }

    @Override
    ContentManager getNeedReloadView() {
        return mPlayList;
    }

    @Override
    ViewCache getViewCache() {
        return mViewCache;
    }

    @Override
    Handler getHandler() {
        return mHandler;
    }
}
