package jp.co.kayo.android.localplayer;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 *      This program is free software; you can redistribute it and/or modify it under
 *      the terms of the GNU General Public License as published by the Free Software
 *      Foundation; either version 2 of the License, or (at your option) any later
 *      version.
 *      
 *      This program is distributed in the hope that it will be useful, but WITHOUT
 *      ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *      FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 *      details.
 *      
 *      You should have received a copy of the GNU General Public License along with
 *      this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *      Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.util.Hashtable;

import jp.co.kayo.android.localplayer.appwidget.AppWidgetHelper;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.service.IMediaPlayerService;
import jp.co.kayo.android.localplayer.service.IMediaPlayerServiceCallback;
import jp.co.kayo.android.localplayer.service.MediaPlayerService;
import jp.co.kayo.android.localplayer.ui.ControlFragment;
import jp.co.kayo.android.localplayer.ui.dialog.DeletePlaylistDialog;
import jp.co.kayo.android.localplayer.ui.dialog.OpenPlaylistDialog;
import jp.co.kayo.android.localplayer.ui.dialog.SavePlaylistDialog;
import jp.co.kayo.android.localplayer.ui.plugin.AlbumArtFragment;
import jp.co.kayo.android.localplayer.ui.plugin.SongsFragment;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.StrictHelper;
import jp.co.kayo.android.localplayer.util.ThemeHelper;
import jp.co.kayo.android.localplayer.util.ViewCache;
import android.support.v4.app.FragmentManager;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.ContentObserver;
import android.database.Cursor;
import android.media.AudioManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.os.ResultReceiver;
import android.preference.PreferenceManager;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager.LayoutParams;
import android.widget.AdapterView;
import android.widget.LinearLayout;
import android.widget.AdapterView.OnItemSelectedListener;

public class PlayActivity extends BaseActivity implements
        OnItemSelectedListener {
    SharedPreferences pref;
    ViewCache mViewCache;
    String mAlbumKey;
    boolean mLock = false;
    AlbumArtFragment mAlbumArtFrgment;
    SongsFragment mSongsFragment;
    Handler mHandler = new Handler();
    private ResultReceiver mReciever = null;

    ContentObserver mMediaContentObserver = new ContentObserver(new Handler()) {
        @Override
        public void onChange(boolean selfChange) {
            super.onChange(selfChange);
            // データベースに変更があったときに呼び出したい処理を書く
            if(getBinder()!=null){
                try {
                    getBinder().setContentsKey("changed");
                } catch (RemoteException e) {
                }
            }
            if (mSongsFragment != null) {
                mSongsFragment.reload();
            }
        }
    };

    ContentObserver mAlbumContentObserver = new ContentObserver(new Handler()) {
        @Override
        public void onChange(boolean selfChange) {
            super.onChange(selfChange);
            if (mAlbumKey != null) {
                Hashtable<String, String> tbl = ContentsUtils.getAlbum(
                        PlayActivity.this, new String[] { AudioAlbum.ALBUM,
                                AudioAlbum.ALBUM_KEY, AudioAlbum.ARTIST },
                        mAlbumKey);
                if (tbl != null && mAlbumArtFrgment != null) {
                    mAlbumArtFrgment.setAlbumImage();
                }
            }
        }
    };

    public SongsFragment getSongFragment() {
        return mSongsFragment;
    }

    public String getAlbumKey() {
        return mAlbumKey;
    }

    IMediaPlayerServiceCallback.Stub mCallback = new IMediaPlayerServiceCallback.Stub() {
        ControlFragment control;

        ControlFragment getControll() {
            if (control == null) {
                FragmentManager m = getSupportFragmentManager();
                control = (ControlFragment) m
                        .findFragmentByTag(SystemConsts.TAG_CONTROL);
            }
            return control;
        }

        @Override
        public void updateView(final boolean updatelist) throws RemoteException {
            Logger.d("updateView=" + updatelist);
            // もし、コントロール部分が表示されているならリストを更新してあげて
            if (mHandler != null) {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        // AlbumList,ArtistList,Song等
                        ControlFragment control = getControll();
                        if (mBinder != null) {
                            try {
                                long media_id = mBinder.getMediaId();
                                mViewCache.setPosition(mBinder.getPosition(),
                                        media_id, mBinder.getPrefetchId());
                            } catch (RemoteException e) {
                            }
                        }

                        if (control != null) {
                            control.updateView();
                        }

                        if (updatelist) {
                            if (mAlbumArtFrgment != null)
                                mAlbumArtFrgment.reload();
                            if (mSongsFragment != null)
                                mSongsFragment.reload();
                        } else {
                            if (mAlbumArtFrgment != null)
                                mAlbumArtFrgment.changedMedia();
                            if (mSongsFragment != null)
                                mSongsFragment.changedMedia();
                        }
                    }
                });
            }
        }

        @Override
        public void updateList() throws RemoteException {
            // もし、再生中のフラグメントが表示されているならリストを更新してあげて
        }

        @Override
        public void onBufferingUpdate(int percent) throws RemoteException {
            // バッファリング中のプログレスバーですよぉ
            ControlFragment f = getControll();
            if (f != null) {
                f.onBufferingUpdate(percent);
            }
        }

        @Override
        public void startProgress(long max) throws RemoteException {
            long media_id = mBinder.getMediaId();
            int pos = mBinder.getPosition();
            long pref_id = mBinder.getPrefetchId();
            mViewCache.setPosition(pos, media_id, pref_id);
            if (mSongsFragment != null) {
                mSongsFragment.startProgress(max);
            }
        }

        @Override
        public void stopProgress() throws RemoteException {
            if (mSongsFragment != null) {
                mSongsFragment.stopProgress();
            }
        }

        @Override
        public void progress(long pos, long max) throws RemoteException {
            if (mSongsFragment != null) {
                mSongsFragment.progress(pos, max);
            }
        }
    };

    @Override
    public void setActionBarSubTitle(int stat) {
        if ((stat & AppWidgetHelper.FLG_PLAY) > 0) {
            IMediaPlayerService binder = getBinder();
            long media_id;
            try {
                media_id = binder.getMediaId();
                if (media_id > 0) {
                    Hashtable<String, String> tbl1 = ContentsUtils.getMedia(
                            this, new String[] { AudioMedia.ALBUM,
                                    AudioMedia.ALBUM_KEY, AudioMedia.ARTIST,
                                    AudioMedia.TITLE }, media_id);
                    String title = tbl1.get(AudioMedia.TITLE);
                    String artist = tbl1.get(AudioMedia.ARTIST);
                    getSupportActionBar().setSubtitle(title + " - " + artist);
                } else {
                    int pos = binder.getPosition();
                    String[] values = binder.getMediaD(pos);
                    if (values != null) {
                        String title = values[0];
                        String artist = values[2];
                        getSupportActionBar().setSubtitle(
                                title + " - " + artist);
                    }
                }
            } catch (RemoteException e) {
            }
        } else {
            getSupportActionBar().setSubtitle(getString(R.string.hello));
        }
    }
    
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent arg2) {
        super.onActivityResult(requestCode, resultCode, arg2);
        if(requestCode == SystemConsts.REQUEST_TAGEDIT){
            if (mSongsFragment != null) {
                mSongsFragment.reload();
            }
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        new ThemeHelper().selectTheme(this);
        StrictHelper.registStrictMode();
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

        super.onCreate(savedInstanceState);

        hideProgressBar();

        pref = PreferenceManager.getDefaultSharedPreferences(this);
        setContentView(R.layout.main_play);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        hideProgressBar();
        getSupportActionBar().setSubtitle(getString(R.string.hello));

        // Audio Volume
        setVolumeControlStream(AudioManager.STREAM_MUSIC);

        getWindow().setSoftInputMode(
                LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);

        mReciever = new ResultReceiver(mHandler) {
            @Override
            protected void onReceiveResult(int resultCode, Bundle resultData) {
                super.onReceiveResult(resultCode, resultData);
                if (resultCode == 0 && resultData != null) {
                    String colname = resultData.getString("colname");
                    String key = resultData.getString("key");
                    getContentResolver().delete(
                            MediaConsts.PLAYLIST_CONTENT_URI,
                            colname + " = ? ", new String[] { key });
                }
            }
        };

        // ContentResolver
        getContentResolver().registerContentObserver(
                MediaConsts.MEDIA_CONTENT_URI, true, mMediaContentObserver);
        getContentResolver().registerContentObserver(
                MediaConsts.ALBUM_CONTENT_URI, true, mAlbumContentObserver);

        mViewCache = (ViewCache) getSupportFragmentManager().findFragmentByTag(
                SystemConsts.TAG_CACHE);

        mAlbumArtFrgment = (AlbumArtFragment) getSupportFragmentManager()
                .findFragmentByTag("ALBUMART");
        mSongsFragment = (SongsFragment) getSupportFragmentManager()
                .findFragmentByTag("SONGLIST");

        // load list
        if (savedInstanceState != null) {
            mAlbumKey = savedInstanceState
                    .getString(SystemConsts.KEY_PLAY_ALBUMKEY);
            mLock = savedInstanceState.getBoolean(SystemConsts.KEY_PLAY_LOCK);
        } else {
            Intent intent = getIntent();
            if (intent != null) {
                Bundle params = intent.getExtras();
                if (params != null) {
                    mAlbumKey = params
                            .getString(SystemConsts.KEY_PLAY_ALBUMKEY);
                    mLock = params.getBoolean(SystemConsts.KEY_PLAY_LOCK);
                }
            }
        }
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putString(SystemConsts.KEY_PLAY_ALBUMKEY, mAlbumKey);
        outState.putBoolean(SystemConsts.KEY_PLAY_LOCK, mLock);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        getContentResolver().unregisterContentObserver(mMediaContentObserver);
        getContentResolver().unregisterContentObserver(mAlbumContentObserver);

        ViewCache.clearImage();
    }

    @Override
    protected void onPause() {
        super.onPause();
        Logger.d("PlayActivity.Pause");
        if (mBinder != null) {
            try {
                mBinder.unregisterCallback(mCallback);
                mBinder = null;
            } catch (RemoteException e) {
            }
            unbindService(mConnection);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        Logger.d("PlayActivity.Resume");
        // 音楽サービスとの接続
        if (mBinder == null) {
            Intent service = new Intent(this, MediaPlayerService.class);
            bindService(service, mConnection, BIND_AUTO_CREATE);
            // startService(service);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getSupportMenuInflater();
        inflater.inflate(R.menu.play_menu_items, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        if (menu == null) {
            return super.onPrepareOptionsMenu(menu);
        }
        MenuItem item = menu.findItem(R.id.mnu_equalizer);
        if (item != null) {
            item.setVisible(Build.VERSION.SDK_INT >= 9).setEnabled(
                    Build.VERSION.SDK_INT >= 9);
        }

        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        try {
            switch (item.getItemId()) {
            case android.R.id.home: {
                finish();
            }
                break;
            case R.id.mnu_playlist: {
                Intent intent = new Intent(this, MainActivity.class);
                intent.setAction(SystemConsts.MAIN_ACITON_SHOWPLAYLIST);
                startActivity(intent);
            }
                break;
            /*
             * case R.id.mnu_play: { ContentsUtils.playAlbum(getBinder(), this,
             * mAlbumKey);
             * 
             * Toast.makeText(this, getString(R.string.txt_action_addalbum),
             * Toast.LENGTH_SHORT) .show(); } break;
             */
            case R.id.mnu_search: {
                onSearchRequested();
                return true;
            }
            case R.id.mnu_albumart: {
                Intent i = new Intent(this, AlbumartActivity.class);
                i.putExtra("album_key", mAlbumKey);
                startActivity(i);
            }
                break;
            case R.id.mnu_config: {
                // 設定画面
                Intent intent = new Intent(this, ConfigActivity.class);
                startActivityForResult(intent, SystemConsts.REQUEST_CONFIG);
            }
                break;
            case R.id.mnu_equalizer: {
                // イコライザー画面
                Intent intent = new Intent(this, EqualizerActivity.class);
                startActivity(intent);
            }
                break;
            case R.id.mnu_share: {
                ControlFragment control = (ControlFragment) getSupportFragmentManager()
                        .findFragmentByTag(SystemConsts.TAG_CONTROL);
                if (control != null) {
                    control.share();
                }
                return true;
            }
            default: {
            }
            }
            return super.onOptionsItemSelected(item);
        } finally {
        }
    }

    @Override
    public IMediaPlayerService getBinder() {
        return mBinder;
    }

    @Override
    public void onItemSelected(AdapterView<?> arg0, View arg1, int position,
            long arg3) {
        // mPager.setCurrentItem(position);
    }

    @Override
    public void onNothingSelected(AdapterView<?> arg0) {
    }

    public boolean playAlbum() {
        Cursor cur = null;
        try {
            if (mAlbumKey != null) {
                cur = getContentResolver().query(
                        MediaConsts.MEDIA_CONTENT_URI,
                        new String[] { AudioMedia._ID, AudioMedia.ALBUM_KEY,
                                AudioMedia.ALBUM, AudioMedia.ARTIST,
                                AudioMedia.TITLE, AudioMedia.DURATION,
                                AudioMedia.DATA, AudioMedia.TRACK },
                        AudioMedia.ALBUM_KEY + " = ?",
                        new String[] { mAlbumKey }, AudioMedia.TRACK + "," + AudioMedia.TITLE);
                if (cur != null && cur.moveToFirst()) {
                    String key = SystemConsts.CONTENTSKEY_ALBUM + mAlbumKey;
                    IMediaPlayerService binder = getBinder();
                    try {
                        if (binder != null) {
                            if (!key.equals(binder.getContentsKey())) {
                                if (!ContentsUtils.playMediaReplace(key,
                                        getBinder(), cur,
                                        MediaConsts.AudioMedia._ID,
                                        MediaConsts.AudioMedia.DURATION,
                                        MediaConsts.AudioMedia.DATA, 0)) {
                                    binder.play();
                                }
                            } else {
                                int stat = binder.stat();
                                if ((stat & AppWidgetHelper.FLG_PLAY) == 0) {
                                    binder.play();
                                }
                            }
                        }
                    } catch (RemoteException e) {
                    }
                    return true;
                }
            }
        } finally {
            if (cur != null) {
                cur.close();
            }
        }

        return false;
    }

    /**
     * PlayActivityで広告とコントロールパネルを除いた分の高さを返却する
     * 
     * @return:高さ(px単位)
     */
    public int getMainPlayHeight() {
        LinearLayout layout = (LinearLayout) findViewById(R.id.mainPlayWrapperFrame);
        View controller = findViewById(R.id.controlLinearLayout01);
        return layout.getHeight() - controller.getHeight();
    }

    public int getThemeBackground() {
        TypedValue typedValue = new TypedValue();
        getTheme().resolveAttribute(R.attr.albumBackground, typedValue, true);
        return typedValue.resourceId;
    }

    @Override
    IMediaPlayerServiceCallback getCallBack() {
        return mCallback;
    }

    @Override
    ContentManager getNeedReloadView() {
        return mSongsFragment;
    }

    @Override
    ViewCache getViewCache() {
        return mViewCache;
    }

    @Override
    Handler getHandler() {
        // TODO Auto-generated method stub
        return mHandler;
    }
}
