package jp.co.kayo.android.localplayer;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 *      This program is free software; you can redistribute it and/or modify it under
 *      the terms of the GNU General Public License as published by the Free Software
 *      Foundation; either version 2 of the License, or (at your option) any later
 *      version.
 *      
 *      This program is distributed in the hope that it will be useful, but WITHOUT
 *      ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *      FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 *      details.
 *      
 *      You should have received a copy of the GNU General Public License along with
 *      this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *      Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioArtist;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.service.IMediaPlayerServiceCallback;
import jp.co.kayo.android.localplayer.service.MediaPlayerService;
import jp.co.kayo.android.localplayer.util.SdCardAccessHelper;
import jp.co.kayo.android.localplayer.util.ThemeHelper;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.SearchItem;
import jp.co.kayo.android.localplayer.util.bean.SearchItem.ITEM_TYPE;

import android.app.SearchManager;
import android.support.v4.app.LoaderManager.LoaderCallbacks;
import android.support.v4.content.AsyncTaskLoader;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v4.content.Loader;
import android.database.Cursor;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class SearchActivity extends BaseActivity implements
        LoaderCallbacks<List<SearchItem>>, OnItemLongClickListener,
        OnItemClickListener {
    private ListView mListView;
    private SearchItemAdapter mAdapter;
    private String mSearchText;
    private SharedPreferences mPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        new ThemeHelper().selectTheme(this);
        super.onCreate(savedInstanceState);

        setContentView(R.layout.search_list_view);
        mPref = PreferenceManager.getDefaultSharedPreferences(this);

        mAdapter = new SearchItemAdapter(this, new ArrayList<SearchItem>());
        mListView = (ListView) findViewById(android.R.id.list);
        mListView.setAdapter(mAdapter);

        if (getIntent() != null
                && Intent.ACTION_SEARCH.equals(getIntent().getAction())) {
            mSearchText = getIntent().getStringExtra(SearchManager.QUERY);

            Bundle args = new Bundle();
            args.putString("filterText", mSearchText);

            getSupportLoaderManager().initLoader(R.layout.search_list_view,
                    args, this);
        }

        mListView.setOnItemClickListener(this);
        mListView.setOnItemLongClickListener(this);

        Intent service = new Intent(this, MediaPlayerService.class);
        startService(service);
        bindService(service, mConnection, BIND_AUTO_CREATE);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mBinder != null) {
            unbindService(mConnection);
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        if (Intent.ACTION_SEARCH.equals(intent.getAction())) {
            mSearchText = intent.getStringExtra(SearchManager.QUERY);

            Bundle args = new Bundle();
            args.putString("filterText", mSearchText);
            getSupportLoaderManager().restartLoader(R.layout.search_list_view,
                    args, this);
        }
    }

    @Override
    public void onItemClick(AdapterView<?> l, View v, int position, long id) {
        SearchItem item = mAdapter.getItem(position);
        if (item.type == ITEM_TYPE.ALBUM) {

            if (mPref.getBoolean("key.autoPlay", false)) {
                ContentsUtils.playAlbum(mBinder, this, item.albumKey);
                finish();
            } else {
                Intent intent = new Intent(this, PlayActivity.class);
                intent.putExtra(SystemConsts.KEY_PLAY_LOCK, true);
                intent.putExtra(SystemConsts.KEY_PLAY_ALBUMKEY, item.albumKey);
                startActivity(intent);
            }

        } else if (item.type == ITEM_TYPE.ARTIST) {

            if (mPref.getBoolean("key.autoPlay", false)) {
                ContentsUtils.playArtist(mBinder, this, item.artistKey);
                finish();
            } else {
                Intent intent = new Intent(this, SubListActivity.class);
                intent.putExtra(SystemConsts.KEY_LIST_SUBTYPE,
                        SystemConsts.PRM_ARTISTLIST);
                intent.putExtra(SystemConsts.KEY_LIST_TITLE, item.artist);
                intent.putExtra(SystemConsts.KEY_LIST_MAX,
                        item.number_of_tracks);
                intent.putExtra(SystemConsts.KEY_LIST_KEY, item.artistKey);
                startActivity(intent);
            }

        } else if (item.type == ITEM_TYPE.SONG) {

            ContentsUtils.playMedia(mBinder, item.id, item.duration, item.data);
            finish();
        }
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> arg0, View arg1, int arg2,
            long arg3) {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public Loader<List<SearchItem>> onCreateLoader(int id, Bundle args) {
        String filterText = args.getString("filterText");
        return new SearchItemLoader(this, filterText);
    }

    @Override
    public void onLoadFinished(Loader<List<SearchItem>> loader,
            List<SearchItem> data) {
        mAdapter.setData(data);
    }

    @Override
    public void onLoaderReset(Loader<List<SearchItem>> data) {
        mAdapter.setData(null);
    }

    private static class SearchItemAdapter extends ArrayAdapter<SearchItem> {
        LayoutInflater inflater;
        boolean isSdCard;

        public SearchItemAdapter(Context context, List<SearchItem> list) {
            super(context, -1, list);
            inflater = (LayoutInflater) context
                    .getSystemService(LAYOUT_INFLATER_SERVICE);
            isSdCard = ContentsUtils.isSDCard(context);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            ViewHolder holder = null;
            ;
            if (convertView == null) {
                holder = new ViewHolder();
                convertView = inflater.inflate(R.layout.search_list_item, null,
                        false);
                convertView.setTag(holder);

                holder.layoutGroup = convertView.findViewById(R.id.layoutGroup);
                holder.layoutItem = convertView.findViewById(R.id.layoutItem);
                holder.text0 = (TextView) convertView.findViewById(R.id.text0);
                holder.text1 = (TextView) convertView.findViewById(R.id.text1);
                holder.text2 = (TextView) convertView.findViewById(R.id.text2);
                holder.text3 = (TextView) convertView.findViewById(R.id.text3);
                holder.imgCache = (ImageView) convertView
                        .findViewById(R.id.imgCache);
            } else {
                holder = (ViewHolder) convertView.getTag();
            }

            SearchItem item = getItem(position);

            if (item.type == ITEM_TYPE.GROUP) {
                holder.layoutGroup.setVisibility(View.VISIBLE);
                holder.layoutItem.setVisibility(View.GONE);
                holder.text0.setText(item.title);
            } else {
                holder.layoutGroup.setVisibility(View.GONE);
                holder.layoutItem.setVisibility(View.VISIBLE);
                if (item.type == ITEM_TYPE.ALBUM) {
                    holder.text2.setVisibility(View.VISIBLE);
                    holder.text3.setVisibility(View.GONE);
                    holder.imgCache.setVisibility(View.GONE);
                    holder.text1.setText(item.album);
                    holder.text2.setText(item.artist);
                } else if (item.type == ITEM_TYPE.ARTIST) {
                    holder.text2.setVisibility(View.GONE);
                    holder.text3.setVisibility(View.GONE);
                    holder.imgCache.setVisibility(View.GONE);
                    holder.text1.setText(item.artist);
                } else if (item.type == ITEM_TYPE.SONG) {
                    holder.text2.setVisibility(View.VISIBLE);
                    holder.text3.setVisibility(View.VISIBLE);
                    holder.imgCache.setVisibility(View.VISIBLE);
                    holder.text1.setText(item.title);
                    holder.text2.setText(item.album);
                    holder.text3.setText(item.artist);
                    if (isSdCard != true
                            && SdCardAccessHelper.existCachFile(item.data)) {
                        holder.imgCache.setVisibility(View.VISIBLE);
                    } else {
                        holder.imgCache.setVisibility(View.GONE);
                    }
                }
            }

            return convertView;
        }

        @Override
        public boolean isEnabled(int position) {
            SearchItem item = getItem(position);
            return item.type != ITEM_TYPE.GROUP;
        }

        public void setData(List<SearchItem> data) {
            clear();
            if (data != null) {
                if (Build.VERSION.SDK_INT >= 11) {
                    addAll(data);
                } else {
                    for (SearchItem i : data) {
                        add(i);
                    }
                }
            }
        }

        class ViewHolder {
            View layoutGroup;
            View layoutItem;
            TextView text0;
            TextView text1;
            TextView text2;
            TextView text3;
            ImageView imgCache;
        }
    }

    private static class SearchItemLoader extends
            AsyncTaskLoader<List<SearchItem>> {
        private String mFilterText;
        private ArrayList<SearchItem> mList;

        public SearchItemLoader(Context context, String filter) {
            super(context);
            mFilterText = filter;
        }

        @Override
        public List<SearchItem> loadInBackground() {
            return makeList();
        }

        private List<SearchItem> makeList() {
            if (mList == null) {
                mList = new ArrayList<SearchItem>();
            } else {
                mList.clear();
            }

            Map<String, SearchItem> albums = new HashMap<String, SearchItem>();
            Map<String, SearchItem> artists = new HashMap<String, SearchItem>();
            ArrayList<SearchItem> songs = new ArrayList<SearchItem>();
            makeListSongs(songs);
            makeListAlbums(albums);
            makeListArtist(artists);
            if (albums.size() > 0) {
                SearchItem grp = new SearchItem();
                grp.type = ITEM_TYPE.GROUP;
                grp.title = getContext().getString(R.string.lb_tab_albums)
                        .concat("(").concat(Integer.toString(albums.size()))
                        .concat(")");
                mList.add(grp);
                mList.addAll(albums.values());
            }
            if (artists.size() > 0) {
                SearchItem grp = new SearchItem();
                grp.type = ITEM_TYPE.GROUP;
                grp.title = getContext().getString(R.string.lb_tab_artist)
                        .concat("(").concat(Integer.toString(artists.size()))
                        .concat(")");
                mList.add(grp);
                mList.addAll(artists.values());
            }
            if (songs.size() > 0) {
                SearchItem grp = new SearchItem();
                grp.type = ITEM_TYPE.GROUP;
                grp.title = getContext().getString(R.string.lb_tab_media)
                        .concat("(").concat(Integer.toString(songs.size()))
                        .concat(")");
                mList.add(grp);
                mList.addAll(songs);
            }

            return mList;
        }

        void makeListSongs(ArrayList<SearchItem> songs) {
            StringBuilder where = new StringBuilder();
            ArrayList<String> whereArgs = new ArrayList<String>();
            if (mFilterText != null && mFilterText.length() > 0) {
                where.append(MediaConsts.AudioMedia.TITLE).append(" like ");
                where.append("'%' || ? || '%'");
                whereArgs.add(mFilterText);
            }

            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.MEDIA_CONTENT_URI, null, where.toString(),
                        whereArgs.toArray(new String[whereArgs.size()]),
                        MediaConsts.AudioMedia.TITLE);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        String albumKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM_KEY));
                        String artistKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST_KEY));
                        String album = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM));
                        String artist = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST));
                        String title = cur.getString(cur
                                .getColumnIndex(AudioMedia.TITLE));

                        SearchItem item = new SearchItem();
                        item.id = cur.getLong(cur
                                .getColumnIndex(AudioMedia._ID));
                        item.type = ITEM_TYPE.SONG;
                        item.albumKey = albumKey;
                        item.artistKey = artistKey;
                        item.album = album;
                        item.artist = artist;
                        item.title = title;
                        item.duration = cur.getLong(cur
                                .getColumnIndex(AudioMedia.DURATION));
                        item.data = cur.getString(cur
                                .getColumnIndex(AudioMedia.DATA));
                        songs.add(item);
                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }

        void makeListAlbums(Map<String, SearchItem> albums) {
            StringBuilder where = new StringBuilder();
            ArrayList<String> whereArgs = new ArrayList<String>();
            if (mFilterText != null && mFilterText.length() > 0) {
                where.append(MediaConsts.AudioAlbum.ALBUM).append(" like ");
                where.append("'%' || ? || '%'");
                whereArgs.add(mFilterText);
            }

            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ALBUM_CONTENT_URI,
                        new String[] { AudioAlbum.ALBUM_KEY, AudioAlbum.ALBUM,
                                AudioAlbum.ARTIST }, where.toString(),
                        whereArgs.toArray(new String[whereArgs.size()]),
                        MediaConsts.AudioAlbum.ALBUM);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        String albumKey = cur.getString(cur
                                .getColumnIndex(AudioAlbum.ALBUM_KEY));
                        String album = cur.getString(cur
                                .getColumnIndex(AudioAlbum.ALBUM));
                        String artist = cur.getString(cur
                                .getColumnIndex(AudioAlbum.ARTIST));

                        SearchItem item = new SearchItem();
                        item.type = ITEM_TYPE.ALBUM;
                        item.albumKey = albumKey;
                        item.album = album;
                        item.artist = artist;
                        albums.put(item.albumKey, item);
                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }

        void makeListArtist(Map<String, SearchItem> artists) {
            StringBuilder where = new StringBuilder();
            ArrayList<String> whereArgs = new ArrayList<String>();
            if (mFilterText != null && mFilterText.length() > 0) {
                where.append(MediaConsts.AudioMedia.ARTIST).append(" like ");
                where.append("'%' || ? || '%'");
                whereArgs.add(mFilterText);
            }

            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ARTIST_CONTENT_URI,
                        new String[] { AudioArtist.ARTIST_KEY,
                                AudioArtist.ARTIST,
                                AudioArtist.NUMBER_OF_TRACKS },
                        where.toString(),
                        whereArgs.toArray(new String[whereArgs.size()]),
                        MediaConsts.AudioArtist.ARTIST);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        String artistKey = cur.getString(cur
                                .getColumnIndex(AudioArtist.ARTIST_KEY));
                        String artist = cur.getString(cur
                                .getColumnIndex(AudioArtist.ARTIST));
                        int max = cur.getInt(cur
                                .getColumnIndex(AudioArtist.NUMBER_OF_TRACKS));

                        SearchItem item = new SearchItem();
                        item.type = ITEM_TYPE.ARTIST;
                        item.artistKey = artistKey;
                        item.artist = artist;
                        item.number_of_tracks = max;
                        artists.put(item.artistKey, item);

                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }

        @Override
        protected void onStartLoading() {
            super.onStartLoading();
            if (mList == null) {
                forceLoad();
            } else {
                deliverResult(mList);
            }
        }

    }

    @Override
    IMediaPlayerServiceCallback getCallBack() {
        return null;
    }

    @Override
    ContentManager getNeedReloadView() {
        return null;
    }

    @Override
    ViewCache getViewCache() {
        return null;
    }

    @Override
    Handler getHandler() {
        return new Handler();
    }

}
