package jp.co.kayo.android.localplayer;

/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.util.ArrayList;

import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.ui.ControlFragment;

import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.os.Bundle;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.View;
import android.view.ViewGroup;

public class TabsAdapter extends FragmentPagerAdapter implements
        ViewPager.OnPageChangeListener, ActionBar.TabListener {
    private final MainActivity mContext;
    private final ActionBar mActionBar;
    private final ViewPager mViewPager;
    private final ArrayList<TabInfo> mTabs = new ArrayList<TabInfo>();

    static final class TabInfo {
        private final String tag;
        private final Class<?> clss;
        private final Bundle args;
        String fragment_tag;

        TabInfo(String _tag, Class<?> _class, Bundle _args) {
            tag = _tag;
            clss = _class;
            args = _args;
        }
    }

    public TabsAdapter(MainActivity activity, ActionBar actionBar,
            ViewPager pager) {
        super(activity.getSupportFragmentManager());
        mContext = activity;
        mActionBar = actionBar;
        mViewPager = pager;
        mViewPager.setAdapter(this);
        mViewPager.setOnPageChangeListener(this);
    }

    public void addTab(ActionBar.Tab tab, Class<?> clss, Bundle args) {
        String tag = tab.getTag().toString();
        TabInfo info = new TabInfo(tag, clss, args);
        mTabs.add(info);
        tab.setTag(info);
        mActionBar.addTab(tab.setTabListener(this));
        notifyDataSetChanged();
    }

    public final ArrayList<TabInfo> getTabs() {
        return mTabs;
    }

    public String findFragmentTag(String tag) {
        for (TabInfo info : mTabs) {
            if (info.tag.equals(tag)) {
                return info.fragment_tag;
            }
        }
        return null;
    }

    @Override
    public int getCount() {
        return mTabs.size();
    }

    @Override
    public Fragment getItem(int position) {
        TabInfo info = mTabs.get(position);
        Fragment fragment = Fragment.instantiate(mContext, info.clss.getName(),
                info.args);
        return fragment;
    }

    @Override
    public Object instantiateItem(View container, int position) {
        Fragment fragment = (Fragment) super.instantiateItem(container,
                position);
        if (position < mTabs.size()) {
            TabInfo info = mTabs.get(position);
            info.fragment_tag = fragment.getTag();
        }
        return fragment;
    }

    public Fragment getFragment(int position) {
        Tab tab = mActionBar.getTabAt(position);
        TabInfo info = (TabInfo) tab.getTag();
        if (info != null) {
            return mContext.getSupportFragmentManager().findFragmentByTag(
                    info.fragment_tag);
        }
        return null;
    }

    @Override
    public void onPageScrolled(int position, float positionOffset,
            int positionOffsetPixels) {
    }

    @Override
    public void onPageSelected(int position) {
        mActionBar.setSelectedNavigationItem(position);
        ContentManager cmr = (ContentManager) getFragment(position);
        if (cmr != null) {
            cmr.changedMedia();
            FragmentManager m = mContext.getSupportFragmentManager();
            ControlFragment control = (ControlFragment) m
                    .findFragmentByTag(SystemConsts.TAG_CONTROL);
            if (control != null && control.getView() != null) {
                control.showControl(false);
            }
        }
    }

    @Override
    public void onPageScrollStateChanged(int state) {
    }

    @Override
    public void onTabReselected(Tab tab, android.app.FragmentTransaction ft) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void onTabSelected(Tab tab, android.app.FragmentTransaction ft) {
        mViewPager.setCurrentItem(tab.getPosition());
        
    }

    @Override
    public void onTabUnselected(Tab tab, android.app.FragmentTransaction ft) {
        // TODO Auto-generated method stub
        
    }
}
