package jp.co.kayo.android.localplayer;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import twitter4j.StatusUpdate;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.auth.AccessToken;
import twitter4j.auth.OAuthAuthorization;
import twitter4j.auth.RequestToken;
import twitter4j.conf.Configuration;
import twitter4j.conf.ConfigurationBuilder;

import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.StrictHelper;
import jp.co.kayo.android.localplayer.util.ThemeHelper;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.database.Cursor;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.MediaStore;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.Toast;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ToggleButton;

public class TweetActivity extends Activity implements OnClickListener,
        TextWatcher, OnCheckedChangeListener {
    String album = null;
    String artist = null;
    String title = null;
    String album_key = null;

    SharedPreferences pref;
    TextView textview;
    EditText editText;
    ToggleButton btnOAuth;
    CheckBox checkAutoTweet;
    // Button btnart;
    // Button btnut;
    Button btntw;
    Button btnYoutube;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        new ThemeHelper().selectTheme(this);
        StrictHelper.registStrictMode();
        super.onCreate(savedInstanceState);
        setContentView(jp.co.kayo.android.localplayer.R.layout.tweet_main);

        editText = (EditText) findViewById(R.id.txtMsg);
        pref = PreferenceManager.getDefaultSharedPreferences(this);

        textview = (TextView) findViewById(R.id.lbCount);
        // btnart = (Button)findViewById(R.id.btnArtwork);
        // btnut = (Button)findViewById(R.id.btnYoutube);
        btntw = (Button) findViewById(R.id.btnTweet);
        btnYoutube = (Button) findViewById(R.id.btnYoutube);

        // btnart.setOnClickListener(this);
        // btnut.setOnClickListener(this);
        btntw.setOnClickListener(this);

        editText.addTextChangedListener(this);

        btnOAuth = (ToggleButton) findViewById(R.id.btnOAuth);
        checkAutoTweet = (CheckBox) findViewById(R.id.checkAutoTweet);

        refreshOAuth();
        btnOAuth.setOnClickListener(this);
        btnYoutube.setOnClickListener(this);

        boolean isautotweet = pref.getBoolean("key.autoTweet", false);
        checkAutoTweet.setChecked(isautotweet);
        checkAutoTweet.setOnCheckedChangeListener(this);

        // IntentからMediaIDを取得する
        Intent it = getIntent();
        if (it != null) {
            album = it.getStringExtra("album");
            artist = it.getStringExtra("artist");
            title = it.getStringExtra("title");
            album_key = it.getStringExtra("key");
        }

        if (album_key != null && album_key.length() > 0) {
            Cursor cur = null;
            try {
                // データベースから情報を取得してタイトルに設定する
                cur = getContentResolver().query(
                        MediaConsts.ALBUM_CONTENT_URI,
                        new String[] { MediaStore.Audio.Albums._ID,
                                MediaStore.Audio.Albums.ALBUM,
                                MediaStore.Audio.Albums.ARTIST },
                        MediaStore.Audio.Albums.ALBUM_KEY + " = ?",
                        new String[] { album_key }, null);

                if (cur.moveToFirst()) {
                    album = cur.getString(cur
                            .getColumnIndex(MediaStore.Audio.Media.ALBUM));
                    artist = cur.getString(cur
                            .getColumnIndex(MediaStore.Audio.Media.ARTIST));
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        } else if (album != null && artist != null) {
            Cursor cur = null;
            try {
                cur = getContentResolver().query(
                        MediaConsts.ALBUM_CONTENT_URI,
                        new String[] { MediaStore.Audio.Albums._ID,
                                MediaStore.Audio.Albums.ALBUM,
                                MediaStore.Audio.Albums.ALBUM_KEY,
                                MediaStore.Audio.Albums.ARTIST },
                        MediaStore.Audio.Albums.ALBUM + " = ? AND "
                                + MediaStore.Audio.Albums.ARTIST + " = ? ",
                        new String[] { album, artist }, null);
                if (cur != null && cur.moveToFirst()) {
                    album_key = cur.getString(cur
                            .getColumnIndex(MediaStore.Audio.Albums.ALBUM_KEY));
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }

        if (album == null || artist == null) {
            finish();
        } else {
            // Tweet Message 設定
            editText.setText(getTweetString(pref, title, artist));
            getWindow().setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
            // InputMethodManager inputMethodManager =
            // (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
            // inputMethodManager.showSoftInput(editText, 0);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
        // case R.id.btnArtwork:{}break;
        // case R.id.btnYoutube:{}break;
        case R.id.btnTweet: {
            if (tweet(this, replaceYoutubeLink())) {
                finish();
            }
        }
            break;
        case R.id.btnYoutube: {
            addYoutubeLink();
        }
            break;
        case R.id.btnOAuth: {
            boolean isChecked = btnOAuth.isChecked();
            if (isChecked) {
                // 認証
                auth();
            } else {
                // 認証削除
                Editor editor = pref.edit();
                editor.putString(SystemConsts.KEY_OAUTH_TOKEN, "");
                editor.putString(SystemConsts.KEY_OAUTH_TOKEN_SECRET, "");
                editor.commit();
                refreshOAuth();
            }

        }
            break;
        }
    }

    public static String getTweetString(SharedPreferences pref, String title,
            String artist) {
        StringBuilder buf = new StringBuilder();
        buf.append(pref.getString(SystemConsts.KEY_HASHTAG, "#nowplaying"))
                .append(" ").append(title).append(" - ").append(artist);
        return buf.toString();
    }

    public static boolean tweet(Context context, String text) {
        SharedPreferences pref = PreferenceManager
                .getDefaultSharedPreferences(context);
        String oauthToken = pref.getString(SystemConsts.KEY_OAUTH_TOKEN, "");
        String oauthTokenSecret = pref.getString(
                SystemConsts.KEY_OAUTH_TOKEN_SECRET, "");

        if (oauthToken.length() > 0 && oauthTokenSecret.length() > 0) {

            // 認証済み（直でTweet)
            AccessToken accessToken = new AccessToken(oauthToken,
                    oauthTokenSecret);
            Configuration config = new ConfigurationBuilder().build();
            Twitter twitter = new TwitterFactory(config)
                    .getInstance(new OAuthAuthorization(config));
            twitter.setOAuthAccessToken(accessToken);

            try {
                StatusUpdate update = new StatusUpdate(text);
                twitter.updateStatus(update);
                Toast.makeText(context,
                        context.getString(R.string.txt_tweet_ok),
                        Toast.LENGTH_SHORT).show();
                return true;
            } catch (TwitterException e) {
                Logger.e("twitter.updateStatus", e);
                Toast.makeText(context,
                        context.getString(R.string.txt_tweet_ng),
                        Toast.LENGTH_SHORT).show();
            }
        } else {

            // 非認証(Intent連携)
            Intent intent = new Intent(Intent.ACTION_SEND);
            intent.setType("text/plain");
            intent.putExtra(Intent.EXTRA_TEXT, text);
            try {
                context.startActivity(intent);
                return true;
            } catch (Exception e) {
            }
        }
        return false;
    }

    @Override
    public void afterTextChanged(Editable s) {
        String str = s.toString();
        final int n = 140 - str.length();
        textview.setText(Integer.toString(n));
        if (n >= 0) {
            btntw.setEnabled(true);
        } else {
            btntw.setEnabled(false);
        }
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count,
            int after) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (buttonView.getId() == R.id.checkAutoTweet) {
            Editor editor = pref.edit();
            editor.putBoolean("key.autoTweet", isChecked);
            editor.commit();
        }
    }

    private String YOUTUBE_LINK = "http://t.co/XXXXXXXX";

    private void addYoutubeLink() {
        StringBuilder buf = new StringBuilder();
        String text = editText.getText().toString();
        buf.append(text);
        buf.append(" ");

        buf.append(YOUTUBE_LINK);

        editText.setText(buf.toString());
    }

    private String replaceYoutubeLink() {
        String text = editText.getText().toString();
        try {
            if (text.indexOf(YOUTUBE_LINK) != -1) {
                StringBuilder url = new StringBuilder();
                url.append("http://www.youtube.com/results?search_query=")
                        .append(URLEncoder.encode(artist, "UTF-8")).append("+")
                        .append(URLEncoder.encode(title, "UTF-8"));

                String youtubelinke = url.toString();
                text = text.replace(YOUTUBE_LINK, youtubelinke);
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            text = text.replace(YOUTUBE_LINK, "");
        }
        return text;
    }

    Twitter twitter = null;
    RequestToken requestToken = null;

    private void auth() {
        // twitterはクラス変数。
        twitter = new TwitterFactory().getInstance();
        twitter.setOAuthConsumer(SystemConsts.CONSUMER_KEY,
                SystemConsts.CONSUMER_SECRET);
        try {
            requestToken = twitter
                    .getOAuthRequestToken(SystemConsts.CALLBACK_URL);
            // 認証用URLをインテントにセット。
            Intent intent = new Intent(this, AuthActivity.class);
            intent.putExtra("auth_url", requestToken.getAuthorizationURL());

            this.startActivityForResult(intent, SystemConsts.REQ_OAUTH);
        } catch (TwitterException e) {
            Logger.e("getOAuthRequestToken", e);
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_LONG).show();
            finish();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == Activity.RESULT_OK && twitter != null) {
            AccessToken accessToken;
            try {
                String verifier = data.getExtras().getString(
                        SystemConsts.KEY_OAUTH_VERIFIER);
                if (verifier != null && verifier.length() > 0) {
                    accessToken = twitter.getOAuthAccessToken(
                            requestToken,
                            data.getExtras().getString(
                                    SystemConsts.KEY_OAUTH_VERIFIER));

                    String oauthToken = accessToken.getToken();
                    String oauthTokenSecret = accessToken.getTokenSecret();
                    Editor editor = pref.edit();
                    editor.putString(SystemConsts.KEY_OAUTH_TOKEN, oauthToken);
                    editor.putString(SystemConsts.KEY_OAUTH_TOKEN_SECRET,
                            oauthTokenSecret);
                    editor.commit();
                } else {
                    Editor editor = pref.edit();
                    editor.putString(SystemConsts.KEY_OAUTH_TOKEN, "");
                    editor.putString(SystemConsts.KEY_OAUTH_TOKEN_SECRET, "");
                    editor.commit();
                }
            } catch (TwitterException e) {
                Logger.e("getOAuthAccessToken", e);
                Toast.makeText(this, e.getMessage(), Toast.LENGTH_LONG).show();
            }
        }
        refreshOAuth();
    }

    private void refreshOAuth() {
        String oauthToken = pref.getString(SystemConsts.KEY_OAUTH_TOKEN, "");
        String oauthTokenSecret = pref.getString(
                SystemConsts.KEY_OAUTH_TOKEN_SECRET, "");

        if (oauthToken.length() > 0 && oauthTokenSecret.length() > 0) {
            btnOAuth.setChecked(true);
        } else {
            btnOAuth.setChecked(false);
        }
    }
}
