package jp.co.kayo.android.localplayer.appwidget;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.util.ThemeHelper;
import android.os.Bundle;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
import android.app.ListActivity;
import android.content.Intent;
import android.graphics.Color;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;

public class ColorPickerDialog extends ListActivity {

    ArrayAdapter<Integer> adapter;

    Integer[] colorlist = new Integer[] { Color.BLACK, Color.BLUE, Color.CYAN,
            Color.DKGRAY, Color.GRAY, Color.GREEN, Color.LTGRAY, Color.MAGENTA,
            Color.RED, Color.WHITE, Color.YELLOW };

    String[] namelist = new String[] { "BLACK", "BLUE", "CYAN", "DKGRAY",
            "GRAY", "GREEN", "LTGRAY", "MAGENTA", "RED", "WHITE", "YELLOW" };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        new ThemeHelper().selectTheme(this);
        super.onCreate(savedInstanceState);

        setContentView(R.layout.colorpicker);

        adapter = new ArrayAdapter<Integer>(this,
                android.R.layout.simple_list_item_1, colorlist) {
            @Override
            public View getView(int position, View convertView, ViewGroup parent) {
                TextView view = (TextView) super.getView(position, convertView,
                        parent);
                TextView text = (TextView) view
                        .findViewById(android.R.id.text1);

                int item = colorlist[position];

                view.setBackgroundColor(item);

                text.setTextSize(14);
                text.setGravity(Gravity.TOP | Gravity.LEFT);

                float[] hsv = new float[3];
                Color.colorToHSV(item, hsv);

                if (hsv[2] < 0.7) {
                    text.setTextColor(Color.LTGRAY);
                } else {
                    text.setTextColor(Color.DKGRAY);
                }

                text.setText(namelist[position]);

                return view;
            }
        };

        getListView().setAdapter(adapter);

        getListView().setOnItemClickListener(new OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> arg0, View arg1, int pos,
                    long arg3) {
                final Intent intent = new Intent();
                int color = colorlist[pos];
                intent.putExtra(Intent.ACTION_PICK, color);
                intent.putExtra("name", namelist[pos]);
                intent.putExtra("color", getColorString(color));
                setResult(RESULT_OK, intent);
                finish();
            }
        });

    }

    public String getColorString(int icolor) {
        int r = Color.red(icolor);
        int g = Color.green(icolor);
        int b = Color.blue(icolor);
        return "#" + toHexString(r, 2) + toHexString(g, 2) + toHexString(b, 2);
    }

    public String toHexString(int value, int n) {
        String s = Integer.toHexString(value);
        StringBuilder sb = new StringBuilder();
        for (int i = s.length(); i < n; i++) {
            sb.append("0");
        }
        sb.append(s);
        return sb.toString().toUpperCase();
    }
}