package jp.co.kayo.android.localplayer.provider;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.File;
import java.io.IOException;

import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.db.JustPlayerDatabaseHelper;
import jp.co.kayo.android.localplayer.util.FileUtils;
import jp.co.kayo.android.localplayer.util.Logger;
import android.app.Activity;
import android.app.DownloadManager;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Environment;
import android.os.IBinder;

public class DownloadReceiver extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {
        Logger.d("DownloadReceiver.onReceive:" + intent.getAction());
        if (intent != null && intent.getAction() != null) {
            if (intent.getAction().equals(
                    DownloadManager.ACTION_DOWNLOAD_COMPLETE)) {
                long id = intent.getLongExtra(
                        DownloadManager.EXTRA_DOWNLOAD_ID, -1);
                Logger.d("download id=" + id);
                if (id != -1) {
                    Intent service = new Intent(context,
                            MyDownloadService.class);
                    service.putExtra(DownloadManager.EXTRA_DOWNLOAD_ID, id);
                    context.startService(service);
                }
            }
        }
    }

    public static class MyDownloadService extends Service {

        @Override
        public IBinder onBind(Intent intent) {
            return null;
        }

        @Override
        public void onStart(Intent intent, int startId) {
            super.onStart(intent, startId);

            if (intent != null) {
                long id = intent.getLongExtra(
                        DownloadManager.EXTRA_DOWNLOAD_ID, -1);
                if (id != -1) {
                    try {
                        downloadComplete(id);
                    } catch (IOException e) {
                        // notify

                    }
                }
            }
        }

        private void downloadComplete(long id) throws IOException {

            boolean needRemove = false;
            JustPlayerDatabaseHelper helper = null;
            SQLiteDatabase db = null;
            Cursor cur = null;
            try {
                helper = new JustPlayerDatabaseHelper(this);
                db = helper.getReadableDatabase();
                cur = db.query(TableConsts.TBNAME_DOWNLOAD, null,
                        TableConsts.DOWNLOAD_ID + " = ?",
                        new String[] { Long.toString(id) }, null, null, null);
                if (cur != null && cur.moveToFirst()) {
                    long media_id = cur.getLong(cur
                            .getColumnIndex(TableConsts.DOWNLOAD_MEDIA_ID));
                    String local_file = cur.getString(cur
                            .getColumnIndex(TableConsts.DOWNLOAD_LOCAL_URI));
                    String cache_file = cur.getString(cur
                            .getColumnIndex(TableConsts.DOWNLOAD_REMOTE_URI));
                    int type = cur.getInt(cur
                            .getColumnIndex(TableConsts.DOWNLOAD_TYPE));
                    int status = cur.getInt(cur
                            .getColumnIndex(TableConsts.DOWNLOAD_STATUS));

                    if (type == 0) {
                        // music
                        needRemove = createCacheFile(media_id, local_file,
                                cache_file, type, status);
                    } else {
                        // video
                    }
                }
            } finally {
                if (needRemove) {
                    Logger.d("needRemove=" + id);
                    DownloadManager downLoadManager = (DownloadManager) getSystemService(Activity.DOWNLOAD_SERVICE);
                    downLoadManager.remove(id);
                }
                if (cur != null) {
                    cur.close();
                }
                if (db != null) {
                    db.close();
                }
                if (helper != null) {
                    helper.close();
                }
            }
        }

        public boolean createCacheFile(long media_id, String local_file,
                String cache_file, int type, int status) throws IOException {
            File pathExternalPublicDir = Environment
                    .getExternalStoragePublicDirectory(Environment.DIRECTORY_MUSIC);
            if (!pathExternalPublicDir.exists()) {
                boolean b = pathExternalPublicDir.mkdirs();
                Logger.d("pathExternalPublicDir="
                        + pathExternalPublicDir.getPath() + " mkdirs=" + b);
            }

            File localFile = new File(pathExternalPublicDir, local_file);
            File cacheFile = new File(cache_file);
            Logger.d("localFile=" + localFile.getPath() + " exists="
                    + localFile.exists());
            Logger.d("cacheFile=" + cacheFile.getPath() + " exists="
                    + cacheFile.exists());
            try {
                if (localFile.exists() && !cacheFile.exists()) {
                    cacheFile.mkdirs();
                    FileUtils.copyFile(localFile, cacheFile);
                    localFile.delete();
                    return true;
                }
                return false;
            } finally {
            }
        }
    }

}
