package jp.co.kayo.android.localplayer.service;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.File;
import java.io.InputStream;
import java.util.Hashtable;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.appwidget.AppWidgetHelper;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.service.RemoteControlClientCompat.MetadataEditorCompat;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.Logger;
import android.app.PendingIntent;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.AudioManager;
import android.media.MediaMetadataRetriever;
import android.media.AudioManager.OnAudioFocusChangeListener;
import android.media.RemoteControlClient;
import android.net.Uri;
import android.os.Build;

public class AudioFocus {
    RemoteControlClientCompat mRemoteControlClientCompat;
    ComponentName mRemoteControlResponder;

    private OnAudioFocusChangeListener mAudioFocusListener = new OnAudioFocusChangeListener() {
        public void onAudioFocusChange(int focusChange) {
            Logger.d("onAudioFocusChange");
        }
    };

    public AudioFocus(Context context, AudioManager audiomgr) {
        mRemoteControlResponder = new ComponentName(context.getPackageName(),
                RemoteControlReceiver.class.getName());
        audiomgr.registerMediaButtonEventReceiver(mRemoteControlResponder);

        if (Build.VERSION.SDK_INT >= 14) {
            Intent intent = new Intent(Intent.ACTION_MEDIA_BUTTON);
            intent.setComponent(mRemoteControlResponder);
            mRemoteControlClientCompat = new RemoteControlClientCompat(
                    PendingIntent.getBroadcast(context, 0 /*
                                                           * requestCode,
                                                           * ignored
                                                           */,
                            intent /* intent */, 0 /* flags */));
            RemoteControlHelper.registerRemoteControlClient(audiomgr,
                    mRemoteControlClientCompat);
        }
    }

    public void unregister(Context context, AudioManager audiomgr) {
        if (Build.VERSION.SDK_INT >= 14) {
            if (mRemoteControlClientCompat != null) {
                mRemoteControlClientCompat
                        .setPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
            }
        }
    }

    public Bitmap register(Context context, AudioManager audiomgr,
            Hashtable<String, String> tbl1, Hashtable<String, String> tbl2) {
        audiomgr.requestAudioFocus(mAudioFocusListener,
                AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);

        Bitmap bitmap = null;
        if (mRemoteControlClientCompat != null) {
            if (Build.VERSION.SDK_INT >= 14) {
                mRemoteControlClientCompat
                        .setPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);

                mRemoteControlClientCompat
                        .setTransportControlFlags(
                                RemoteControlClient.FLAG_KEY_MEDIA_PLAY
                                | RemoteControlClient.FLAG_KEY_MEDIA_PAUSE
                                | RemoteControlClient.FLAG_KEY_MEDIA_NEXT
                                | RemoteControlClient.FLAG_KEY_MEDIA_PREVIOUS
                                | RemoteControlClient.FLAG_KEY_MEDIA_STOP);
            }

            // Update the remote controls
            try {
                Integer key = AppWidgetHelper
                        .getAlbumKey(tbl2.get(AudioAlbum.ALBUM),
                                tbl2.get(AudioAlbum.ARTIST));
                File file = AppWidgetHelper.createAlbumArtFile(key);
                if (file != null && file.exists()) {
                    bitmap = loadBitmap(context, Uri.fromFile(file), 
                            context.getResources().getDimensionPixelSize(R.dimen.albumart_size));
                } else {
                    bitmap = BitmapFactory.decodeResource(
                            context.getResources(), R.drawable.appicon);
                }

                MetadataEditorCompat editor = mRemoteControlClientCompat
                        .editMetadata(true)
                        .putString(MediaMetadataRetriever.METADATA_KEY_ARTIST,
                                tbl1.get(AudioMedia.ARTIST))
                        .putString(MediaMetadataRetriever.METADATA_KEY_ALBUM,
                                tbl1.get(AudioMedia.ALBUM))
                        .putString(MediaMetadataRetriever.METADATA_KEY_TITLE,
                                tbl1.get(AudioMedia.TITLE))
                        .putLong(MediaMetadataRetriever.METADATA_KEY_DURATION,
                                Funcs.parseLong(tbl1.get(AudioMedia.DURATION)));
                if (bitmap != null) {
                    editor.putBitmap(
                            RemoteControlClientCompat.MetadataEditorCompat.METADATA_KEY_ARTWORK,
                            bitmap);
                }
                editor.apply();
            } finally {
            }
        }
        audiomgr.requestAudioFocus(mAudioFocusListener,
                AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);

        return bitmap;
    }

    public void unregister(AudioManager audiomgr) {
        if (Build.VERSION.SDK_INT >= 14) {
            if (mRemoteControlClientCompat != null) {
                mRemoteControlClientCompat
                        .setPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
            }
        }
        audiomgr.unregisterMediaButtonEventReceiver(mRemoteControlResponder);
        audiomgr.abandonAudioFocus(mAudioFocusListener);
    }

    public static Bitmap loadBitmap(Context context, Uri uri, int fitsize) {
        InputStream is = null;
        Bitmap bmp = null;
        try {
            // 読み込む画像へのUriがある場合は読み込む
            if (uri != null) {
                // まずはオリジナルの縦と横のサイズを取得するため、
                // 画像読み込みなし(inJustDecodeBounds = true)で画像を読み込む
                is = context.getContentResolver().openInputStream(uri);
                BitmapFactory.Options opts = new BitmapFactory.Options();
                opts.inJustDecodeBounds = true;
                BitmapFactory.decodeStream(is, null, opts);
                is.close();
                is = null;

                // fitsizeに合わせてX方向とY方向それぞれのinSampleSizeを求める
                int sample_x = 1 + (opts.outWidth / fitsize);
                int sample_y = 1 + (opts.outHeight / fitsize);

                is = context.getContentResolver().openInputStream(uri);
                opts.inSampleSize = Math.max(sample_x, sample_y);
                opts.inJustDecodeBounds = false;
                bmp = BitmapFactory.decodeStream(is, null, opts);
                return bmp;
            }
        } catch (Exception e) {
            Logger.e("loadBitmap", e);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (Exception e) {
                }
            }
        }
        return null;
    }
}
