package jp.co.kayo.android.localplayer.service;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Random;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.RemoteException;
import android.preference.PreferenceManager;

import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.provider.DeviceContentProvider;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.bean.MediaData;

public class JukeBox {
    public static final String PREF_DS = "pref.ds";
    private static final String PREF_POSITION = "pref.position";
    private static final String PREF_PLAYPOSITION = "pref.playposition";

    private int repeatmode;
    private ArrayList<MediaData> playlist = new ArrayList<MediaData>();
    private ArrayList<Integer> shuffle = null;
    private int position;
    private int playPosition;
    private int seekPosition = 0;
    private Random rand = new Random(System.nanoTime());
    private boolean isLoaded = false;
    private SharedPreferences pref;
    private Context mContext;

    public JukeBox(Context context) {
        mContext = context;
        pref = PreferenceManager.getDefaultSharedPreferences(context);
    }

    public boolean isLoaded() {
        return isLoaded;
    }

    public void load() {
        isLoaded = true;
        playlist = new ArrayList<MediaData>();
        shuffle = null;
        // load playlist
        String cur_contenturi = pref.getString(SystemConsts.PREF_CONTENTURI,
                DeviceContentProvider.MEDIA_AUTHORITY);
        String save_contenturi = pref.getString(PREF_DS, "");
        if (save_contenturi.equals(cur_contenturi)) {
            if (mContext.getFileStreamPath("history.dat").exists()) {
                FileInputStream fileInputStream = null;
                try {
                    fileInputStream = mContext.openFileInput("history.dat");
                    ObjectInputStream oi = new ObjectInputStream(
                            fileInputStream);
                    ArrayList<MediaData> pp = (ArrayList<MediaData>) oi
                            .readObject();
                    if (pp != null) {
                        playlist = pp;
                    }
                    ArrayList<Integer> sp = (ArrayList<Integer>) oi
                            .readObject();
                    if (sp != null) {
                        shuffle = sp;
                    }
                } catch (Exception e) {
                    Logger.e(e.getMessage(), e);
                } finally {
                    if (fileInputStream != null) {
                        try {
                            fileInputStream.close();
                        } catch (IOException e) {
                        }
                    }
                }
            }

            setPlayPosition(pref.getInt(PREF_PLAYPOSITION, 0));
            setPosition(pref.getInt(PREF_POSITION, 0));
        }

        setRepeatMode(pref.getInt(SystemConsts.PREF_REPEATFLG,
                SystemConsts.FLG_REPEAT_NO));

    }

    public void save() {
        Editor editor = pref.edit();

        if (getList().size() > 0) {
            FileOutputStream fileOutputStream = null;
            try {
                fileOutputStream = mContext.openFileOutput("history.dat",
                        Context.MODE_PRIVATE);
                ObjectOutputStream oo = new ObjectOutputStream(fileOutputStream);
                oo.writeObject(getList());
                if (getShaffle() != null && getShaffle().size() > 0) {
                    oo.writeObject(getShaffle());
                }
                oo.close();
            } catch (IOException e) {
                mContext.deleteFile("history.dat");
                Logger.e(e.getMessage(), e);
            } finally {
                if (fileOutputStream != null) {
                    try {
                        fileOutputStream.close();
                    } catch (IOException e) {
                    }
                }
            }
        } else {
            mContext.deleteFile("history.dat");
        }

        editor.putString(PREF_DS, pref.getString(SystemConsts.PREF_CONTENTURI,
                DeviceContentProvider.MEDIA_AUTHORITY));
        editor.putInt(PREF_POSITION, getPosition());
        editor.putInt(PREF_PLAYPOSITION, getPlayPosition());
        editor.putInt(SystemConsts.PREF_REPEATFLG, getRepeatMode());

        editor.commit();
    }

    /***
     * ノーチェック処理（復帰時に使用する)
     * 
     * @param shuffle
     */
    public void setShaffle(ArrayList<Integer> shuffle) {
        this.shuffle = shuffle;
    }

    /***
     * ノーチェック処理（復帰時に使用する)
     * 
     * @param shuffle
     */
    public void setPosition(int pos) {
        position = pos;
    }

    public int getSeekPosition() {
        return seekPosition;
    }

    public void setSeekPosition(int seekPosition) {
        this.seekPosition = seekPosition;
    }

    public ArrayList<Integer> getShaffle() {
        return this.shuffle;
    }

    public int getRepeatMode() {
        return repeatmode;
    }

    public void setRepeatMode(int mode) {
        this.repeatmode = mode;
    }

    public void addMedia(MediaData media) {
        playlist.add(media);

        if (shuffle != null) {
            int pos = playlist.size() - 1;
            int index = shuffle.size() > 1 ? rand.nextInt(shuffle.size() - 1) + 1
                    : 0;
            if (shuffle.size() > index && index > 0) {
                shuffle.add(index, pos);
            } else {
                shuffle.add(pos);
            }
        }
    }

    public ArrayList<MediaData> getList() {
        return playlist;
    }

    public int getPosition() {
        return position;
    }

    public int getPlayPosition() {
        return playPosition;
    }

    public void setPlayPosition(int playPosition) {
        this.playPosition = playPosition;
    }

    public void incRepeat() {
        if (repeatmode < 2) {
            repeatmode++;
        } else {
            repeatmode = 0;
        }

        Editor editor = pref.edit();
        editor.putInt(SystemConsts.PREF_REPEATFLG, getRepeatMode());
        editor.commit();
    }

    public void shuffle() {
        if (shuffle == null) {
            makeShuffle();
        } else {
            shuffle = null;
        }

        save();
    }

    public boolean clear() {
        position = -1;
        if (playlist.size() > 0) {
            playlist.clear();
            if (shuffle != null) {
                shuffle.clear();
            }
            return true;
        } else {
            return false;
        }
    }

    public boolean clearcut() throws RemoteException {
        Logger.d("MediaPlayerService.clearcut");
        boolean dirty = false;
        // 未再生のものだけ消す
        if (position >= 0) {
            dirty = true;
            if (shuffle != null) {
                while (shuffle.size() > 1) {
                    int n = shuffle.remove(0);
                    if (playlist.size() > n) {
                        playlist.remove(n);
                    }
                }
                shuffle.clear();
            } else {
                while (playlist.size() > (position + 1)) {
                    playlist.remove(playlist.size() - 1);
                }
            }
        } else {
            if (playlist.size() > 0) {
                dirty = true;
                playlist.clear();
                if (shuffle != null) {
                    shuffle.clear();
                }
            }
        }
        if (dirty) {
            position = playlist.size();
            return true;
        } else {
            return false;
        }
    }

    public boolean removePosition(int pos) throws RemoteException {
        MediaData item = playlist.get(pos);
        playlist.remove(item);
        if (shuffle != null && shuffle.size() > 0) {
            for (int i = 0; i < shuffle.size(); i++) {
                if (shuffle.get(i) == pos) {
                    shuffle.remove(i);
                    break;
                }
            }
        }
        if (pos == position) {
            return true;
        } else {
            if (pos < position) {
                position--;
            }
        }

        return false;
    }

    public void movePosition(int pos) {
        position = pos;
        if (shuffle != null && shuffle.size() > 0) {
            for (int i = 0; i < shuffle.size(); i++) {
                if (shuffle.get(i) == pos) {
                    Integer p = shuffle.remove(i);
                    shuffle.add(0, p);
                    return;
                }
            }
            shuffle.add(0, new Integer(pos));
        }
    }

    public boolean moveNext() {
        if (repeatmode == SystemConsts.FLG_REPEAT_ONCE) {
            return position != -1;
        }
        if (shuffle != null) {
            if (shuffle.size() > 0) {
                shuffle.remove(0);
                if (shuffle.size() > 0) {
                    position = shuffle.get(0);
                } else {
                    if (repeatmode == SystemConsts.FLG_REPEAT_ALL) {
                        if (playlist.size() > 0) {
                            makeShuffle();
                            position = shuffle.get(0);
                        } else {
                            position = -1;
                        }
                    } else {
                        position = -1;
                    }
                }
                return true;
            } else {
                if (repeatmode == SystemConsts.FLG_REPEAT_ALL) {
                    if (playlist.size() > 0) {
                        makeShuffle();
                        position = shuffle.get(0);
                        return true;
                    } else {
                        position = -1;
                        return false;
                    }
                } else {
                    position = -1;
                    return false;
                }
            }
        } else {
            if ((position + 1) < playlist.size() && position >= 0) {
                position++;
                return true;
            } else {
                if (repeatmode == SystemConsts.FLG_REPEAT_ALL) {
                    if (playlist.size() > 0) {
                        position = 0;
                        return true;
                    } else {
                        position = -1;
                        return false;
                    }
                } else {
                    position = -1;
                    return false;
                }
            }
        }
    }

    public MediaData getCurrent() {
        if (shuffle != null) {
            if (shuffle.size() > 0 && playlist.size() > 0) {
                return playlist.get(shuffle.get(0));
            }
        } else {
            if (position != -1) {
                if (playlist.size() > position) {
                    return playlist.get(position);
                }
            } else if (playlist.size() > 0) {
                position = 0;
                return playlist.get(0);
            }
        }
        return null;
    }

    public MediaData getNextPrefetch(int next) {
        if (shuffle != null) {
            if (shuffle.size() > next) {
                int pos = shuffle.get(next);
                if (playlist.size() > pos) {
                    return playlist.get(pos);
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            if (playlist.size() > (position + next)) {
                return playlist.get(position + next);
            } else {
                int pos = (position + next) - playlist.size();// 二週目があるとばぐっちゃう。だけど呼ばれることはない
                if (playlist.size() > pos) {
                    return playlist.get(pos);
                } else {
                    return null;
                }
            }
        }
    }

    public int getNextPosition() {
        if (shuffle != null) {
            if (shuffle.size() > 1) {
                return shuffle.get(1);
            } else {
                if (repeatmode == SystemConsts.FLG_REPEAT_ONCE
                        && shuffle.size() != 0) {
                    return shuffle.get(0);
                } else if (repeatmode == SystemConsts.FLG_REPEAT_ALL) {
                    return 0;
                } else {
                    return -1;
                }
            }
        } else {
            if (playlist.size() > (position + 1)) {
                return position + 1;
            } else {
                if (repeatmode == SystemConsts.FLG_REPEAT_ONCE) {
                    return position;
                } else if (repeatmode == SystemConsts.FLG_REPEAT_ALL) {
                    return 0;
                } else {
                    return -1;
                }
            }
        }
    }

    public int getBack(int pos, long duration) {
        // 全体の１０％以下なら頭出し、そうでなければ前の曲
        if (duration > 0 && pos > 0) {
            float pt = ((float) pos / duration) * 100.0f;
            if (pt > 10) {
                return position;
            } else {
                if (position > 0) {
                    return position - 1;
                } else {
                    return position;
                }
            }
        } else {
            if (position > 0) {
                return position - 1;
            } else {
                return position;
            }
        }
    }

    public boolean hasNext() {
        if (shuffle != null) {
            if (shuffle.size() > 1) {
                return true;
            } else {
                if (repeatmode == SystemConsts.FLG_REPEAT_ONCE
                        && shuffle.size() > 0) {
                    return true;
                } else if (repeatmode == SystemConsts.FLG_REPEAT_ALL
                        && shuffle.size() > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        } else {
            if (playlist.size() > (position + 1)) {
                return true;
            } else {
                if (repeatmode == SystemConsts.FLG_REPEAT_ONCE
                        && playlist.size() > 0) {
                    return true;
                } else if (repeatmode == SystemConsts.FLG_REPEAT_ALL
                        && playlist.size() > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        }
    }

    public boolean drop(int from, int to) {
        MediaData item = playlist.get(from);
        playlist.remove(item);
        playlist.add(to, item);
        if (from == position) {
            position = to;
        } else {
            if (to < position) {
                if (from > position) {
                    position++;
                }
            } else {
                if (from < position) {
                    position--;
                }
            }
        }
        return true;
    }

    private void makeShuffle() {
        int count = playlist.size();
        if (shuffle == null) {
            shuffle = new ArrayList<Integer>(count);
        } else if (shuffle.size() > 0) {
            shuffle.clear();
        }
        for (int i = 0; i < count; i++) {
            int index = shuffle.size() > 0 ? rand.nextInt(shuffle.size()) : 0;
            if (shuffle.size() > index) {
                shuffle.add(index, i);
            } else {
                shuffle.add(i);
            }
        }

        for (int i = 0; i < shuffle.size(); i++) {
            if (shuffle.get(i) == position) {
                Integer p = shuffle.remove(i);
                shuffle.add(0, p);
                break;
            }
        }
    }

}
