
package jp.co.kayo.android.localplayer.service;

/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.File;
import java.io.IOException;
import java.lang.ref.WeakReference;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Hashtable;

import jp.co.kayo.android.localplayer.AlertDialogActivity;
import jp.co.kayo.android.localplayer.LockScreenActivity;
import jp.co.kayo.android.localplayer.MainActivity;
import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.appwidget.AppWidgetHelper;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.Auth;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.fx.AudioFx;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.ui.pref.CacheConfigPreference;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.NetworkHelper;
import jp.co.kayo.android.localplayer.util.SSLHelper;
import jp.co.kayo.android.localplayer.util.SoundManager;
import jp.co.kayo.android.localplayer.util.bean.MediaData;
import android.app.Activity;
import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnBufferingUpdateListener;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnErrorListener;
import android.media.MediaPlayer.OnPreparedListener;
import android.net.ConnectivityManager;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.net.wifi.WifiManager.WifiLock;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteCallbackList;
import android.os.RemoteException;
import android.os.SystemClock;
import android.preference.PreferenceManager;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.widget.Toast;

public class MediaPlayerService extends Service implements OnPreparedListener,
        OnCompletionListener, OnBufferingUpdateListener, OnErrorListener,
        OnPrefetchProgressListener {

    public static final String CALL_RESUME = "MediaPlayerService.Call.Resume";
    public static final String CALL_KEEPALIVE = "MediaPlayerService.Call.KeepAlive";
    public static final String CALL_KEEPSESSION = "MediaPlayerService.Call.KeepSession";
    public static final int NFCSHARE_RUNNING = 999;
    private static final int KEEP_DELAY = 60 * 30;

    private MediaPlayer mp;
    private AudioFx audiofx;
    private final RemoteCallbackList<IMediaPlayerServiceCallback> callbackList = new RemoteCallbackList<IMediaPlayerServiceCallback>();
    JukeBox jukebox;
    private boolean prepared = false;
    private boolean firsttime = true;
    private boolean isInitService = false;
    SoundManager soundmanager = new SoundManager();
    TelephonyManager tm;
    AudioManager audiomgr;
    AudioFocus audiofocus;
    private StreamCacherServer streamcacheserv;
    SharedPreferences pref;
    private boolean error = false; // 再生エラーが発生した場合
    private boolean retryConnect = false;
    private boolean updatelist = false;
    private boolean updatelock = false;
    private long prefetchId = -1;
    private WifiLock mWifiLock = null;
    Notification mNotification;

    public static final int MSG_CONNECTEXCEPTION = 1;
    public static final int MSG_IOEXCEPTION = 2;
    public static final int MSG_FILENOTFOUNDEXCEPTION = 3;
    public static final int MSG_STREAMING_NEXT = 4;
    public static final int MSG_STREAMING_CACHE_NOTHAVESPACE = 5;

    private final String META_CHANGED = "fm.last.android.metachanged";
    private final String PLAYBACK_FINISHED = "fm.last.android.playbackcomplete";
    private final String PLAYBACK_STATE_CHANGED = "fm.last.android.playstatechanged";
    private final String STATION_CHANGED = "fm.last.android.stationchanged";
    private final String PLAYBACK_ERROR = "fm.last.android.playbackerror";
    private final String UNKNOWN = "fm.last.android.unknown";

    enum RING_STATE {
        STATE_RINGING, STATE_OFFHOOK, STATE_NORMAL
    };

    private static final Class<?>[] mSetForegroundSignature = new Class[] {
        boolean.class
    };
    private static final Class<?>[] mStartForegroundSignature = new Class[] {
            int.class, Notification.class
    };
    private static final Class<?>[] mStopForegroundSignature = new Class[] {
        boolean.class
    };

    private NotificationManager mNM;
    private Method mSetForeground;
    private Method mStartForeground;
    private Method mStopForeground;
    private Object[] mSetForegroundArgs = new Object[1];
    private Object[] mStartForegroundArgs = new Object[2];
    private Object[] mStopForegroundArgs = new Object[1];

    void invokeMethod(Method method, Object[] args) {
        try {
            method.invoke(this, args);
        } catch (InvocationTargetException e) {
            // Should not happen.
            Logger.e("Unable to invoke method", e);
        } catch (IllegalAccessException e) {
            // Should not happen.
            Logger.e("Unable to invoke method", e);
        }
    }

    /**
     * This is a wrapper around the new startForeground method, using the older
     * APIs if it is not available.
     */
    void startForegroundCompat(int id, Notification notification) {
        // If we have the new startForeground API, then use it.
        if (mStartForeground != null) {
            mStartForegroundArgs[0] = Integer.valueOf(id);
            mStartForegroundArgs[1] = notification;
            invokeMethod(mStartForeground, mStartForegroundArgs);
            return;
        }

        // Fall back on the old API.
        mSetForegroundArgs[0] = Boolean.TRUE;
        invokeMethod(mSetForeground, mSetForegroundArgs);
        mNM.notify(id, notification);
    }

    /**
     * This is a wrapper around the new stopForeground method, using the older
     * APIs if it is not available.
     */
    void stopForegroundCompat(int id) {
        // If we have the new stopForeground API, then use it.
        if (mStopForeground != null) {
            mStopForegroundArgs[0] = Boolean.TRUE;
            invokeMethod(mStopForeground, mStopForegroundArgs);
            return;
        }

        // Fall back on the old API. Note to cancel BEFORE changing the
        // foreground state, since we could be killed at that point.
        mNM.cancel(id);
        mSetForegroundArgs[0] = Boolean.FALSE;
        invokeMethod(mSetForeground, mSetForegroundArgs);
    }

    private Handler mHandler = new MyHandler(this);

    private static class MyHandler extends Handler {
        WeakReference<MediaPlayerService> refService;

        MyHandler(MediaPlayerService service) {
            refService = new WeakReference<MediaPlayerService>(service);
        }

        @Override
        public void handleMessage(Message msg) {
            MediaPlayerService service = refService.get();
            if (service != null) {
                switch (msg.what) {
                    case MSG_CONNECTEXCEPTION: {
                        Logger.d("MediaPlayerSerivce.MSG_CONNECTEXCEPTION");
                        service.stop();
                        service.notifyOff(false);
                    }
                        break;
                    case MSG_IOEXCEPTION: {
                        // MediaPlayerService.this.stop();
                    }
                        break;
                    case MSG_FILENOTFOUNDEXCEPTION: {
                        // getContentResolver().query(MediaConsts.CLEAR_CONTENT_URI,
                        // null, null, null, null);
                    }
                        break;
                    case MSG_STREAMING_NEXT: {
                        if (msg.obj != null) {
                            Object[] params = (Object[]) msg.obj;
                            if (params != null && params.length > 1 && params[0] instanceof Long) {
                                ContentValues values = new ContentValues();
                                values.put(AudioMedia.AUDIO_CACHE_FILE, (String) params[1]);
                                service.getContentResolver().update(
                                        ContentUris.withAppendedId(
                                                MediaConsts.MEDIA_CONTENT_URI, (Long) params[0]),
                                        values, null, null);
                            }
                        }
                        service.startPrefetch(false);
                    }
                        break;
                    case MSG_STREAMING_CACHE_NOTHAVESPACE: {
                        if (service.mNM != null) {
                            Notification notification;
                            notification = new Notification(R.drawable.status,
                                    service.getString(R.string.txt_notify_info),
                                    System.currentTimeMillis());

                            Intent intent = new Intent(service,
                                    CacheConfigPreference.class);

                            PendingIntent contentIntent = PendingIntent.getActivity(
                                    service.getContext(), 0, intent,
                                    Intent.FLAG_ACTIVITY_NEW_TASK);
                            notification.setLatestEventInfo(service,
                                    service.getString(R.string.app_name),
                                    service.getString(R.string.txt_notify_cache_limit_nospace),
                                    contentIntent);

                            service.mNM.notify(R.string.txt_notify_info, notification);
                        }
                    }
                        break;
                }
            }
        }

    };

    IMediaPlayerService.Stub bind = new IMediaPlayerService.Stub() {
        private String contentkey = null;

        @Override
        public boolean setContentsKey(String key) throws RemoteException {
            if (contentkey == null) {
                contentkey = key;
                return true;
            } else {
                if (contentkey.equals(key)) {
                    contentkey = key;
                    return false;
                } else {
                    contentkey = key;
                    return true;
                }
            }
        }

        @Override
        public void registerCallback(IMediaPlayerServiceCallback callback)
                throws RemoteException {
            callbackList.register(callback);
        }

        @Override
        public void unregisterCallback(IMediaPlayerServiceCallback callback)
                throws RemoteException {
            callbackList.unregister(callback);
        }

        @Override
        public void addMedia(long id, long duration, String data)
                throws RemoteException {
            updatelist = true;
            // Logger.d("MediaPlayerService.addMedia");
            jukebox.addMedia(new MediaData(id, duration, data));
        }

        @Override
        public void addMediaD(long id, long duration, String title,
                String album, String artist, String data)
                throws RemoteException {
            updatelist = true;
            // Logger.d("MediaPlayerService.addMedia");
            jukebox.addMedia(new MediaData(id, duration, title, album, artist,
                    data));
        }

        @Override
        public void clear() throws RemoteException {
            Logger.d("MediaPlayerService.clear");
            if (jukebox.clear()) {
                updatelist = true;
                prepared = false;
                if (mp != null && mp.isPlaying()) {
                    mp.stop();
                    mp.reset();
                    notifyOff(false);
                } else {
                    if (mp != null) {
                        mp.reset();
                    }
                }
                Editor editor = pref.edit();
                editor.putString("lastSetTitle", "");
                editor.putString("lastSetArtist", "");
                editor.putString("lastSetAlbum", "");
                editor.commit();

                updateView();
            }
            instantSave();
        }

        @Override
        public void clearcut() throws RemoteException {
            Logger.d("MediaPlayerService.clearcut");
            if (jukebox.clearcut()) {
                updatelist = true;
                if (mp != null) {
                    if (mp.isPlaying()) {
                        mp.stop();
                        mp.reset();
                        prepared = false;
                        notifyOff(false);
                    } else {
                        mp.reset();
                        prepared = false;
                    }
                }
                updateView();
            }
        }

        @Override
        public void release() throws RemoteException {
            Logger.d("release");
            if (mp == null || !mp.isPlaying()) {
                setResumeStart(60, AppWidgetHelper.CALL_STOP);
            }
        }

        @Override
        public void stop() throws RemoteException {
            MediaPlayerService.this.stop();
        }

        @Override
        public long[] getList() throws RemoteException {
            ArrayList<MediaData> list = jukebox.getList();
            long[] ret = new long[list.size()];
            for (int i = 0; i < list.size(); i++) {
                ret[i] = list.get(i).id;
            }
            return ret;
        }

        @Override
        public String[] getMediaD(int pos) throws RemoteException {
            ArrayList<MediaData> list = jukebox.getList();
            if (list.size() > pos) {
                MediaData data = list.get(pos);
                String[] ret = new String[] {
                        data.title, data.album,
                        data.artist, Long.toString(data.duration)
                };
                return ret;
            }
            return null;
        }

        @Override
        public void play() throws RemoteException {
            MediaPlayerService.this.play();
        }

        @Override
        public void pause() throws RemoteException {
            MediaPlayerService.this.pause();
        }

        @Override
        public void ff() throws RemoteException {
            MediaPlayerService.this.ff();
        }

        @Override
        public void rew() throws RemoteException {
            MediaPlayerService.this.rew();
        }

        @Override
        public void shuffle() throws RemoteException {
            MediaPlayerService.this.shuffle();
        }

        @Override
        public void repeat() throws RemoteException {
            MediaPlayerService.this.repeat();
        }

        @Override
        public void commit() throws RemoteException {
            updatelist = true;
            updatelock = false;
            updateView();

            startPrefetch(false);
        }

        @Override
        public int stat() throws RemoteException {
            return MediaPlayerService.this.stat(0);
        }

        @Override
        public void setPosition(int pos) throws RemoteException {
            int ppos = jukebox.getPosition();
            if (ppos != pos) {
                jukebox.setPlayPosition(0);
                jukebox.movePosition(pos);
            }
        }

        @Override
        public long getMediaId() throws RemoteException {
            MediaData inf = jukebox.getCurrent();
            if (inf != null) {
                return inf.id;
            } else {
                return -1;
            }
        }

        @Override
        public long getSeekPosition() throws RemoteException {
            try {
                if (mp != null && prepared) {
                    int pos = mp.getCurrentPosition();
                    jukebox.setPlayPosition(pos);
                    return pos;
                } else {
                    return 0;
                }
            } catch (Exception e) {
                Logger.e("getSeekPosition", e);
                throw new RemoteException();
            }
        }

        @Override
        public long getDuration() throws RemoteException {
            return MediaPlayerService.this.getDuration();
        }

        @Override
        public void seek(int msec) throws RemoteException {
            if (mp != null && prepared) {
                mp.seekTo(msec);
            }
        }

        @Override
        public int getCount() throws RemoteException {
            return jukebox.getList().size();
        }

        @Override
        public void remove(int pos) throws RemoteException {
            Logger.d("MediaPlayerService.remove");
            try {
                if (jukebox.removePosition(pos)) {
                    if (mp != null && mp.isPlaying()) {
                        mp.stop();
                        mp.reset();
                        prepared = false;
                        notifyOff(false);
                        if (jukebox.getList().size() > 0) {
                            MediaData inf = jukebox.getCurrent();
                            try {
                                jukebox.setPlayPosition(0);
                                setDataSource(inf);
                                return;
                            } catch (Exception e) {
                                Logger.e("play", e);
                            }
                        }
                    }
                }
                if (mp != null && mp.isPlaying()) {
                    startPrefetch(true);
                }
            } finally {
                updatelist = true;
                updateView();
            }
        }

        @Override
        public void drop(int from, int to) throws RemoteException {
            Logger.d("MediaPlayerService.drop");
            jukebox.drop(from, to);
            if (mp != null && mp.isPlaying()) {
                startPrefetch(true);
            }
            updatelist = true;
            updateView();
        }

        @Override
        public int getPosition() throws RemoteException {
            return jukebox.getPosition();
        }

        @Override
        public long getPrefetchId() throws RemoteException {
            return prefetchId;
        }

        @Override
        public int getAudioSessionId() throws RemoteException {
            if (mp != null) {
                if (Build.VERSION.SDK_INT > 8) {
                    return MediaPlayerHelper.getAudioSessionId(mp);
                }
            }
            return -1;
        }

        @Override
        public void lockUpdateToPlay() throws RemoteException {
            updatelock = true;
        }

        @Override
        public void setEqEnabled(boolean enabled) throws RemoteException {
            if (audiofx != null) {
                audiofx.setEqEnabled(enabled);
                try {
                    audiofx.save();
                } catch (Exception e) {
                }
            }
        }

        @Override
        public boolean getEqEnabled() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getEqEnabled();
            }
            return false;
        }

        @Override
        public void setRvEnabled(boolean enabled) throws RemoteException {
            if (audiofx != null) {
                audiofx.setRvEnabled(enabled);
                try {
                    audiofx.save();
                } catch (Exception e) {
                }
            }
        }

        @Override
        public boolean getRvEnabled() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getRvEnabled();
            }
            return false;
        }

        @Override
        public void setBsEnabled(boolean enabled) throws RemoteException {
            if (audiofx != null) {
                audiofx.setBsEnabled(enabled);
                try {
                    audiofx.save();
                } catch (Exception e) {
                }
            }
        }

        @Override
        public boolean getBsEnabled() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getBsEnabled();
            }
            return false;
        }

        @Override
        public int getNumberOfPresets() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getNumberOfPresets();
            }
            return 0;
        }

        @Override
        public String getPresetName(int n) throws RemoteException {
            if (audiofx != null) {
                return audiofx.getPresetName((short) n);
            }
            return null;
        }

        @Override
        public int getBandLevel(int band) throws RemoteException {
            if (audiofx != null) {
                return audiofx.getBandLevel((short) band);
            }
            return 0;
        }

        @Override
        public int getCenterFreq(int band) throws RemoteException {
            if (audiofx != null) {
                return audiofx.getCenterFreq((short) band);
            }
            return 0;
        }

        @Override
        public int getCurrentPreset() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getCurrentPreset();
            }
            return 0;
        }

        @Override
        public int getNumberOfBands() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getNumberOfBands();
            }
            return 0;
        }

        @Override
        public int getMinEQLevel() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getMinEQLevel();
            }
            return 0;
        }

        @Override
        public int getMaxEQLevel() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getMaxEQLevel();
            }
            return 0;
        }

        @Override
        public void usePreset(int preset) throws RemoteException {
            if (audiofx != null) {
                audiofx.usePreset((short) preset);
            }
        }

        @Override
        public void setBandLevel(int band, int level) throws RemoteException {
            if (audiofx != null) {
                audiofx.setBandLevel((short) band, (short) level);
            }
        }

        @Override
        public int getRvPreset() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getRvPreset();
            }
            return 0;
        }

        @Override
        public void setRvPreset(int preset) throws RemoteException {
            if (audiofx != null) {
                audiofx.setRvPreset((short) preset);
            }
        }

        @Override
        public int getStrength() throws RemoteException {
            if (audiofx != null) {
                return audiofx.getStrength();
            }
            return 0;
        }

        @Override
        public void setStrength(int strength) throws RemoteException {
            if (audiofx != null) {
                audiofx.setStrength((short) strength);
            }
        }

        @Override
        public String getContentsKey() throws RemoteException {
            return contentkey;
        }

        @Override
        public int getPort() throws RemoteException {
            if (streamcacheserv != null) {
                return streamcacheserv.getPort();
            }

            return -1;
        }

        @Override
        public boolean startBeam() throws RemoteException {
            if (jukebox != null && streamcacheserv != null) {
                streamcacheserv.setAllowIndex(jukebox);
                return true;
            }
            return false;
        }

        @Override
        public boolean startNfcServer() throws RemoteException {
            if (streamcacheserv != null) {
                try {
                    streamcacheserv.setNfcServer(true);
                    return true;
                } catch (IOException e) {
                    streamcacheserv = null;
                }
            }
            return false;
        }

        @Override
        public void stopNfcServer() throws RemoteException {
            if (streamcacheserv != null) {
                try {
                    streamcacheserv.setNfcServer(false);
                    // NFC共有の通知を終了
                    nfcShareNotifyOff();
                } catch (IOException e) {
                    streamcacheserv = null;
                }
            }
        }
    };

    protected void repeat() {
        jukebox.incRepeat();
        setRepeatMode();
        updateView();
    }

    protected void shuffle() {
        jukebox.shuffle();
        if (mp != null && mp.isPlaying()) {
            startPrefetch(true);
        }
        // saveShuffleFlg();
        updateView();
    }

    PendingIntent createIntent(Context context, String action) {
        Intent i = new Intent(context, MediaPlayerService.class);
        i.setAction(action);
        return PendingIntent.getService(this, 0, i, 0);
    }

    private void notifyOn(boolean prepared) {
        Logger.d("notify on:" + prepared);
        if (Build.VERSION.SDK_INT >= 11) {
            setResumeStop(AppWidgetHelper.CALL_NOTIFYOFF);
        }
        if (pref != null) {
            MediaData media = jukebox.getCurrent();
            if (media != null) {
                String lastSetAlbum = null, lastSetTitle = null, lastSetArtist = null;
                long duration = 0;
                Hashtable<String, String> tbl1 = null, tbl2 = null;
                Bitmap bitmap = null;
                if (media.getId() > 0) {
                    tbl1 = ContentsUtils.getMedia(this, new String[] {
                            AudioMedia.ALBUM, AudioMedia.ALBUM_KEY,
                            AudioMedia.ARTIST, AudioMedia.TITLE,
                            AudioMedia.DURATION
                    }, media.id);

                    if (Build.VERSION.SDK_INT > 7) {
                        if (audiofocus == null) {
                            audiofocus = new AudioFocus(this, audiomgr);
                        }
                        tbl2 = ContentsUtils.getAlbum(this, new String[] {
                                AudioAlbum.ALBUM, AudioAlbum.ARTIST,
                                AudioAlbum.ALBUM_ART
                        },
                                tbl1.get(AudioMedia.ALBUM_KEY));
                        // LockScreen MediaControl with
                        bitmap = audiofocus
                                .register(this, audiomgr, tbl1, tbl2);
                    }
                    lastSetAlbum = tbl1.get(AudioMedia.ALBUM);
                    lastSetTitle = tbl1.get(AudioMedia.TITLE);
                    lastSetArtist = tbl1.get(AudioMedia.ARTIST);
                    duration = Funcs.parseLong(tbl1.get(AudioMedia.DURATION));
                } else {
                    lastSetAlbum = media.album;
                    lastSetArtist = media.artist;
                    lastSetTitle = media.title;
                    duration = media.duration;
                    tbl1 = new Hashtable<String, String>();
                    tbl1.put(AudioMedia.ALBUM, media.album);
                    tbl1.put(AudioMedia.ARTIST, media.artist);
                    tbl1.put(AudioMedia.TITLE, media.title);
                    tbl1.put(AudioMedia.DURATION, Long.toString(media.duration));

                    tbl2 = new Hashtable<String, String>();
                    tbl2.put(AudioMedia.ALBUM, media.album);
                    tbl2.put(AudioMedia.ARTIST, media.artist);
                    bitmap = null;
                }

                mNotification = new Notification(R.drawable.status,
                        lastSetTitle, System.currentTimeMillis());
                mNotification.flags = Notification.FLAG_NO_CLEAR
                        | Notification.FLAG_ONGOING_EVENT;

                Intent intent = new Intent(this, MainActivity.class);

                PendingIntent contentIntent = PendingIntent.getActivity(this,
                        0, intent, Intent.FLAG_ACTIVITY_NEW_TASK);
                mNotification.setLatestEventInfo(this,
                        getString(R.string.app_name), lastSetTitle + "/"
                                + lastSetAlbum, contentIntent);
                if (Build.VERSION.SDK_INT >= 11) {
                    MediaPlayerHelper.setCustomizedNotify(this, mNotification,
                            tbl1, tbl2, bitmap);
                }

                // nm.notify(R.string.app_name, notification);
                startForegroundCompat(R.string.app_name, mNotification);

                updateWidgetTitle(lastSetArtist, lastSetTitle, lastSetAlbum,
                        stat(AppWidgetHelper.FLG_PLAY));

                startScrobbler(lastSetTitle, lastSetArtist, lastSetAlbum,
                        duration);

                // album, artist, title;
                Editor editor = pref.edit();
                editor.putString("lastSetTitle", lastSetTitle);
                editor.putString("lastSetArtist", lastSetArtist);
                editor.putString("lastSetAlbum", lastSetAlbum);

                editor.commit();
            }
        }
    }

    private void notifyOff(boolean force) {
        Logger.d("notify off");
        if (pref != null) {
            instantSave();
            boolean keepNotification = pref.getBoolean("key.keepNotification",
                    true);
            if (!force && keepNotification && Build.VERSION.SDK_INT >= 11
                    && mNotification != null
                    && mNotification.contentView != null) {
                mNotification.contentView.setInt(R.id.btnPlay,
                        "setBackgroundResource", R.drawable.widget_play);
                // startForegroundCompat(R.string.app_name, mNotification);
                setResumeStart(15, AppWidgetHelper.CALL_NOTIFYOFF);
            } else {
                if (Build.VERSION.SDK_INT > 7) {
                    if (audiofocus != null && jukebox.getList().size() > 0) {
                        audiofocus.unregister(this, audiomgr);
                    }
                }
                // nm.cancel(R.string.app_name);
                stopForegroundCompat(R.string.app_name);
            }

            String lastSetTitle = pref.getString("lastSetTitle", "");
            String lastSetArtist = pref.getString("lastSetArtist", "");
            String lastSetAlbum = pref.getString("lastSetAlbum", "");
            updateWidgetTitle(lastSetArtist, lastSetTitle, lastSetAlbum,
                    stat(AppWidgetHelper.FLG_STOP));

            stopScrobbler();
        }
    }

    private void setRepeatMode() {
        switch (jukebox.getRepeatMode()) {
            case SystemConsts.FLG_REPEAT_NO:
            case SystemConsts.FLG_REPEAT_ALL:
                if (mp != null) {
                    mp.setLooping(false);
                }
                break;
            case SystemConsts.FLG_REPEAT_ONCE:
                if (mp != null) {
                    mp.setLooping(true);
                }
                break;
        }
    }

    private void nfcShareNotifyOn() {
        Notification notification;
        notification = new Notification(android.R.drawable.ic_menu_share,
                getString(R.string.lbNfcshare_title),
                System.currentTimeMillis());
        notification.flags = Notification.FLAG_NO_CLEAR;

        Intent intent = new Intent(this, AlertDialogActivity.class);
        PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
                intent, Intent.FLAG_ACTIVITY_NEW_TASK);
        notification.setLatestEventInfo(this,
                getString(R.string.lbNfcshare_title),
                getString(R.string.lbNfcshare_summary), contentIntent);
        mNM.notify(NFCSHARE_RUNNING, notification);
    }

    private void nfcShareNotifyOff() {
        mNM.cancel(NFCSHARE_RUNNING);
    }

    BroadcastReceiver connectivityAction = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equalsIgnoreCase(
                    ConnectivityManager.CONNECTIVITY_ACTION)) {
                getContentResolver().query(MediaConsts.CLEAR_CONTENT_URI, null,
                        null, null, null);
                if (streamcacheserv != null) {
                    streamcacheserv.start();
                }
            }
        }
    };

    private boolean firstPluggd = true;
    BroadcastReceiver headsetplug = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equalsIgnoreCase(Intent.ACTION_HEADSET_PLUG)) {
                int state = intent.getIntExtra("state", 0);
                Logger.v("Intent.ACTION_HEADSET_PLUG state is " + state);
                // イヤホンを刺した
                if (state == 1 && firstPluggd) {
                    firstPluggd = false;
                    // AutoVolume
                    safeVolume();
                    firsttime = false;

                    // AutoReplay
                    boolean autoreplay = pref.getBoolean("key.useAutoPlay",
                            true);
                    if (autoreplay) {
                        if (mp != null && jukebox.getList().size() > 0
                                && !mp.isPlaying()) {
                            Toast.makeText(
                                    MediaPlayerService.this,
                                    getString(R.string.txt_action_headset_plug),
                                    Toast.LENGTH_SHORT).show();
                            play();
                        }
                    }
                } else if (state != 1) {
                    firstPluggd = true;
                }
            }
        }
    };

    BroadcastReceiver earPhoneChecker = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            // イヤホンを抜いた
            Toast.makeText(MediaPlayerService.this,
                    getString(R.string.txt_action_audio_becoming_noisy),
                    Toast.LENGTH_SHORT).show();
            pause();
        }
    };

    private PhoneStateListener telephone = new PhoneStateListener() {
        boolean onhook = false;
        RING_STATE callstaet;

        public void onCallStateChanged(int state, String number) {
            switch (state) {
                case TelephonyManager.CALL_STATE_RINGING: {
                    callstaet = RING_STATE.STATE_RINGING;
                    if (mp != null) {
                        if (mp.isPlaying()) {
                            pause();
                            onhook = true;
                            setResumeStop(CALL_RESUME);
                        }
                    }
                }
                    break;
                case TelephonyManager.CALL_STATE_OFFHOOK: {
                    if (callstaet == RING_STATE.STATE_RINGING) {
                        callstaet = RING_STATE.STATE_OFFHOOK;
                    } else {
                        callstaet = RING_STATE.STATE_NORMAL;
                        if (mp != null) {
                            if (mp.isPlaying()) {
                                pause();
                                onhook = true;
                                setResumeStop(CALL_RESUME);
                            }
                        }
                    }
                }
                    break;
                case TelephonyManager.CALL_STATE_IDLE: {
                    if (onhook) {
                        onhook = false;
                        setResumeStart(5, CALL_RESUME);
                    }
                    callstaet = RING_STATE.STATE_NORMAL;
                }
                    break;
                default: {

                }
            }
        }
    };

    BroadcastReceiver mScreenLock = null;

    private BroadcastReceiver createScreenLockReciever() {
        if (mScreenLock == null) {
            mScreenLock = new BroadcastReceiver() {
                @Override
                public void onReceive(Context context, Intent intent) {
                    if (Intent.ACTION_SCREEN_ON.equals(intent.getAction())) {
                        // if(Intent.ACTION_USER_PRESENT.equals(intent.getAction())){
                        boolean b = pref.getBoolean("key.useLockscreen", false);
                        if (b) {
                            if (mp != null && mp.isPlaying()) {
                                Intent i = new Intent(context,
                                        LockScreenActivity.class);
                                i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                                startActivity(i);
                            }
                        }
                    }
                }
            };
        }
        return mScreenLock;
    }

    /***
     * isFirstSetは複数の呼び出しがある場合において、最初のよびだしっていう意味のフラグ
     * 
     * @param isFirstSet
     */
    synchronized private void startPrefetch(boolean isFirstSet) {
        long cachesize = Funcs.getCachSize(pref);
        if (cachesize != 0) {
            if (isFirstSet) {
                MediaData next = jukebox.getNextPrefetch(1);
                if (next != null && !next.data.startsWith("http")) {
                    if (streamcacheserv != null) {
                        File cacheFile = StreamCacherServer.getCacheFile(this,
                                next.data);
                        if (cacheFile != null && !cacheFile.exists()) {
                            streamcacheserv.prefetch(next.id, next.data, true,
                                    this);
                            prefetchId = next.id;
                        }
                    }
                }
            } else {
                int length = Funcs
                        .getInt(pref.getString("key.cachecount", "3"));
                int size = jukebox.getList().size();
                boolean isAllcache = pref.getBoolean("key.useAllCache", true);
                if (isAllcache && length < size) {
                    boolean isWifi = NetworkHelper
                            .isWifiConnected(getApplicationContext());
                    if (isWifi) {
                        length = size;
                    }
                }
                for (int i = 0; i < length; i++) {
                    MediaData next = jukebox.getNextPrefetch(i + 1);
                    if (next != null) {
                        if (!next.data.startsWith("http")) {
                            if (streamcacheserv != null) {
                                File cacheFile = StreamCacherServer
                                        .getCacheFile(this, next.data);
                                if (cacheFile != null && !cacheFile.exists()) {
                                    streamcacheserv.prefetch(next.id,
                                            next.data, false, this);
                                    prefetchId = next.id;
                                    break;
                                }
                            } else {
                                break;
                            }
                        }
                    } else {
                        break;
                    }
                }
            }
        }
    }

    private void setDataSource(MediaData inf) throws IllegalArgumentException,
            SecurityException, IllegalStateException, IOException {
        if (streamcacheserv == null) {
            streamcacheserv = new StreamCacherServer(this, mHandler);
        }
        streamcacheserv.start();
        // URLなら直で再生、そうでないならローカル再生
        if (inf.data.startsWith("http")) {
            // NofityLoading
            notifyOn(false);
            mp.setDataSource(inf.data);
        } else {
            File cacheFile = StreamCacherServer.getCacheFile(this, inf.data);
            if (cacheFile != null) {
                if (cacheFile.exists()) {
                    cacheFile.setLastModified(System.currentTimeMillis());
                    mp.setDataSource(getContext(),
                            Uri.parse(cacheFile.toString()));
                    startPrefetch(true);
                    if (inf.id > 0) {
                        ContentValues values = new ContentValues();
                        values.put(AudioMedia.AUDIO_CACHE_FILE, cacheFile.getName());
                        getContentResolver().update(
                                ContentUris.withAppendedId(
                                        MediaConsts.MEDIA_CONTENT_URI, inf.id),
                                values, null, null);
                    }
                } else if (streamcacheserv.isUseCache()) {
                    // NofityLoading
                    notifyOn(false);
                    Logger.v("cache uri: " + inf.data);
                    String geturi = "http://127.0.0.1:"
                            + streamcacheserv.getPort() + "/"
                            + URLEncoder.encode(inf.data);
                    mp.setDataSource(geturi);
                    /*
                     * if (inf.id > 0) { ContentValues values = new
                     * ContentValues(); values.put(AudioMedia.AUDIO_CACHE_FLG,
                     * 0); getContentResolver().update(
                     * ContentUris.withAppendedId(
                     * MediaConsts.MEDIA_CONTENT_URI, inf.id), values, null,
                     * null); }
                     */
                } else {
                    // CacheServerを利用できない
                    String geturi = StreamCacherServer.getContentUri(this,
                            inf.data);
                    Logger.d("nocache url: " + geturi);
                    if (geturi != null) {
                        // NofityLoading
                        notifyOn(false);
                        if (geturi.startsWith("http")) {
                            mp.setDataSource(geturi);
                            /*
                             * if (inf.id > 0) { ContentValues values = new
                             * ContentValues();
                             * values.put(AudioMedia.AUDIO_CACHE_FLG, 0);
                             * getContentResolver().update(
                             * ContentUris.withAppendedId(
                             * MediaConsts.MEDIA_CONTENT_URI, inf.id), values,
                             * null, null); }
                             */
                        } else {
                            mp.setDataSource(getContext(), Uri.parse(geturi));
                        }
                    } else {
                        Logger.d("データがおかしい data=" + inf.data);
                        return;
                    }
                }
            } else {
                // ZIPファイルかどうかチェック
                if (inf.data.startsWith("zip://")) {
                    Logger.v("cache uri: " + inf.data);
                    String geturi = "http://127.0.0.1:"
                            + streamcacheserv.getPort() + "/"
                            + URLEncoder.encode(inf.data);
                    mp.setDataSource(geturi);
                } else {
                    mp.setDataSource(getContext(), Uri.parse(inf.data));
                }
            }
        }
        mp.prepareAsync();

        // SessionKeepAlive
        setKeepSession(-1);
    }

    @Override
    public IBinder onBind(Intent intent) {
        return bind;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Logger.d("MediaPlayerSerivce.onCreate");
        jukebox = new JukeBox(this);
        mNM = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        try {
            mStartForeground = getClass().getMethod("startForeground",
                    mStartForegroundSignature);
            mStopForeground = getClass().getMethod("stopForeground",
                    mStopForegroundSignature);
            return;
        } catch (NoSuchMethodException e) {
            // Running on an older platform.
            mStartForeground = mStopForeground = null;
        }
        try {
            mSetForeground = getClass().getMethod("setForeground",
                    mSetForegroundSignature);
        } catch (NoSuchMethodException e) {
            throw new IllegalStateException(
                    "OS doesn't have Service.startForeground OR Service.setForeground!");
        }

        mWifiLock = ((WifiManager) getSystemService(Context.WIFI_SERVICE))
                .createWifiLock(WifiManager.WIFI_MODE_FULL, "mylock");
        lockWifi();

        if (Build.VERSION.SDK_INT < 8) {
        } else {
            if (Build.VERSION.SDK_INT > 7) {
                new SSLHelper().disableSSLVerifier();
            }
        }
    }

    private synchronized void initService() {
        if (mp != null) {
            return;
        }
        pref = PreferenceManager.getDefaultSharedPreferences(this);

        soundmanager.init(this);

        // MediaPlayer生成
        if (mp == null) {
            mp = new MediaPlayer();
            mp.setOnPreparedListener(this);
            mp.setOnCompletionListener(this);
            mp.setOnBufferingUpdateListener(this);
            mp.setLooping(false);
            mp.setAudioStreamType(AudioManager.STREAM_MUSIC);
            prepared = false;
            mp.setOnErrorListener(this);
        }
        if (mp != null && audiofx == null) {
            if (Build.VERSION.SDK_INT > 8) {
                try {
                    audiofx = new AudioFx(this,
                            MediaPlayerHelper.getAudioSessionId(mp));
                    audiofx.load();
                    audiofx.attach(mp);
                } catch (Exception e) {
                    audiofx = null;
                }
            }
        }

        // Ampache Session Keep
        // if (!ContentsUtils.isSDCard(pref)) {
        SessionKeepTask task = new SessionKeepTask();
        task.execute((Void[]) null);
        // }

        // load playlist
        if (jukebox.isLoaded() != true) {
            jukebox.load();
        }

        if (streamcacheserv == null) {
            try {
                streamcacheserv = new StreamCacherServer(
                        getApplicationContext(), mHandler);
                streamcacheserv.start();
            } catch (UnknownHostException e) {
                Logger.e("StreamCacherServer.UnknownHostException", e);
                streamcacheserv = null;
            } catch (IOException e) {
                Logger.e("StreamCacherServer.IOException", e);
                streamcacheserv = null;
            }
        }

        registerReceiver(headsetplug, new IntentFilter(
                Intent.ACTION_HEADSET_PLUG));
        registerReceiver(earPhoneChecker, new IntentFilter(
                AudioManager.ACTION_AUDIO_BECOMING_NOISY));
        if (Build.VERSION.SDK_INT < 14) {
            registerReceiver(createScreenLockReciever(), new IntentFilter(
                    Intent.ACTION_SCREEN_ON));
        }

        registerReceiver(connectivityAction, new IntentFilter(
                ConnectivityManager.CONNECTIVITY_ACTION));

        tm = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
        tm.listen(telephone, PhoneStateListener.LISTEN_CALL_STATE);

        audiomgr = (AudioManager) getSystemService(Context.AUDIO_SERVICE);

        // BlueTooth
        if (Build.VERSION.SDK_INT > 7) {
            audiofocus = new AudioFocus(this, audiomgr);
        } else {

        }

        // ステータスリセット
        Editor editor = pref.edit();
        editor.putBoolean("earPhoneChecker", false);
        editor.commit();

        setRepeatStart(30, CALL_KEEPALIVE);

        // 初期化サービス
        Intent service = new Intent(this, SystemInitService.class);
        startService(service);
    }

    private void instantSave() {
        if (pref != null) {
            if (audiofx != null) {
                try {
                    audiofx.save();
                } catch (Exception e) {
                }
            }

            if (jukebox.isLoaded()) {
                jukebox.save();
            }

        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Logger.d("MediaPlayerSerivce.onDestroy");
        freeWifi();

        if (mp != null) {
            mp.release();
            mp = null;
        }
        if (audiofx != null) {
            try {
                audiofx.release();
            } catch (Exception e) {
                audiofx = null;
            }
        }
        if (streamcacheserv != null) {
            streamcacheserv.release();
            streamcacheserv.stop();
            streamcacheserv = null;
        }

        // nm.cancel(R.string.app_name);
        stopForegroundCompat(R.string.app_name);

        if (pref != null) {
            setResumeStop(CALL_KEEPSESSION);
            setResumeStop(CALL_KEEPALIVE);
            setResumeStop(AppWidgetHelper.CALL_STOP);
            setResumeStop(CALL_RESUME);

            if (soundmanager != null) {
                soundmanager.release();
            }

            if (headsetplug != null) {
                unregisterReceiver(headsetplug);
            }
            if (earPhoneChecker != null) {
                unregisterReceiver(earPhoneChecker);
            }
            if (mScreenLock != null) {
                unregisterReceiver(mScreenLock);
            }

            if (connectivityAction != null) {
                unregisterReceiver(connectivityAction);
            }

            if (audiofocus != null) {
                audiofocus.unregister(audiomgr);
                audiofocus = null;
            }

            if (tm != null) {
                tm.listen(telephone, PhoneStateListener.LISTEN_NONE);
            }
        }
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if (intent != null) {
            // PendinIntentもしくはBroadcastによってStartされた
            Logger.d("onStart" + intent.getAction());
            if (CALL_KEEPALIVE.equals(intent.getAction())) {
                // なにもしない（サービスの生存期間を伸ばすためのアクション）
                return START_NOT_STICKY;
            } else if (CALL_KEEPSESSION.equals(intent.getAction())) {
                // KeepSession 継続するなら更新処理を呼んであげる
                SessionKeepTask task = new SessionKeepTask();
                task.execute((Void[]) null);
                return START_NOT_STICKY;
            } else {
                if (CALL_RESUME.equals(intent.getAction())) {
                    initService();
                    setResumeStop(AppWidgetHelper.CALL_STOP);
                    play();
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_STOP.equals(intent.getAction())) {
                    Logger.d("MediaPlayerSerivce.CALL_STOP");
                    if (mp == null || !mp.isPlaying()) {
                        Logger.d("mp=" + mp);
                        stopSelf();
                    }
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_FF.equals(intent.getAction())) {
                    initService();
                    setResumeStop(AppWidgetHelper.CALL_STOP);
                    if (mp != null) {
                        ff();
                    }
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_REW.equals(intent.getAction())) {
                    initService();
                    setResumeStop(AppWidgetHelper.CALL_STOP);
                    if (mp != null) {
                        rew();
                    }
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_SHUFFLE.equals(intent
                        .getAction())) {
                    if (!jukebox.isLoaded()) {
                        jukebox.load();
                    }
                    shuffle();
                    if (mp == null) {
                        stopSelf();
                    }
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_LOOP.equals(intent.getAction())) {
                    if (!jukebox.isLoaded()) {
                        jukebox.load();
                    }
                    repeat();
                    if (mp == null) {
                        stopSelf();
                    }
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_NOTIFYOFF.equals(intent
                        .getAction())) {
                    notifyOff(true);
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_PLAY_PAUSE.equals(intent
                        .getAction())) {
                    Logger.d("MediaPlayerSerivce.CALL_PLAY_PAUSE");
                    initService();
                    int stat = stat(0);
                    if ((stat & AppWidgetHelper.FLG_PLAY) > 0) {
                        pause();
                        setResumeStart(0, AppWidgetHelper.CALL_STOP);
                    } else {
                        play();
                        setResumeStop(AppWidgetHelper.CALL_STOP);
                    }
                    return START_NOT_STICKY;
                } else if (AppWidgetHelper.CALL_UPDATEWIDGET.equals(intent
                        .getAction())) {
                    Logger.d("call CALL_UPDATEWIDGET");
                    if (!jukebox.isLoaded()) {
                        jukebox.load();
                    }
                    SharedPreferences preference = PreferenceManager
                            .getDefaultSharedPreferences(this);
                    String title = preference.getString("lastSetTitle", "");
                    String artist = preference.getString("lastSetArtist", "");
                    String album = preference.getString("lastSetAlbum", "");
                    updateWidgetTitle(artist, title, album, stat(0));
                    if (mp == null) {
                        stopSelf();
                    }
                    return START_NOT_STICKY;
                } else if (SystemConsts.CALL_NFC_SHARE.equals(intent
                        .getAction())) {
                    // NFC共有中を通知
                    nfcShareNotifyOn();
                }
            }
        }
        if (!isInitService || mp == null) {
            isInitService = true;
            initService();
        }
        setResumeStop(AppWidgetHelper.CALL_STOP);
        return START_NOT_STICKY;
    }

    @Override
    public void onBufferingUpdate(MediaPlayer mp, int percent) {
        // 読み込みバッファが変わった場合に通知される。プログレスバーの更新に使用
        progress(percent);
        // if(jukebox.getSeekPosition()>0){
        jukebox.setPlayPosition(mp.getCurrentPosition());
        // }
    }

    @Override
    public void onCompletion(MediaPlayer mp) {
        Logger.d("onCompletion");
        // 曲の再生が終わった
        // 曲の再生が終わったときに通知される。次の曲の読み込みに使用
        stopScrobbler();
        if (hasError() != true) {
            int position = jukebox.getPosition();
            if (retryConnect || jukebox.moveNext()) {
                int pos = jukebox.getPosition();
                if (retryConnect != true && pos != -1 && pos == position) {
                    prepared = true;
                    mp.seekTo(0);
                    if (!mp.isPlaying()) {
                        mp.start();
                    }
                } else {
                    retryConnect = false;
                    prepared = false;

                    mp.stop();
                    mp.reset();

                    if (pos != -1) {
                        MediaData inf = jukebox.getCurrent();
                        try {
                            jukebox.setPlayPosition(0);
                            setDataSource(inf);
                        } catch (Exception e) {
                            Logger.e("play", e);
                        }
                    }
                    updateView();
                }
            } else {
                try {
                    saveVolume();
                    notifyOff(false);
                    // 曲の再生終了
                    boolean useendoflist = pref.getBoolean("key.useEndOfList",
                            true);
                    if (useendoflist && prepared) {
                        Thread.sleep(500);
                        soundmanager.play(R.raw.cassette_off, 1.0f);
                    }
                } catch (InterruptedException e) {
                } finally {
                    prepared = false;
                }
            }
        }
        updateView();
    }

    @Override
    public void onPrepared(MediaPlayer mp) {
        Logger.d("onPrepared");
        // 曲のバッファリングが終わったときに通知される。曲の再生開始に使用
        prepared = true;
        mp.start();
        int msec = jukebox.getSeekPosition();
        if (msec > 0) {
            jukebox.setSeekPosition(0);
            mp.seekTo(msec);
        }
        MediaData inf = jukebox.getCurrent();
        if (inf != null && inf.duration == 0) {
            inf.duration = mp.getDuration();
        }
        setRepeatMode();
        notifyOn(true);
        updateView();
    }

    public void progress(int percent) {
        // 再描画を通知する
        synchronized (callbackList) {
            int n = callbackList.beginBroadcast();
            for (int i = 0; i < n; i++) {
                try {
                    callbackList.getBroadcastItem(i).onBufferingUpdate(percent);
                } catch (RemoteException e) {
                }
            }
            callbackList.finishBroadcast();
        }
    }

    public void updateView() {
        if (!updatelock) {
            // 再描画を通知する
            boolean flg = updatelist;
            updatelist = false;
            synchronized (callbackList) {
                int n = callbackList.beginBroadcast();
                for (int i = 0; i < n; i++) {
                    Logger.d("callbackList.getBroadcastItem(i).updateView i="
                            + i);
                    try {
                        callbackList.getBroadcastItem(i).updateView(flg);
                    } catch (RemoteException e) {
                        Logger.e("updateView", e);
                    }
                }
                callbackList.finishBroadcast();
            }
        }

        SharedPreferences preference = PreferenceManager
                .getDefaultSharedPreferences(this);
        String title = preference.getString("lastSetTitle", "");
        String artist = preference.getString("lastSetArtist", "");
        String album = preference.getString("lastSetAlbum", "");
        updateWidgetTitle(artist, title, album, stat(0));
    }

    private void updateWidgetTitle(String artist, String title, String album,
            int stat) {
        Intent intent = new Intent(AppWidgetHelper.CALL_APPWIDGET_DISPLAY);
        intent.putExtra("artist", artist);
        intent.putExtra("title", title);
        intent.putExtra("album", album);
        intent.putExtra("stat", stat);
        sendBroadcast(intent);
    }

    private Context getContext() {
        return this;
    }

    private long getDuration() {
        MediaData inf = jukebox.getCurrent();
        if (inf != null) {
            return inf.duration;
        } else {
            return 0;
        }
    }

    public void safeVolume() {
        if (audiomgr != null && pref != null) {
            boolean b = pref.getBoolean(SystemConsts.KEY_AUTO_VOLUMEAJUST, true);
            if (b) {
                int volume = pref.getInt(SystemConsts.KEY_FIRST_VOLUME, 2);
                if (volume != 0) {
                    audiomgr.setStreamVolume(AudioManager.STREAM_MUSIC, volume,
                            AudioManager.FLAG_SHOW_UI);
                }
            }
        }
    }

    public void saveVolume() {
        if (audiomgr != null && pref != null) {
            if (HeadsetHelper.isWiredHeadsetOn(audiomgr)) {
                int volume = audiomgr.getStreamVolume(AudioManager.STREAM_MUSIC);
                Editor editor = pref.edit();
                editor.putInt(SystemConsts.KEY_FIRST_VOLUME, volume);
                editor.commit();
            }
        }
    }

    int stat(int forceState) {
        // Logger.d("stat");
        int shf = jukebox.getShaffle() != null ? AppWidgetHelper.FLG_SHUFFLE
                : 0;
        int next = 0;
        int back = 0;
        int list = 0;
        int repeat = jukebox.getRepeatMode() == SystemConsts.FLG_REPEAT_ONCE ? AppWidgetHelper.FLG_LOOP1
                : jukebox.getRepeatMode() == SystemConsts.FLG_REPEAT_ALL ? AppWidgetHelper.FLG_LOOP2
                        : SystemConsts.FLG_REPEAT_NO;
        list = jukebox.getList().size() > 0 ? AppWidgetHelper.FLG_HASLIST : 0;
        if (list != 0) {
            next = jukebox.hasNext() ? AppWidgetHelper.FLG_HASNEXT : 0;
            back = jukebox.getPosition() >= 0 ? AppWidgetHelper.FLG_HASFOWD : 0;
        }

        if (forceState == 0) {
            if (mp != null && mp.isPlaying()) {
                return AppWidgetHelper.FLG_PLAY | next | back | repeat | shf
                        | list;
            } else {
                if (list != 0) {
                    return AppWidgetHelper.FLG_PAUSE | next | back | repeat
                            | shf | list;
                } else {
                    return AppWidgetHelper.FLG_STOP | next | back | repeat
                            | shf;
                }
            }
        } else {
            return forceState | next | back | repeat | shf | list;
        }
    }

    private void play() {
        Logger.d("play");
        setError(false);
        retryConnect = false;
        updatelock = false;

        if (mp == null) {
            isInitService = true;
            initService();
            if (mp == null) {
                return;
            }
        }

        if (firsttime) {
            if (HeadsetHelper.isWiredHeadsetOn(audiomgr)) {
                safeVolume();
                firsttime = false;
            }
        }

        if (prepared) {
            mp.start();
            notifyOn(true);
            updateView();
        } else {
            MediaData inf = jukebox.getCurrent();
            if (inf != null) {
                try {
                    mp.reset();
                    int playposition = jukebox.getPlayPosition();
                    if (playposition > 0) {
                        jukebox.setSeekPosition(playposition);
                    }
                    setDataSource(inf);
                    progress(playposition);
                } catch (Exception e) {
                    Logger.e("play", e);
                }
            }
        }
    }

    private void startScrobbler(String title, String artist, String album,
            long duration) {
        if (pref != null) {
            boolean b = pref.getBoolean("key.useLastFM", false);
            if (b) {
                try {
                    Intent i = new Intent(META_CHANGED);
                    i.putExtra("artist", artist);
                    i.putExtra("track", title);
                    if (album != null)
                        i.putExtra("album", album);
                    i.putExtra("duration", duration);
                    sendBroadcast(i);
                } catch (Exception e) {
                    Logger.e("startScrobbler:" + META_CHANGED, e);
                    Editor editor = pref.edit();
                    editor.putBoolean("key.useLastFM", false);
                    editor.commit();
                }
            }
        }
    }

    private void stopScrobbler() {
        if (pref != null) {
            boolean b = pref.getBoolean("key.useLastFM", false);
            if (b) {
                try {
                    Intent i = new Intent(PLAYBACK_FINISHED);
                    sendBroadcast(i);
                } catch (Exception e) {
                    Logger.e("stopScrobbler:" + PLAYBACK_FINISHED, e);
                    Editor editor = pref.edit();
                    editor.putBoolean("key.useLastFM", false);
                    editor.commit();
                }
            }
        }
    }

    private void pauseScrobbler() {
        Intent i = new Intent(META_CHANGED);
        // i.putExtra("artist", artist);
        // i.putExtra("track", title);
        // if(album!=null)
        // i.putExtra("album", album);
        i.putExtra("duration", getDuration());
        sendBroadcast(i);
    }

    private void pause() {
        Logger.d("pause");
        if (mp != null) {
            int pos = mp.getCurrentPosition();
            jukebox.setPlayPosition(pos);
            saveVolume();
            mp.pause();
            notifyOff(false);
            updateView();
        }
    }

    private void ff() {
        Logger.d("ff");
        if (mp == null) {
            isInitService = true;
            initService();
            if (mp == null) {
                return;
            }
        }

        int position = jukebox.getPosition();
        if (jukebox.moveNext() != true) {
            return;
        }
        int pos = jukebox.getPosition();
        if (pos != -1 && pos == position) {
            mp.seekTo(0);
        } else {
            prepared = false;

            if (mp.isPlaying()) {
                stopScrobbler();
            }

            mp.stop();
            mp.reset();

            if (pos != -1) {
                MediaData inf = jukebox.getCurrent();
                try {
                    jukebox.setPlayPosition(0);
                    setDataSource(inf);
                } catch (Exception e) {
                    Logger.e("play", e);
                }
            }
            updateView();
        }
    }

    private void rew() {
        if (mp == null) {
            isInitService = true;
            initService();
            if (mp == null) {
                return;
            }
        }
        if (jukebox.getList().size() == 0) {
            return;
        }
        int position = jukebox.getPosition();
        int current = mp != null ? mp.getCurrentPosition() : 0;
        int duration = mp != null ? mp.getDuration() : 0;
        int pos = jukebox.getBack(current, duration);
        if (pos == position) {
            mp.seekTo(0);
        } else {
            prepared = false;

            if (mp.isPlaying()) {
                stopScrobbler();
            }
            mp.stop();
            mp.reset();
            jukebox.movePosition(pos);
            MediaData inf = jukebox.getCurrent();
            try {
                jukebox.setPlayPosition(0);
                setDataSource(inf);
            } catch (Exception e) {
                Logger.e("play", e);
            }
        }
        updateView();
    }

    public void stop() {
        Logger.d("stop");
        if (mp != null) {
            if (mp.isPlaying()) {
                stopScrobbler();
            }
            saveVolume();
            mp.stop();
            mp.reset();
            prepared = false;
            notifyOff(false);
            updateView();
        } else {
            prepared = false;
            notifyOff(false);
            updateView();
        }
    }

    private void setRepeatStart(int delay, String action) {
        Intent intent = new Intent(this, MediaPlayerService.class);
        intent.setAction(action);
        PendingIntent operation = PendingIntent.getService(this, 0, intent,
                PendingIntent.FLAG_UPDATE_CURRENT);

        AlarmManager am = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        am.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP,
                SystemClock.elapsedRealtime(), delay * 60 * 1000, operation);
    }

    private void setResumeStart(int delay, String action) {
        Intent intent = new Intent(this, MediaPlayerService.class);
        intent.setAction(action);
        PendingIntent operation = PendingIntent.getService(this, 0, intent,
                PendingIntent.FLAG_UPDATE_CURRENT);

        Calendar cal = Calendar.getInstance();
        if (delay > 0) {
            cal.add(Calendar.SECOND, delay);
        }

        AlarmManager am = (AlarmManager) this
                .getSystemService(Activity.ALARM_SERVICE);
        am.set(AlarmManager.RTC_WAKEUP, cal.getTimeInMillis(), operation);
    }

    private void setResumeStop(String action) {
        Intent intent = new Intent(this, MediaPlayerService.class);
        intent.setAction(action);
        PendingIntent operation = PendingIntent.getService(this, 0, intent,
                PendingIntent.FLAG_UPDATE_CURRENT);
        AlarmManager am = (AlarmManager) this
                .getSystemService(Activity.ALARM_SERVICE);
        am.cancel(operation);
    }

    @Override
    public boolean onError(MediaPlayer mp, int what, int extra) {
        Logger.d("MediaPlayer.onError--->   what:" + what + "    extra:"
                + extra);
        // MediaPlayer Error things: http://android.joao.jp/
        if (extra == -1002) { // -1002:
                              // HTTPStream::connect(pd.vog.sprintpcs.com, 8085)
                              // failed. ERROR_UNKNOWN_HOST
            Toast.makeText(getContext(),
                    getString(R.string.txt_error_disconnect),
                    Toast.LENGTH_SHORT).show();
            setError(true);
        } else if (extra == -1004 && what == 1) { // セッション切れ
            Toast.makeText(getContext(),
                    getString(R.string.txt_error_disconnect),
                    Toast.LENGTH_SHORT).show();
            Cursor cur = null;
            try {
                cur = getContentResolver().query(MediaConsts.PING_CONTENT_URI,
                        null, "force", null, null);
                if (cur != null && cur.moveToFirst()) {
                    int col = cur.getColumnIndex(Auth.AUTH_KEY);
                    if (col != -1) {
                        String auth = cur.getString(col);
                        if (auth != null && auth.length() > 0) {
                            retryConnect = true;
                            return false;
                        }
                    }
                }
                setError(true);
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        } else if (extra == -1 && what == 1) { // ?
            Toast.makeText(getContext(),
                    getString(R.string.txt_error_disconnect),
                    Toast.LENGTH_SHORT).show();
            setError(true);
        } else if (what == 1) {
            // helper.clearAuth();
            // setError(true);
        }
        return false;
    }

    private synchronized boolean hasError() {
        return error;
    }

    private synchronized void setError(boolean b) {
        error = b;
    }

    private void setKeepSession(long expired) {
        // Serverを使うならセッションをKeepする
        // if (!ContentsUtils.isSDCard(pref)) {
        Intent intent = new Intent(getContext(), MediaPlayerService.class);
        intent.setAction(MediaPlayerService.CALL_KEEPSESSION);
        PendingIntent operation = PendingIntent.getService(getContext(), 0,
                intent, PendingIntent.FLAG_UPDATE_CURRENT);

        long nextime = 0;
        if (expired > 0) {
            Calendar cal = Calendar.getInstance();
            cal.setTimeInMillis(expired);
            cal.add(Calendar.SECOND, 60);
            nextime = cal.getTimeInMillis();
        } else {
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.SECOND, KEEP_DELAY);
            nextime = cal.getTimeInMillis();
        }

        AlarmManager am = (AlarmManager) getContext().getSystemService(
                Activity.ALARM_SERVICE);
        am.set(AlarmManager.RTC_WAKEUP, nextime, operation);
        // }
    }

    class SessionKeepTask extends AsyncTask<Void, Void, Void> {

        @Override
        protected Void doInBackground(Void... params) {
            if (pref != null && !ContentsUtils.isSDCard(pref)) {
                getContentResolver().query(MediaConsts.PING_CONTENT_URI, null,
                        null, null, null);
            }

            if (mp != null) {
                setKeepSession(-1);
            }
            return null;
        }

    }

    @Override
    public void startProgress(long max) {
        synchronized (callbackList) {
            int n = callbackList.beginBroadcast();
            for (int i = 0; i < n; i++) {
                try {
                    callbackList.getBroadcastItem(i).startProgress(max);
                } catch (RemoteException e) {
                }
            }
            callbackList.finishBroadcast();
        }
    }

    @Override
    public void stopProgress() {
        synchronized (callbackList) {
            int n = callbackList.beginBroadcast();
            for (int i = 0; i < n; i++) {
                try {
                    callbackList.getBroadcastItem(i).stopProgress();
                } catch (RemoteException e) {
                }
            }
            callbackList.finishBroadcast();
        }
    }

    @Override
    public void progress(long pos, long max) {
        synchronized (callbackList) {
            int n = callbackList.beginBroadcast();
            for (int i = 0; i < n; i++) {
                try {
                    callbackList.getBroadcastItem(i).progress(pos, max);
                } catch (RemoteException e) {
                }
            }
            callbackList.finishBroadcast();
        }
    }

    void freeWifi() {
        if (mWifiLock != null && mWifiLock.isHeld()) {
            mWifiLock.release();
        }
    }

    void lockWifi() {
        if (mWifiLock != null) {
            mWifiLock.acquire();
        }
    }
}
