package jp.co.kayo.android.localplayer.ui;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Hashtable;

import jp.co.kayo.android.localplayer.BaseActivity;
import jp.co.kayo.android.localplayer.BaseListFragment;
import jp.co.kayo.android.localplayer.PlayActivity;
import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.SubListActivity;
import jp.co.kayo.android.localplayer.TagEditActivity;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioGenresMember;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.core.ForceSizeChangeListView;
import jp.co.kayo.android.localplayer.core.ContextMenuFragment;
import jp.co.kayo.android.localplayer.core.ServiceBinderHolder;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.provider.DownloadHelper;
import jp.co.kayo.android.localplayer.service.IMediaPlayerService;
import jp.co.kayo.android.localplayer.ui.dialog.LyricsDialog;
import jp.co.kayo.android.localplayer.ui.dialog.RatingDialog;
import jp.co.kayo.android.localplayer.ui.dialog.YoutubeSearchDialog;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.ViewCache;
import android.support.v4.app.LoaderManager.LoaderCallbacks;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.support.v4.content.CursorLoader;
import android.content.Intent;
import android.support.v4.content.Loader;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.database.MatrixCursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;

import android.view.ActionMode;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ListView;
import android.widget.Toast;

public class GenresListFragment extends BaseListFragment implements
        ContentManager, OnItemLongClickListener, ContextMenuFragment,
        LoaderCallbacks<Cursor>, OnScrollListener, IProgressView {
    private String genres_id;
    private ViewCache viewcache;
    SharedPreferences mPref;
    private GenresListAdapter mAdapter;
    String genrestitle;
    boolean getall = false;
    MatrixCursor mcur;
    long lastQueryTime = 0;
    // int limit;
    private ActionMode mMode;
    boolean isReadFooter;
    Runnable runner = null;

    private String getAudioIdCol(Cursor cursor) {
        String ret = MediaConsts.AudioGenresMember._ID;
        int col = cursor.getColumnIndex(MediaConsts.AudioGenresMember.AUDIO_ID);
        if (col != -1) {
            ret = MediaConsts.AudioGenresMember.AUDIO_ID;
        }
        return ret;
    }

    View mFooter = null;

    private View getFooter() {
        if (mFooter == null) {
            mFooter = getActivity().getLayoutInflater().inflate(
                    R.layout.listview_footer, null);
            mFooter.findViewById(R.id.progress1).setVisibility(View.GONE);
        }
        return mFooter;
    }

    private void showFooter() {
        getFooter().findViewById(R.id.progress1).setVisibility(View.VISIBLE);
    }

    private void hideFooter() {
        getFooter().findViewById(R.id.progress1).setVisibility(View.GONE);
    }

    private void invisibleFooter() {
        getall = true;
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                hideFooter();
            }
        });
    }

    private IMediaPlayerService getBinder() {
        if (getActivity() instanceof ServiceBinderHolder) {
            return ((ServiceBinderHolder) getActivity()).getBinder();
        } else {
            return null;
        }
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        getLoaderManager().initLoader(getLoaderID(), null, this);
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Bundle args = getArguments();
        if (args != null) {
            CharSequence id = args.getCharSequence("selectedkey");
            if (id != null) {
                genres_id = id.toString();
            }
            CharSequence title = args.getCharSequence("selectedtitle");
            if (title != null) {
                genrestitle = title.toString();
            }
        }
        if (savedInstanceState != null) {
            getall = savedInstanceState.getBoolean("getall");
        }
        mPref = PreferenceManager.getDefaultSharedPreferences(getActivity());
        viewcache = (ViewCache) getFragmentManager().findFragmentByTag(
                SystemConsts.TAG_CACHE);
        // limit = Integer.parseInt(mPref.getString(SystemConsts.PREF_LIMIT,
        // SystemConsts.DEFAULT_LIST_LIMIT));
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putBoolean("getall", getall);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.genres_list_view, container,
                false);

        ForceSizeChangeListView list = (ForceSizeChangeListView) root
                .findViewById(android.R.id.list);
        list.setOnItemLongClickListener(this);
        list.setOnScrollListener(this);
        list.addFooterView(getFooter(), null, false);
        isReadFooter = ContentsUtils.isNoCacheAction(mPref);

        return root;
    }

    private int getLoaderID() {
        return R.layout.listview_body;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        getLoaderManager().destroyLoader(getLoaderID());
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> arg0, View view,
            int position, long arg3) {
        Cursor selectedCursor = (Cursor) mAdapter.getItem(position);
        if (selectedCursor != null) {
            BaseActivity act = (BaseActivity) getActivity();
            mMode = act.startActionMode(new AnActionModeOfEpicProportions(
                    getActivity(), position, mHandler));
            mMode.setTitle(selectedCursor.getString(selectedCursor
                    .getColumnIndex(AudioMedia.TITLE)));
            mMode.setSubtitle(getString(R.string.lb_tab_media));
            return true;
        }

        return false;
    }

    private final class AnActionModeOfEpicProportions extends
            MediaContentActionCallback {

        public AnActionModeOfEpicProportions(Context context,
                int selectedIndex, Handler handler) {
            super(context, selectedIndex, handler);
        }

        @Override
        public void onDestroyActionMode(ActionMode mode) {
            mMode = null;
        }

        @Override
        public boolean onCreateActionMode(ActionMode mode, Menu menu) {

            /*
             * menu.add(getString(R.string.sub_mnu_play)).setIcon(R.drawable.
             * ic_menu_play_clip)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
             */

            menu.add(getString(R.string.sub_mnu_order))
                    .setIcon(R.drawable.ic_menu_btn_add)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.sub_mnu_artist))
                    .setIcon(R.drawable.ic_menu_btn_artist)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.sub_mnu_album))
                    .setIcon(R.drawable.ic_menu_album)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.sub_mnu_rating))
                    .setIcon(R.drawable.ic_menu_star)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.txt_web_more))
                    .setIcon(R.drawable.ic_menu_wikipedia)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            // 文字化け修正メニューを追加
            // if (ContentsUtils.isSDCard(mPref)) {
            menu.add(getString(R.string.sub_mnu_edit))
                    .setIcon(R.drawable.ic_menu_strenc)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
            // }

            /*
             * menu.add(getString(R.string.txt_web_wikipedia)).setIcon(R.drawable
             * .ic_menu_wikipedia)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
             * 
             * menu.add(getString(R.string.txt_web_lyrics)).setIcon(R.drawable.
             * ic_menu_lyrics)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
             */

            boolean b = mPref.getBoolean("key.useLastFM", false);
            if (b) {
                menu.add(getString(R.string.sub_mnu_love)).setShowAsAction(
                        MenuItem.SHOW_AS_ACTION_IF_ROOM);
                menu.add(getString(R.string.sub_mnu_ban)).setShowAsAction(
                        MenuItem.SHOW_AS_ACTION_IF_ROOM);
            }

            /*
             * if (ContentsUtils.canDownload(mPref)) {
             * menu.add(getString(R.string
             * .sub_mnu_download)).setIcon(R.drawable.ic_menu_download)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM); }
             */

            if (!ContentsUtils.isSDCard(mPref)) {
                menu.add(getString(R.string.sub_mnu_clearcache))
                        .setIcon(R.drawable.ic_menu_refresh)
                        .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
            }
            return true;
        }

    };

    @Override
    protected void messageHandle(int what, int selectedPosition) {
        Cursor selectedCursor = (Cursor) mAdapter.getItem(selectedPosition);
        if (selectedCursor != null) {
            long mediaId = selectedCursor.getLong(selectedCursor
                    .getColumnIndex(AudioMedia._ID));
            String title = selectedCursor.getString(selectedCursor
                    .getColumnIndex(MediaConsts.AudioMedia.TITLE));
            String album = selectedCursor.getString(selectedCursor
                    .getColumnIndex(MediaConsts.AudioMedia.ALBUM));
            String artist = selectedCursor.getString(selectedCursor
                    .getColumnIndex(MediaConsts.AudioMedia.ARTIST));
            String data = selectedCursor.getString(selectedCursor
                    .getColumnIndex(MediaConsts.AudioMedia.DATA));
            long duration = selectedCursor.getLong(selectedCursor
                    .getColumnIndex(MediaConsts.AudioMedia.DURATION));

            switch (what) {
            case SystemConsts.EVT_SELECT_RATING: {
                RatingDialog dlg = new RatingDialog(getActivity(),
                        TableConsts.FAVORITE_TYPE_SONG, mediaId, mHandler);
                dlg.show();
            }
                break;
            case SystemConsts.EVT_SELECT_PLAY: {
                ContentsUtils.playMedia(getBinder(), mediaId, duration, data);
                Toast.makeText(getActivity(),
                        getString(R.string.txt_action_addartist),
                        Toast.LENGTH_SHORT).show();
            }
                break;
            case SystemConsts.EVT_SELECT_ARTIST: {
                Intent intent = new Intent(getActivity(), SubListActivity.class);
                intent.putExtra(SystemConsts.KEY_LIST_SUBTYPE,
                        SystemConsts.PRM_ARTISTLIST);
                intent.putExtra(SystemConsts.KEY_LIST_TITLE, artist);
                intent.putExtra(SystemConsts.KEY_LIST_MAX, -1);
                // intent.putExtra(SystemConsts.KEY_LIST_KEY, artist_key);
                startActivity(intent);
            }
                break;
            case SystemConsts.EVT_SELECT_ADD: {
                ContentsUtils.addOrderMediaList(getBinder(), mediaId, duration,
                        data);

                Toast.makeText(getActivity(),
                        getString(R.string.txt_action_addsong),
                        Toast.LENGTH_SHORT).show();
            }
                break;
            case SystemConsts.EVT_SELECT_OPENALBUM: {
                // TODO OpenAlbum未実装
                Hashtable<String, String> tbl = ContentsUtils.getMedia(
                        getActivity(), new String[] { AudioMedia.ALBUM_KEY },
                        mediaId);
                String album_key = tbl.get(AudioMedia.ALBUM_KEY);
                if (album_key != null) {
                    Intent intent = new Intent(getActivity(),
                            PlayActivity.class);
                    intent.putExtra(SystemConsts.KEY_PLAY_LOCK, true);
                    intent.putExtra(SystemConsts.KEY_PLAY_ALBUMKEY, album_key);
                    getActivity().startActivity(intent);
                }
            }
                break;
            case SystemConsts.EVT_SELECT_MORE: {
                showInfoDialog(album, artist, title);
            }
                break;
            case SystemConsts.EVT_SELECT_DOWNLOAD: {
                DownloadHelper helper = new DownloadHelper(getActivity());
                try {
                    if (helper.enqueueMusic(mediaId, title, data) != -1) {
                        Toast.makeText(getActivity(),
                                getString(R.string.txt_action_download),
                                Toast.LENGTH_SHORT).show();
                    }
                } catch (MalformedURLException e) {
                    Logger.e("Create Uri failed.", e);
                }
            }
                break;
            case SystemConsts.EVT_SELECT_CLEARCACHE: {
                ContentsUtils.clearMediaCache(getActivity(), data);
                ContentValues values = new ContentValues();
                values.put(TableConsts.AUDIO_CACHE_FILE, (String)null);
                getActivity().getContentResolver().update(
                        ContentUris.withAppendedId(
                                MediaConsts.MEDIA_CONTENT_URI, mediaId),
                        values, null, null);
                datasetChanged();
            }
                break;
            case SystemConsts.EVT_SELECT_LYRICS: {
                LyricsDialog dlg = new LyricsDialog();
                Bundle b = new Bundle();
                b.putString("title", title);
                b.putString("artist", artist);
                dlg.setArguments(b);
                dlg.show(getFragmentManager(), SystemConsts.TAG_WIKIPEDIA_DLG);
            }
                break;
            case SystemConsts.EVT_SELECT_YOUTUBE: {
                YoutubeSearchDialog dlg = new YoutubeSearchDialog();
                Bundle b = new Bundle();
                b.putString("title", title);
                b.putString("artist", artist);
                dlg.setArguments(b);
                dlg.show(getFragmentManager(), SystemConsts.TAG_YOUTUBE_DLG);
            }
                break;
            case SystemConsts.EVT_SELECT_LOVE: {
                ContentsUtils.lastfmLove(getActivity(), title, artist, album,
                        Long.toString(duration));
            }
                break;
            case SystemConsts.EVT_SELECT_BAN: {
                ContentsUtils.lastfmBan(getActivity(), title, artist, album,
                        Long.toString(duration));
            }
                break;
            case SystemConsts.EVT_SELECT_EDIT: {
                // 文字化け修正
                Intent intent = new Intent(getActivity(), TagEditActivity.class);
                intent.putExtra(SystemConsts.KEY_EDITTYPE,
                        TagEditActivity.MEDIA);
                intent.putExtra(SystemConsts.KEY_EDITKEY,
                        Long.toString(mediaId));
                getActivity().startActivityForResult(intent, SystemConsts.REQUEST_TAGEDIT);
            }
                break;
            default: {
            }
            }
        }
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        Cursor cursor = (Cursor) mAdapter.getItem(position);
        if (cursor != null) {
            String audio_id_colname = getAudioIdCol(cursor);
            ContentsUtils.playMediaReplace(SystemConsts.CONTENTSKEY_GENRES
                    + lastQueryTime, getBinder(), cursor, audio_id_colname,
                    MediaConsts.AudioGenresMember.DURATION,
                    MediaConsts.AudioGenresMember.DATA, position);
        }
    }

    @Override
    public void reload() {
        getLoaderManager().restartLoader(getLoaderID(), null, this);
    }

    @Override
    public void changedMedia() {
        IMediaPlayerService binder = getBinder();
        if (mAdapter != null) {
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    public void release() {
    }

    public void play(MenuItem item) {
        ContentsUtils.playGenres(getBinder(), (BaseActivity) getActivity(),
                genres_id);
        Toast.makeText(getActivity(), getString(R.string.txt_action_addgenres),
                Toast.LENGTH_SHORT).show();
    }

    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        if (mAdapter == null) {
            mAdapter = new GenresListAdapter(getActivity(), null, viewcache);
            setListAdapter(mAdapter);
        } else {
            Cursor cur = mAdapter.swapCursor(null);
            if (cur != null) {
                cur.close();
            }
        }
        getall = false;
        hideFooter();
        return new CursorLoader(getActivity(),
                MediaConsts.GENRES_MEMBER_CONTENT_URI, null,
                MediaConsts.AudioGenresMember.GENRE_ID + " = ?",
                new String[] { genres_id }, null);
    }

    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor data) {
        try {
            showFooter();
            lastQueryTime = Calendar.getInstance().getTimeInMillis();
            if (mAdapter != null) {
                if (ContentsUtils.isNoCacheAction(mPref)) {
                    if (data != null && !data.isClosed()) {
                        try {
                            mcur = new MatrixCursor(data.getColumnNames(),
                                    data.getCount());
                            if (data.getCount() > 0) {
                                data.moveToFirst();
                                do {
                                    ArrayList<Object> values = new ArrayList<Object>();
                                    for (int i = 0; i < mcur.getColumnCount(); i++) {
                                        if (AudioGenresMember.DURATION
                                                .equals(mcur.getColumnName(i))) {
                                            values.add(data.getLong(i));
                                        } else {
                                            values.add(data.getString(i));
                                        }
                                    }
                                    mcur.addRow(values
                                            .toArray(new Object[values.size()]));
                                } while (data.moveToNext());
                                Cursor cur = mAdapter.swapCursor(mcur);
                            }
                        } finally {
                            if (data != null) {
                                data.close();
                            }
                        }
                    }
                } else {
                    Cursor cur = mAdapter.swapCursor(data);
                    invisibleFooter();
                }
            }
        } finally {
        }
    }

    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
        showFooter();
        mcur = null;
    }

    @Override
    public void startProgress(final long max) {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    viewcache.startProgress(max);
                    IMediaPlayerService binder = getBinder();
                    if (binder != null) {
                        try {
                            viewcache.setPrefetchId(binder.getPrefetchId());
                        } catch (RemoteException e) {
                        }
                    }
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void stopProgress() {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    viewcache.stopProgress();
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void progress(final long pos, final long max) {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    viewcache.progress(pos, max);
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    AsyncTask<Void, Void, Void> loadtask = null;

    @Override
    public void onScroll(AbsListView view, int firstVisibleItem,
            int visibleItemCount, int totalItemCount) {
        if (mAdapter == null || loadtask != null) {
            return;
        }
        if (isReadFooter) {
            if (!getall && totalItemCount > 0
                    && totalItemCount == (firstVisibleItem + visibleItemCount)) {
                final int current = mAdapter.getCount();
                if (current > 0) {
                    loadtask = new AsyncTask<Void, Void, Void>() {

                        @Override
                        protected Void doInBackground(Void... params) {
                            Cursor cursor = null;
                            try {
                                cursor = getActivity()
                                        .getContentResolver()
                                        .query(MediaConsts.GENRES_MEMBER_CONTENT_URI,
                                                null,
                                                MediaConsts.AudioGenresMember.GENRE_ID
                                                        + " = ? AND LIMIT ? AND OFFSET ?",
                                                new String[] {
                                                        genres_id,
                                                        Integer.toString(SystemConsts.DEFAULT_LIST_LIMIT),
                                                        Long.toString(current) },
                                                null);
                                if (mcur != null && cursor.getCount() > 0) {
                                    cursor.moveToFirst();
                                    do {
                                        ArrayList<Object> values = new ArrayList<Object>();
                                        for (int i = 0; i < mcur
                                                .getColumnCount(); i++) {
                                            String colname = mcur
                                                    .getColumnName(i);
                                            int colid = cursor
                                                    .getColumnIndex(colname);
                                            if (colid != -1) {
                                                if (AudioMedia.DURATION
                                                        .equals(colname)) {
                                                    values.add(cursor
                                                            .getLong(colid));
                                                } else {
                                                    values.add(cursor
                                                            .getString(colid));
                                                }
                                            }
                                        }
                                        mHandler.sendMessage(mHandler
                                                .obtainMessage(
                                                        SystemConsts.ACT_ADDROW,
                                                        values.toArray(new Object[values
                                                                .size()])));
                                    } while (cursor.moveToNext());
                                } else {
                                    invisibleFooter();
                                }
                            } finally {
                                if (cursor != null) {
                                    cursor.close();
                                }
                                mHandler.sendEmptyMessage(SystemConsts.ACT_NOTIFYDATASETCHANGED);
                                loadtask = null;
                            }

                            return null;
                        }
                    };
                    loadtask.execute((Void) null);
                }
            }
        }
    }

    @Override
    public void onScrollStateChanged(AbsListView view, int scrollState) {
        Logger.d("onScrollStateChanged:" + scrollState);
        long time = Funcs.getHideTime(mPref);
        if (time > 0) {
            if (scrollState == 1) {
                if (runner != null) {
                    mHandler.removeCallbacks(runner);
                    runner = null;
                }
                if (getFragmentManager() != null) {
                    ControlFragment control = (ControlFragment) getFragmentManager()
                            .findFragmentByTag(SystemConsts.TAG_CONTROL);
                    if (control != null) {
                        control.hideControl(false);
                    }
                }
            } else if (scrollState == 0) {
                if (runner != null) {
                    mHandler.removeCallbacks(runner);
                    runner = null;
                }
                runner = new Runnable() {
                    @Override
                    public void run() {
                        try {
                            if (getFragmentManager() != null) {
                                ControlFragment control = (ControlFragment) getFragmentManager()
                                        .findFragmentByTag(
                                                SystemConsts.TAG_CONTROL);
                                if (control != null) {
                                    control.showControl(false);
                                }
                            }
                        } finally {
                            runner = null;
                        }
                    }
                };
                mHandler.postDelayed(runner, time);
            }
        }
    }

    @Override
    public boolean onBackPressed() {
        if (mMode != null) {
            mMode.finish();
            mMode = null;
            return true;
        }
        return false;
    }

    @Override
    public void selectSort() {
        // TODO Auto-generated method stub

    }

    @Override
    protected void datasetChanged() {
        if (mAdapter != null) {
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    protected void addRow(Object[] values) {

    }

    @Override
    protected void hideMenu() {
        if (mMode != null) {
            mMode.finish();
            mMode = null;
        }
    }
}
