package jp.co.kayo.android.localplayer.ui;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.util.List;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.core.ViewHolder;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.SdCardAccessHelper;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.FavoriteInfo;
import jp.co.kayo.android.localplayer.util.bean.OrderInfo;
import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.RatingBar;
import android.widget.TextView;

public class PlayOrderListViewAdapter extends ArrayAdapter<OrderInfo> {
    LayoutInflater inflator;
    Context context;
    ViewCache mViewCache;
    SharedPreferences pref;
    List<OrderInfo> objects;
    private boolean isSdCard;

    public static final String[] projection_sdcard = new String[] {
            MediaConsts.AudioMedia._ID, MediaConsts.AudioMedia.TITLE,
            MediaConsts.AudioMedia.ALBUM, MediaConsts.AudioMedia.ARTIST,
            MediaConsts.AudioMedia.DURATION, MediaConsts.AudioMedia.DATA

    };

    public static final String[] projection_nonsdcard = new String[] {
            MediaConsts.AudioMedia._ID, MediaConsts.AudioMedia.MEDIA_KEY,
            MediaConsts.AudioMedia.TITLE, MediaConsts.AudioMedia.ALBUM,
            MediaConsts.AudioMedia.ARTIST, MediaConsts.AudioMedia.DURATION,
            MediaConsts.AudioMedia.DATA

    };

    boolean loadskip = false;

    public void setLoadSkip(boolean b) {
        loadskip = b;
    }

    public String[] getProjection() {
        return ContentsUtils.isSDCard(pref) ? projection_sdcard
                : projection_nonsdcard;
    }

    public PlayOrderListViewAdapter(Context context, List<OrderInfo> list,
            ViewCache cache) {
        super(context, 0, list);
        this.objects = list;
        this.context = context;
        this.mViewCache = cache;
        this.pref = PreferenceManager.getDefaultSharedPreferences(context);
    }

    public LayoutInflater getInflator(Context context) {
        if (inflator == null) {
            inflator = (LayoutInflater) context
                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        }
        return inflator;
    }

    public void setData(List<OrderInfo> collection) {
        clear();
        if (collection != null) {
            objects.addAll(collection);
            notifyDataSetChanged();
        }
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder holder = null;
        if (convertView == null) {
            convertView = getInflator(context).inflate(R.layout.grip_list_row1,
                    parent, false);
            holder = new ViewHolder();
            holder.setImgGrip(convertView.findViewById(R.id.imgGrip));
            holder.setText1((TextView) convertView.findViewById(R.id.textTrack));
            holder.setText2((TextView) convertView.findViewById(R.id.textTitle));
            holder.setText3((TextView) convertView.findViewById(R.id.textAlbum));
            holder.setText4((TextView) convertView
                    .findViewById(R.id.textArtist));
            holder.setText5((TextView) convertView
                    .findViewById(R.id.textDuration));
            holder.setRating1((RatingBar) convertView
                    .findViewById(R.id.ratingBar1));
            holder.setProgressBar1((ProgressBar) convertView
                    .findViewById(R.id.progressBar1));
            holder.setImgCache((ImageView) convertView
                    .findViewById(R.id.imgCache));
            holder.setImgPlay((ImageView) convertView
                    .findViewById(R.id.imgPlay));
            holder.setBackground(convertView.findViewById(R.id.background));
            convertView.setTag(holder);
        } else {
            holder = (ViewHolder) convertView.getTag();
        }

        OrderInfo item = getItem(position);
        if (!loadskip || item.isLoaded()) {
            item.load(context, getProjection());

            holder.getText1().setText(Funcs.getTrack(item.index + 1));
            holder.getText2().setText(item.title);
            holder.getText3().setText(item.album);
            holder.getText4().setText(item.artist);
            FavoriteInfo inf = mViewCache.getFavorite(item.id,
                    TableConsts.FAVORITE_TYPE_SONG);
            holder.getRating1().setRating(inf.rating);
            holder.getText5().setText(Funcs.makeTimeString(item.duration));
            if (isSdCard != true && SdCardAccessHelper.existCachFile(item.data)) {
                holder.getImgCache().setVisibility(View.VISIBLE);
            } else {
                holder.getImgCache().setVisibility(View.GONE);
            }
        } else {
            holder.getText1().setText(Funcs.getTrack(item.index + 1));
            holder.getText2().setText("...");
            holder.getText3().setText("...");
            holder.getText4().setText("...");
            holder.getRating1().setRating(0);
            holder.getText5().setText("00:00");
            holder.getImgCache().setVisibility(View.GONE);
        }

        if (mViewCache.getCurrentPos() == position
                && mViewCache.getCurrentId() == item.id && position >= 0) {
            holder.getImgPlay().setVisibility(View.VISIBLE);
            holder.getBackground().setVisibility(View.VISIBLE);
        } else {
            holder.getImgPlay().setVisibility(View.GONE);
            holder.getBackground().setVisibility(View.GONE);
        }
        if (mViewCache.getPrefetchId() == -1
                || mViewCache.getPrefetchId() != item.id) {
            holder.getProgressBar1().setVisibility(View.GONE);
        } else {
            if (mViewCache.getMax() > 0) {
                holder.getProgressBar1().setVisibility(View.VISIBLE);
                holder.getProgressBar1().setMax(
                        (int) (mViewCache.getMax() / 1000));
                holder.getProgressBar1().setProgress(
                        (int) (mViewCache.getPos() / 1000));
            } else {
                holder.getProgressBar1().setVisibility(View.GONE);
            }
        }

        return convertView;
    }
}
