package jp.co.kayo.android.localplayer.ui;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import jp.co.kayo.android.localplayer.BaseActivity;
import jp.co.kayo.android.localplayer.BaseListFragment;
import jp.co.kayo.android.localplayer.MainActivity;
import jp.co.kayo.android.localplayer.PlayActivity;
import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.SubListActivity;
import jp.co.kayo.android.localplayer.TabFragment;
import jp.co.kayo.android.localplayer.TagEditActivity;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.core.ContextMenuFragment;
import jp.co.kayo.android.localplayer.core.DragListView;
import jp.co.kayo.android.localplayer.core.ServiceBinderHolder;
import jp.co.kayo.android.localplayer.core.DragListView.DropListener;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.provider.DownloadHelper;
import jp.co.kayo.android.localplayer.service.IMediaPlayerService;
import jp.co.kayo.android.localplayer.ui.dialog.DeletePlaylistDialog;
import jp.co.kayo.android.localplayer.ui.dialog.LyricsDialog;
import jp.co.kayo.android.localplayer.ui.dialog.OpenPlaylistDialog;
import jp.co.kayo.android.localplayer.ui.dialog.RatingDialog;
import jp.co.kayo.android.localplayer.ui.dialog.SavePlaylistDialog;
import jp.co.kayo.android.localplayer.ui.dialog.YoutubeSearchDialog;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.OrderInfo;
import android.app.AlertDialog;
import android.support.v4.app.LoaderManager.LoaderCallbacks;
import android.support.v4.content.AsyncTaskLoader;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.support.v4.content.Loader;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.os.ResultReceiver;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;

import android.view.ActionMode;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnLongClickListener;
import android.view.ViewGroup;
import android.view.View.OnTouchListener;
import android.widget.AbsListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

public class PlayOrderListViewFragment extends BaseListFragment implements
        ContentManager, ContextMenuFragment, DropListener,
        OnItemLongClickListener, TabFragment, LoaderCallbacks<List<OrderInfo>>,
        OnScrollListener, IProgressView, android.view.View.OnClickListener,
        OnLongClickListener {

    SharedPreferences mPref;
    PlayOrderListViewAdapter mAdapter;
    private ViewCache mViewcache;
    Runnable mTask = null;
    private ActionMode mMode;

    private IMediaPlayerService getBinder() {
        if (getActivity() instanceof ServiceBinderHolder) {
            return ((ServiceBinderHolder) getActivity()).getBinder();
        } else {
            return null;
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mPref = PreferenceManager.getDefaultSharedPreferences(getActivity());
        mViewcache = (ViewCache) getFragmentManager().findFragmentByTag(
                SystemConsts.TAG_CACHE);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.playorder_grid_view, container,
                false);

        DragListView list = (DragListView) root.findViewById(android.R.id.list);
        list.setDropListener(this);

        list.setOnItemLongClickListener(this);
        list.setOnScrollListener(this);

        root.findViewById(R.id.btnPlaylistOpen).setOnClickListener(this);
        root.findViewById(R.id.btnPlaylistSave).setOnClickListener(this);
        root.findViewById(R.id.btnPlaylistDelete).setOnClickListener(this);
        root.findViewById(R.id.btnPlaylistClear).setOnClickListener(this);

        root.findViewById(R.id.btnPlaylistOpen).setOnLongClickListener(this);
        root.findViewById(R.id.btnPlaylistSave).setOnLongClickListener(this);
        root.findViewById(R.id.btnPlaylistDelete).setOnLongClickListener(this);
        root.findViewById(R.id.btnPlaylistClear).setOnLongClickListener(this);

        return root;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        getLoaderManager().initLoader(R.layout.playorder_grid_view, null, this);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onResume() {
        super.onResume();
        Logger.d("PlayOrder.onResume");
        ((MainActivity) getActivity()).mPlayList = this;
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void reload() {
        if (getActivity() != null && mAdapter!=null) {
            getLoaderManager().restartLoader(R.layout.playorder_grid_view,
                    null, this);
        }
    }

    @Override
    public void changedMedia() {
        if (mAdapter != null) {
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> arg0, View view,
            int position, long arg3) {
        final OrderInfo oi = mAdapter.getItem(position);
        if (oi != null) {
            BaseActivity act = (BaseActivity) getActivity();
            mMode = act.startActionMode(new AnActionModeOfEpicProportions(
                    getActivity(), position, mHandler));
            mMode.setTitle(oi.title);
            mMode.setSubtitle(getString(R.string.lb_tab_playlist));
            return true;
        }

        return false;
    }

    private final class AnActionModeOfEpicProportions extends
            MediaContentActionCallback {

        public AnActionModeOfEpicProportions(Context context,
                int selectedIndex, Handler handler) {
            super(context, selectedIndex, handler);
        }

        @Override
        public void onDestroyActionMode(ActionMode mode) {
            mMode = null;
        }

        @Override
        public boolean onCreateActionMode(ActionMode mode, Menu menu) {

            menu.add(getString(R.string.sub_mnu_remove))
                    .setIcon(R.drawable.ic_menu_delete)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.sub_mnu_artist))
                    .setIcon(R.drawable.ic_menu_btn_artist)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.sub_mnu_album))
                    .setIcon(R.drawable.ic_menu_album)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.sub_mnu_rating))
                    .setIcon(R.drawable.ic_menu_star)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            menu.add(getString(R.string.txt_web_more))
                    .setIcon(R.drawable.ic_menu_wikipedia)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);

            /*
             * menu.add(getString(R.string.txt_web_youtube)).setIcon(R.drawable.
             * ic_menu_youtube)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
             * 
             * menu.add(getString(R.string.txt_web_wikipedia)).setIcon(R.drawable
             * .ic_menu_wikipedia)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
             * 
             * menu.add(getString(R.string.txt_web_lyrics)).setIcon(R.drawable.
             * ic_menu_lyrics)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
             */

            // 文字化け修正メニューを追加
            // if (ContentsUtils.isSDCard(mPref)) {
            menu.add(getString(R.string.sub_mnu_edit))
                    .setIcon(R.drawable.ic_menu_strenc)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
            // }

            boolean b = mPref.getBoolean("key.useLastFM", false);
            if (b) {
                menu.add(getString(R.string.sub_mnu_love)).setShowAsAction(
                        MenuItem.SHOW_AS_ACTION_IF_ROOM);
                menu.add(getString(R.string.sub_mnu_ban)).setShowAsAction(
                        MenuItem.SHOW_AS_ACTION_IF_ROOM);
            }

            /*
             * if (ContentsUtils.canDownload(mPref)) {
             * menu.add(getString(R.string
             * .sub_mnu_download)).setIcon(R.drawable.ic_menu_download)
             * .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM); }
             */

            if (!ContentsUtils.isSDCard(mPref)) {
                menu.add(getString(R.string.sub_mnu_clearcache))
                        .setIcon(R.drawable.ic_menu_refresh)
                        .setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
            }
            return true;
        }

    };

    @Override
    protected void messageHandle(int what, final int selectedPosition) {
        final OrderInfo oi = mAdapter.getCount()>selectedPosition?mAdapter.getItem(selectedPosition) : null;
        if (oi != null) {
            long mediaId = oi.id;
            String title = oi.title;
            String album = oi.album;
            String artist = oi.artist;
            String data = oi.data;
            long duration = oi.duration;

            switch (what) {
            case SystemConsts.EVT_SELECT_RATING: {
                if (mediaId >= 0) {
                    RatingDialog dlg = new RatingDialog(getActivity(),
                            TableConsts.FAVORITE_TYPE_SONG, mediaId, mHandler);
                    dlg.show();
                }
            }
                break;
            case SystemConsts.EVT_SELECT_PLAY: {
                ContentsUtils.playMedia(getBinder(), mediaId, duration, data);
                Toast.makeText(getActivity(),
                        getString(R.string.txt_action_addartist),
                        Toast.LENGTH_SHORT).show();
            }
                break;
            case SystemConsts.EVT_SELECT_ARTIST: {
                Intent intent = new Intent(getActivity(), SubListActivity.class);
                intent.putExtra(SystemConsts.KEY_LIST_SUBTYPE,
                        SystemConsts.PRM_ARTISTLIST);
                intent.putExtra(SystemConsts.KEY_LIST_TITLE, artist);
                intent.putExtra(SystemConsts.KEY_LIST_MAX, -1);
                // intent.putExtra(SystemConsts.KEY_LIST_KEY, artist_key);
                startActivity(intent);
            }
                break;
            case SystemConsts.EVT_SELECT_DEL: {
                AlertDialog.Builder builder = new AlertDialog.Builder(
                        getActivity());

                builder.setTitle(R.string.lb_confirm);
                builder.setMessage(String.format(
                        getString(R.string.fmt_remove_orderlist), title));
                builder.setPositiveButton(R.string.lb_ok,
                        new OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog,
                                    int which) {
                                mAdapter.remove(oi);
                                try {
                                    getBinder().setContentsKey(null);
                                    getBinder().remove(selectedPosition);
                                } catch (RemoteException e) {
                                }
                            }
                        });
                builder.setNegativeButton(R.string.lb_cancel, null);
                builder.show();
            }
                break;
            case SystemConsts.EVT_SELECT_OPENALBUM: {
                // TODO OpenAlbum未実装
                Hashtable<String, String> tbl = ContentsUtils.getMedia(
                        getActivity(), new String[] { AudioMedia.ALBUM_KEY },
                        oi.id);
                String album_key = tbl.get(AudioMedia.ALBUM_KEY);
                if (album_key != null) {
                    Intent intent = new Intent(getActivity(),
                            PlayActivity.class);
                    intent.putExtra(SystemConsts.KEY_PLAY_LOCK, true);
                    intent.putExtra(SystemConsts.KEY_PLAY_ALBUMKEY, album_key);
                    getActivity().startActivity(intent);
                }
            }
                break;
            case SystemConsts.EVT_SELECT_MORE: {
                showInfoDialog(album, artist, title);
            }
                break;
            case SystemConsts.EVT_SELECT_DOWNLOAD: {
                DownloadHelper helper = new DownloadHelper(getActivity());
                try {
                    if (helper.enqueueMusic(mediaId, title, data) != -1) {
                        Toast.makeText(getActivity(),
                                getString(R.string.txt_action_download),
                                Toast.LENGTH_SHORT).show();
                    }
                } catch (MalformedURLException e) {
                    Logger.e("Create Uri failed.", e);
                }
            }
                break;
            case SystemConsts.EVT_SELECT_CLEARCACHE: {
                ContentsUtils.clearMediaCache(getActivity(), data);
                ContentValues values = new ContentValues();
                values.put(TableConsts.AUDIO_CACHE_FILE, (String)null);
                getActivity().getContentResolver().update(
                        ContentUris.withAppendedId(
                                MediaConsts.MEDIA_CONTENT_URI, mediaId),
                        values, null, null);
                datasetChanged();
            }
                break;
            case SystemConsts.EVT_SELECT_LYRICS: {
                LyricsDialog dlg = new LyricsDialog();
                Bundle b = new Bundle();
                b.putString("title", title);
                b.putString("artist", artist);
                dlg.setArguments(b);
                dlg.show(getFragmentManager(), SystemConsts.TAG_WIKIPEDIA_DLG);
            }
                break;
            case SystemConsts.EVT_SELECT_YOUTUBE: {
                YoutubeSearchDialog dlg = new YoutubeSearchDialog();
                Bundle b = new Bundle();
                b.putString("title", title);
                b.putString("artist", artist);
                dlg.setArguments(b);
                dlg.show(getFragmentManager(), SystemConsts.TAG_YOUTUBE_DLG);
            }
                break;
            case SystemConsts.EVT_SELECT_LOVE: {
                ContentsUtils.lastfmLove(getActivity(), title, artist, album,
                        Long.toString(duration));
            }
                break;
            case SystemConsts.EVT_SELECT_BAN: {
                ContentsUtils.lastfmBan(getActivity(), title, artist, album,
                        Long.toString(duration));
            }
                break;
            case SystemConsts.EVT_SELECT_EDIT: {
                // 文字化け修正
                Intent intent = new Intent(getActivity(), TagEditActivity.class);
                intent.putExtra(SystemConsts.KEY_EDITTYPE,
                        TagEditActivity.MEDIA);
                intent.putExtra(SystemConsts.KEY_EDITKEY,
                        Long.toString(mediaId));
                getActivity().startActivityForResult(intent, SystemConsts.REQUEST_TAGEDIT);
            }
                break;
            default: {
            }
            }
        }
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        OrderInfo item = (OrderInfo) getListAdapter().getItem(position);
        if (item != null) {
            try {
                getBinder().stop();
                getBinder().setPosition(position);
                getBinder().play();

                mViewcache.setCurrentPos(position);
                mViewcache.setCurrentId(item.id);
                // ArrayAdapter<OrderInfo> adapter = (ArrayAdapter<OrderInfo>)
                // getListAdapter();
                // adapter.notifyDataSetChanged();
            } catch (RemoteException e) {
            }
        }
    }

    @Override
    public void drop(int from, int to) {
        if (from == to || from < 0 || to < 0) {
            return;
        }

        ArrayAdapter<OrderInfo> adapter = (ArrayAdapter<OrderInfo>) getListAdapter();
        OrderInfo item = adapter.getItem(from);

        adapter.remove(item);
        adapter.insert(item, to);

        try {
            getBinder().setContentsKey(null);
            getBinder().drop(from, to);
        } catch (RemoteException e) {
        }
    }

    @Override
    public void release() {
        // TODO Auto-generated method stub

    }

    @Override
    public int getFragmentId() {
        return R.layout.playorder_grid_view;
    }

    static class OrderInfoLoader extends AsyncTaskLoader<List<OrderInfo>> {
        PlayOrderListViewFragment fargment;
        ArrayList<OrderInfo> mFiles;

        public OrderInfoLoader(PlayOrderListViewFragment fargment) {
            super(fargment.getActivity());
            this.fargment = fargment;
        }

        private IMediaPlayerService getBinder() {
            if (fargment.getActivity() instanceof ServiceBinderHolder) {
                return ((ServiceBinderHolder) fargment.getActivity())
                        .getBinder();
            } else {
                return null;
            }
        }

        @Override
        public List<OrderInfo> loadInBackground() {
            final IMediaPlayerService binder = getBinder();
            mFiles = new ArrayList<OrderInfo>();
            if (binder != null) {
                long[] ids;
                try {
                    ids = binder.getList();
                    for (int i = 0; i < ids.length; i++) {
                        OrderInfo inf = new OrderInfo();
                        inf.id = ids[i];
                        inf.index = i;
                        if (inf.id <= 0) {
                            String[] values = binder.getMediaD(inf.index);
                            if (values != null) {
                                inf.title = values[0];
                                inf.album = values[1];
                                inf.artist = values[2];
                                inf.duration = Funcs.parseLong(values[3]);
                                inf.loaded = true;
                            }
                        }
                        mFiles.add(inf);
                    }
                } catch (RemoteException e) {
                } finally {
                }
            }
            return mFiles;
        }

        @Override
        protected void onStartLoading() {
            super.onStartLoading();
            forceLoad();
        }
    }

    public class FastOnTouchListener implements OnTouchListener {
        PlayOrderListViewAdapter adapter;
        Handler handler;

        public FastOnTouchListener(Handler handler,
                PlayOrderListViewAdapter adapter) {
            this.handler = handler;
            this.adapter = adapter;
        }

        @Override
        public boolean onTouch(View v, MotionEvent event) {
            int action = event.getAction();
            if (action == MotionEvent.ACTION_DOWN) {
                adapter.setLoadSkip(true);
            } else if (action == MotionEvent.ACTION_UP
                    || action == MotionEvent.ACTION_CANCEL) {
                adapter.setLoadSkip(false);
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        adapter.notifyDataSetChanged();
                    }
                });
            }
            return false;
        }
    }

    @Override
    public Loader<List<OrderInfo>> onCreateLoader(int id, Bundle args) {
        if (mAdapter == null && getListView() != null) {
            mAdapter = new PlayOrderListViewAdapter(getActivity(),
                    new ArrayList<OrderInfo>(), mViewcache);
            setListAdapter(mAdapter);
            getListView().setOnTouchListener(
                    new FastOnTouchListener(new Handler(), mAdapter));

        }
        showProgressBar();
        OrderInfoLoader l = new OrderInfoLoader(this);
        return l;
    }

    @Override
    public void onLoadFinished(Loader<List<OrderInfo>> loader,
            List<OrderInfo> data) {
        hideProgressBar();
        if (mAdapter != null) {
            mAdapter.setData(data);
            IMediaPlayerService binder = getBinder();
            if (binder != null) {
                int position;
                try {
                    position = binder.getPosition();
                    if (isVisible()) {
                        getListView().setSelectionFromTop(position, 0);
                    }
                } catch (RemoteException e) {
                }
            }
        }
    }

    @Override
    public void onLoaderReset(Loader<List<OrderInfo>> loader) {
        hideProgressBar();
        if (mAdapter != null) {
            mAdapter.setData(null);
        }
    }

    @Override
    public void startProgress(final long max) {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    mViewcache.startProgress(max);
                    IMediaPlayerService binder = getBinder();
                    if (binder != null) {
                        try {
                            mViewcache.setPrefetchId(binder.getPrefetchId());
                        } catch (RemoteException e) {
                        }
                    }
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void stopProgress() {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    mViewcache.stopProgress();
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void progress(final long pos, final long max) {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    mViewcache.progress(pos, max);
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void onScroll(AbsListView view, int firstVisibleItem,
            int visibleItemCount, int totalItemCount) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onScrollStateChanged(AbsListView view, int scrollState) {
        Logger.d("onScrollStateChanged:" + scrollState);
        long time = Funcs.getHideTime(mPref);
        if (time > 0) {
            if (scrollState == 1) {
                if (mTask != null) {
                    mHandler.removeCallbacks(mTask);
                    mTask = null;
                }
                if (getFragmentManager() != null) {
                    ControlFragment control = (ControlFragment) getFragmentManager()
                            .findFragmentByTag(SystemConsts.TAG_CONTROL);
                    if (control != null) {
                        control.hideControl(false);
                    }
                }
            } else if (scrollState == 0) {
                if (mTask != null) {
                    mHandler.removeCallbacks(mTask);
                    mTask = null;
                }
                mTask = new Runnable() {
                    @Override
                    public void run() {
                        try {
                            if (getFragmentManager() != null) {
                                ControlFragment control = (ControlFragment) getFragmentManager()
                                        .findFragmentByTag(
                                                SystemConsts.TAG_CONTROL);
                                if (control != null) {
                                    control.showControl(false);
                                }
                            }
                        } finally {
                            mTask = null;
                        }
                    }
                };
                mHandler.postDelayed(mTask, time);
            }
        }
    }

    @Override
    public boolean onBackPressed() {
        if (mMode != null) {
            mMode.finish();
            mMode = null;
            return true;
        }
        return false;
    }

    @Override
    public void selectSort() {
        // TODO Auto-generated method stub

    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
        case R.id.btnPlaylistOpen: {
            procPlaylistOpen();
        }
            break;
        case R.id.btnPlaylistSave: {
            procPlaylistSave();
        }
            break;
        case R.id.btnPlaylistDelete: {
            procPlaylistDelete();
        }
            break;
        case R.id.btnPlaylistClear: {
            procPlaylistClear();
        }
            break;
        }
    }

    @Override
    public boolean onLongClick(View v) {
        switch (v.getId()) {
        case R.id.btnPlaylistOpen: {
            Toast.makeText(getActivity(),
                    getString(R.string.playlist_btn_open), Toast.LENGTH_SHORT)
                    .show();
            return true;
        }
        case R.id.btnPlaylistSave: {
            Toast.makeText(getActivity(),
                    getString(R.string.playlist_btn_save), Toast.LENGTH_SHORT)
                    .show();
            return true;
        }
        case R.id.btnPlaylistDelete: {
            Toast.makeText(getActivity(),
                    getString(R.string.playlist_btn_delete), Toast.LENGTH_SHORT)
                    .show();
            return true;
        }
        case R.id.btnPlaylistClear: {
            Toast.makeText(getActivity(),
                    getString(R.string.playlist_btn_clear), Toast.LENGTH_SHORT)
                    .show();
            return true;
        }
        }
        return false;
    }

    private void procPlaylistOpen() {
        OpenPlaylistDialog dlg = new OpenPlaylistDialog();
        dlg.show(getFragmentManager(), "playlist.dlg");
    }

    private void procPlaylistSave() {
        final IMediaPlayerService binder = getBinder();
        if (binder != null) {
            long[] ids;
            try {
                ids = binder.getList();
                SavePlaylistDialog dlg = new SavePlaylistDialog();
                Bundle b = new Bundle();
                b.putLongArray("playlist", ids);
                dlg.setArguments(b);
                dlg.show(getFragmentManager(), SystemConsts.TAG_RATING_DLG);
            } catch (RemoteException e) {
                Logger.e("mnu_save_playlist", e);
            } finally {

            }

        }
    }

    private void procPlaylistDelete() {
        DeletePlaylistDialog dlg = new DeletePlaylistDialog();
        dlg.show(getFragmentManager(), "playlist.dlg");
    }

    private void procPlaylistClear() {
        final IMediaPlayerService binder = getBinder();
        if (binder != null) {
            try {
                getBinder().setContentsKey(null);
                getBinder().clear();
            } catch (RemoteException e) {
            }
        }
    }

    @Override
    protected void hideMenu() {
        if (mMode != null) {
            mMode.finish();
            mMode = null;
        }
    }

    @Override
    protected void datasetChanged() {
        if (mAdapter != null) {
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    protected void addRow(Object[] values) {
        // TODO Auto-generated method stub

    }
}
