package jp.co.kayo.android.localplayer.ui.plugin;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.lang.ref.WeakReference;
import java.util.Hashtable;
import java.util.Timer;
import java.util.TimerTask;

import jp.co.kayo.android.localplayer.AlbumartActivity;
import jp.co.kayo.android.localplayer.PlayActivity;
import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioFavorite;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.ui.ControlFragment;
import jp.co.kayo.android.localplayer.util.Funcs;
import jp.co.kayo.android.localplayer.util.ImageObserver;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.FavoriteInfo;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.content.ContentValues;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.animation.ScaleAnimation;
import android.view.ViewGroup;
import android.widget.FrameLayout.LayoutParams;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RatingBar;
import android.widget.RatingBar.OnRatingBarChangeListener;
import android.widget.TextView;

public class AlbumArtFragment extends Fragment implements ContentManager,
        OnRatingBarChangeListener, OnClickListener {
    private ViewCache mViewCache;
    Bitmap sourcebmp;
    ImageView imageAlbumArt;
    TextView txtTitle1;
    TextView txtTitle2;
    RatingBar ratingBar1;
    Fragment parent;
    Handler mHandler;
    String balbum_key;
    long mAlbumId;
    int imagesize;
    private FavoriteInfo favorite = null;
    SharedPreferences mPref;
    private LinearLayout mPlayViewLayout;
    private LinearLayout mSongInfoLayout;
    private LinearLayout mSongTxtLayout;

    public PlayActivity getMain() {
        return (PlayActivity) getActivity();
    }

    public String getAlbumKey() {
        return getMain().getAlbumKey();
    }

    ViewCache getCache() {
        if (mViewCache == null) {
            mViewCache = (ViewCache) getFragmentManager().findFragmentByTag(
                    SystemConsts.TAG_CACHE);
        }
        return mViewCache;
    }
    
    private static class MyHandler extends Handler {
        WeakReference<AlbumArtFragment> refFragment;
        MyHandler(AlbumArtFragment activity){
            refFragment = new WeakReference<AlbumArtFragment>(activity);
        }
        
        @Override
        public void handleMessage(Message msg) {
            AlbumArtFragment fragment = refFragment.get();
            if(fragment!=null){
                switch (msg.what) {
                case SystemConsts.EVT_UPDATE_RATING: {
                    if (fragment.balbum_key != null) {
                        Hashtable<String, String> tbl2 = ContentsUtils
                                .getAlbum(fragment.getActivity(),
                                        new String[] { AudioAlbum._ID,
                                                AudioAlbum.ALBUM,
                                                AudioAlbum.ARTIST,
                                                AudioAlbum.ALBUM_KEY },
                                                fragment.balbum_key);
                        String id = tbl2.get(AudioAlbum._ID);
                        FavoriteInfo inf = fragment.getCache().getFavorite(
                                Funcs.getInt(id),
                                TableConsts.FAVORITE_TYPE_ALBUM);
                        fragment.ratingBar1.setRating(inf.rating);
                    }
                }
                    break;
                }
            }
        }
    };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mHandler = new MyHandler(this);
        if (mPref == null) {
            mPref = PreferenceManager
                    .getDefaultSharedPreferences(getActivity());
        }

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.play_view_albumart, null);

        imageAlbumArt = (ImageView) root.findViewById(R.id.imageAlbumArt);
        txtTitle1 = (TextView) root.findViewById(R.id.txtTitle1);
        txtTitle2 = (TextView) root.findViewById(R.id.txtTitle2);
        ratingBar1 = (RatingBar) root.findViewById(R.id.ratingBar1);
        ratingBar1.setOnRatingBarChangeListener(this);
        imageAlbumArt.setOnClickListener(this);

        // 画面が縦向きならアルバムアートを拡大するためにアルバムアートレイアウトを取得
        Configuration config = getResources().getConfiguration();
        if (config.orientation == Configuration.ORIENTATION_PORTRAIT) {
            mPlayViewLayout = (LinearLayout) root
                    .findViewById(R.id.playViewLayout);
            mPlayViewLayout.setOnClickListener(this);
            mSongInfoLayout = (LinearLayout) root
                    .findViewById(R.id.songInfoLayout);
            mSongTxtLayout = (LinearLayout) root
                    .findViewById(R.id.songTxtLayout);
        }

        return root;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        this.imagesize = getResources().getDimensionPixelSize(
                R.dimen.albumart_size);
    }

    @Override
    public void onResume() {
        super.onResume();
        setAlbumImage();
    }

    @Override
    public void onPause() {
        super.onPause();
        imageAlbumArt.setImageBitmap(null);
        balbum_key = null;
        if (sourcebmp != null) {
            sourcebmp.recycle();
            sourcebmp = null;
            imageAlbumArt.setImageBitmap(null);
        }

        // アルバムアートを縮小処理のタイマーが動いていたら停止
        if (mTimer != null) {
            mTimer.cancel();
            mTimer = null;
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    public void setAlbumImage() {
        String album_key = getAlbumKey();
        if (album_key != null) {
            if (balbum_key == null || !balbum_key.equals(album_key)) {
                balbum_key = album_key;
                Hashtable<String, String> tbl2 = ContentsUtils.getAlbum(
                        getActivity(), new String[] { AudioAlbum._ID,
                                AudioAlbum.ALBUM, AudioAlbum.ARTIST,
                                AudioAlbum.ALBUM_KEY }, album_key);
                String id = tbl2.get(AudioAlbum._ID);
                txtTitle1.setText(tbl2.get(AudioAlbum.ALBUM));
                txtTitle2.setText(tbl2.get(AudioAlbum.ARTIST));
                // textArtist.setText(artist);
                imageAlbumArt.setImageResource(R.drawable.brank_album_white);
                getCache().getUnManagerImage(tbl2.get(AudioAlbum.ALBUM),
                        tbl2.get(AudioAlbum.ARTIST), album_key,
                        new MyImageObserver(), imagesize);

                if (id != null) {
                    FavoriteInfo inf = getCache().getFavorite(Funcs.getInt(id),
                            TableConsts.FAVORITE_TYPE_ALBUM);
                    ratingBar1.setRating(inf.rating);
                    mAlbumId = Long.parseLong(id);

                    Cursor cur = null;
                    try {
                        cur = getActivity()
                                .getContentResolver()
                                .query(MediaConsts.FAVORITE_CONTENT_URI,
                                        new String[] { AudioFavorite._ID,
                                                AudioFavorite.POINT },
                                        AudioFavorite.MEDIA_ID + " = ? and "
                                                + AudioFavorite.TYPE + " = ?",
                                        new String[] { Long.toString(mAlbumId),
                                                TableConsts.FAVORITE_TYPE_ALBUM },
                                        null);
                        if (cur != null && cur.moveToFirst()) {
                            int rating = cur.getInt(cur
                                    .getColumnIndex(AudioFavorite.POINT));
                            favorite = new FavoriteInfo();
                            favorite.media_id = mAlbumId;
                            favorite.rating = rating;
                            ratingBar1.setRating(rating);
                        }
                    } finally {
                        if (cur != null) {
                            cur.close();
                        }
                    }
                }
            }
        }
    }

    private void setBitmap(Bitmap bm) {
        sourcebmp = bm;
        imageAlbumArt.setImageBitmap(bm);
    }

    class MyImageObserver implements ImageObserver {

        @Override
        public void onLoadImage(final Bitmap bmp) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    setBitmap(bmp);
                }
            });
        }
    }

    @Override
    public void reload() {
        if (isVisible()) {
            setAlbumImage();
        }
    }

    @Override
    public void onRatingChanged(RatingBar ratingBar, float rating,
            boolean fromUser) {
        // 更新
        if (favorite != null) {
            ContentValues values = new ContentValues();
            values.put(AudioFavorite.POINT, (int) rating);
            getActivity().getContentResolver().update(
                    MediaConsts.FAVORITE_CONTENT_URI,
                    values,
                    AudioFavorite.MEDIA_ID + " = ? AND " + AudioFavorite.TYPE
                            + " = ?",
                    new String[] { Long.toString(mAlbumId),
                            TableConsts.FAVORITE_TYPE_ALBUM });
            favorite.rating = (int) rating;
        } else {
            Cursor cur = null;
            try {
                ContentValues values = new ContentValues();
                values.put(AudioFavorite.MEDIA_ID, mAlbumId);
                values.put(AudioFavorite.POINT, (int) rating);
                values.put(AudioFavorite.TYPE, TableConsts.FAVORITE_TYPE_ALBUM);
                getActivity().getContentResolver().insert(
                        MediaConsts.FAVORITE_CONTENT_URI, values);

                favorite = new FavoriteInfo();
                favorite.media_id = mAlbumId;
                favorite.rating = (int) rating;
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    @Override
    public void release() {
        // TODO Auto-generated method stub

    }

    @Override
    public void changedMedia() {
        // TODO Auto-generated method stub

    }

    private Timer mTimer = null;
    private boolean mAlbumartScale = false;
    private int mOrgLayoutSize;
    private int mScaleLayoutSize;

    /**
     * アルバムアートを拡大
     */
    private void scaleUpAlbumArt() {
        // 背景のCDジャケットを消す
        imageAlbumArt.setBackgroundColor(Color.argb(00, 00, 00, 00));

        // 拡大サイズを設定
        mOrgLayoutSize = mPlayViewLayout.getHeight();
        PlayActivity act = (PlayActivity) getActivity();
        if (act != null) {
            mScaleLayoutSize = act.getMainPlayHeight();
        }

        // 拡大アニメーション開始
        float toX = Float.valueOf(getResources().getString(R.string.scaleX));
        float toY = Float.valueOf(getResources().getString(R.string.scaleY));
        int pivotX = getResources().getInteger(R.integer.pivotX);
        int pivotY = getResources().getInteger(R.integer.pivotY);
        ScaleAnimation scale = new ScaleAnimation(1, toX, 1, toY, pivotX,
                pivotY);
        scale.setDuration(300);
        scale.setFillAfter(true);
        imageAlbumArt.startAnimation(scale);

        // アルバムアートの拡大に合わせてレイアウトの高さを変更
        LayoutParams params = new LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, mScaleLayoutSize);
        mPlayViewLayout.setOrientation(LinearLayout.VERTICAL);
        mPlayViewLayout.setLayoutParams(params);

        // 曲情報をアルバムアートの下に移動
        if (act != null) {
            int resource_id = act.getThemeBackground();
            mSongTxtLayout.setBackgroundResource(resource_id);
            mSongTxtLayout.setPadding(20, 4, 20, 4);
        }
        mSongInfoLayout.setGravity(Gravity.BOTTOM | Gravity.CENTER_HORIZONTAL);
        txtTitle1.setGravity(Gravity.CENTER);
        txtTitle2.setGravity(Gravity.CENTER);

        // ソングリストを隠す
        FragmentManager fm = getFragmentManager();
        Fragment f = fm.findFragmentByTag("SONGLIST");
        FragmentTransaction ft = fm.beginTransaction();
        if (f != null) {
            ft.setCustomAnimations(R.anim.alpha_show, R.anim.alpha_hide)
                    .hide(f).commit();
        }

        // 再生コントロールを表示状態にする
        ControlFragment control = (ControlFragment) getFragmentManager()
                .findFragmentByTag("tag_control");
        control.showControl(true);
    }

    /**
     * アルバムアートを縮小
     */
    private void scaleDownAlbumArt() {
        // 拡大サイズを設定
        mScaleLayoutSize = mOrgLayoutSize;

        // 縮小アニメーション開始
        float fromX = Float.valueOf(getResources().getString(R.string.scaleX));
        float fromY = Float.valueOf(getResources().getString(R.string.scaleY));
        int pivotX = getResources().getInteger(R.integer.pivotX);
        int pivotY = getResources().getInteger(R.integer.pivotY);
        ScaleAnimation scale = new ScaleAnimation(fromX, 1, fromY, 1, pivotX,
                pivotY);
        scale.setDuration(300);
        scale.setFillAfter(true);
        imageAlbumArt.startAnimation(scale);

        // アルバムアートの拡大に合わせてレイアウトの高さを変更
        if (mTimer == null) {
            mTimer = new Timer(true);
            final Handler handler = new Handler();
            mTimer.schedule(new TimerTask() {
                @Override
                public void run() {
                    handler.post(new Runnable() {
                        public void run() {
                            // 背景のCDジャケットをセット
                            imageAlbumArt
                                    .setBackgroundResource(R.drawable.album_border_mini);

                            // アルバムアート表示領域を元に戻す
                            LayoutParams params = new LayoutParams(
                                    ViewGroup.LayoutParams.MATCH_PARENT,
                                    mOrgLayoutSize);
                            mPlayViewLayout
                                    .setOrientation(LinearLayout.HORIZONTAL);
                            mPlayViewLayout.setLayoutParams(params);

                            // 曲情報を元の位置に戻す
                            mSongTxtLayout.setBackgroundColor(Color.argb(0, 0,
                                    0, 0));
                            mSongInfoLayout.setGravity(Gravity.LEFT);
                            txtTitle1.setGravity(Gravity.LEFT);
                            txtTitle2.setGravity(Gravity.LEFT);

                            // ソングリストを表示
                            FragmentManager fm = getFragmentManager();
                            Fragment f = fm.findFragmentByTag("SONGLIST");
                            FragmentTransaction ft = fm.beginTransaction();
                            if (f != null) {
                                ft.setCustomAnimations(R.anim.alpha_show,
                                        R.anim.alpha_hide).show(f).commit();
                            }

                            if (mTimer != null) {
                                mTimer.cancel();
                                mTimer = null;
                            }
                        }
                    });
                }
            }, 200, 1000);
        }
    }

    @Override
    public void onClick(View v) {
        Configuration config = getResources().getConfiguration();
        if (config.orientation == Configuration.ORIENTATION_PORTRAIT) {
            int id = v.getId();
            switch (id) {
            case R.id.imageAlbumArt:
                if (mTimer == null && getMain() != null) {
                    if (mAlbumartScale == false) {
                        // アルバムアートの拡大
                        scaleUpAlbumArt();
                        mAlbumartScale = true;
                    } else {
                        // アルバムアートを元のサイズに戻す
                        scaleDownAlbumArt();
                        mAlbumartScale = false;
                    }
                }
                break;
            case R.id.playViewLayout:
                if (mTimer == null && getMain() != null) {
                    if (mAlbumartScale == true) {
                        // アルバムアートを元のサイズに戻す
                        scaleDownAlbumArt();
                        mAlbumartScale = false;
                    }
                }
                break;
            }
        }
    }
}
