package jp.co.kayo.android.localplayer.ui.pref;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.util.ArrayList;
import java.util.List;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.preference.DialogPreference;
import android.preference.PreferenceManager;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.ListView;
import android.widget.Toast;

public class ChooseTabPreference extends DialogPreference implements
        OnItemClickListener {
    ArrayList<ValueItem> items = new ArrayList<ValueItem>();
    BooleanListAdapter adapter;

    public ChooseTabPreference(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    public ChooseTabPreference(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    @Override
    protected View onCreateDialogView() {
        LayoutInflater inflater = (LayoutInflater) getContext()
                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View view = inflater.inflate(R.layout.checkboxlist, null);

        SharedPreferences pref = PreferenceManager
                .getDefaultSharedPreferences(getContext());
        items.clear();
        items.add(new ValueItem(getContext().getString(R.string.lb_tab_albums),
                pref.getBoolean(SystemConsts.VISIBLE_TAG_ALBUM, true)));
        items.add(new ValueItem(getContext().getString(R.string.lb_tab_artist),
                pref.getBoolean(SystemConsts.VISIBLE_TAG_ARTIST, true)));
        items.add(new ValueItem(getContext().getString(R.string.lb_tab_media),
                pref.getBoolean(SystemConsts.VISIBLE_TAG_MEDIA, true)));
        items.add(new ValueItem(getContext().getString(R.string.lb_tab_genres),
                pref.getBoolean(SystemConsts.VISIBLE_TAG_GENRES, true)));
        items.add(new ValueItem(getContext().getString(R.string.lb_tab_folder),
                pref.getBoolean(SystemConsts.VISIBLE_TAG_FOLDER, true)));
        items.add(new ValueItem(getContext().getString(R.string.lb_tab_videos),
                pref.getBoolean(SystemConsts.VISIBLE_TAG_VIDEO, false)));
        // items.add(new
        // ValueItem(getContext().getString(R.string.lb_tab_order),
        // pref.getBoolean(SystemConsts.VISIBLE_TAG_ORDER, true)));

        adapter = new BooleanListAdapter(getContext(), items);

        ListView listview = (ListView) view.findViewById(R.id.listView1);
        listview.setAdapter(adapter);

        listview.setOnItemClickListener(this);

        return view;
    }

    @Override
    protected void onDialogClosed(boolean positiveResult) {
        if (positiveResult) {
            SharedPreferences pref = PreferenceManager
                    .getDefaultSharedPreferences(getContext());
            Editor editor = pref.edit();
            editor.putBoolean(SystemConsts.VISIBLE_TAG_ALBUM,
                    items.get(0).checked);
            editor.putBoolean(SystemConsts.VISIBLE_TAG_ARTIST,
                    items.get(1).checked);
            editor.putBoolean(SystemConsts.VISIBLE_TAG_MEDIA,
                    items.get(2).checked);
            editor.putBoolean(SystemConsts.VISIBLE_TAG_GENRES,
                    items.get(3).checked);
            editor.putBoolean(SystemConsts.VISIBLE_TAG_FOLDER,
                    items.get(4).checked);
            editor.putBoolean(SystemConsts.VISIBLE_TAG_VIDEO,
                    items.get(5).checked);
            // editor.putBoolean(SystemConsts.VISIBLE_TAG_ORDER,
            // items.get(5).checked);
            editor.commit();
            Toast.makeText(getContext(),
                    getContext().getString(R.string.txt_pref_changed_notice),
                    Toast.LENGTH_SHORT).show();
        }

        super.onDialogClosed(positiveResult);
    }

    @Override
    public void onItemClick(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
        items.get(arg2).checked = !items.get(arg2).checked;

        if (adapter != null) {
            adapter.notifyDataSetChanged();
        }
    }

    class ValueItem {
        boolean checked;
        String name;

        ValueItem(String n, boolean b) {
            name = n;
            checked = b;
        }
    }

    class BooleanListAdapter extends ArrayAdapter<ValueItem> {
        public BooleanListAdapter(Context context, List<ValueItem> items) {
            super(context, -1, items);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            CheckBox checkBox = (CheckBox) convertView;
            if (checkBox == null) {
                checkBox = new CheckBox(getContext());
                checkBox.setFocusable(false);
                checkBox.setClickable(false);
            }

            ValueItem item = getItem(position);
            checkBox.setChecked(item.checked);
            checkBox.setText(item.name);

            return checkBox;
        }
    }
}
