package jp.co.kayo.android.localplayer.ui.pref;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import android.content.Context;
import android.media.AudioManager;
import android.preference.DialogPreference;
import android.util.AttributeSet;
import android.view.View;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;

public class VolumeDialogPreference extends DialogPreference {
    private SeekBar seekBar = null;
    AudioManager am;
    private int currentVol;

    public VolumeDialogPreference(Context context, AttributeSet attrs) {
        super(context, attrs);
        hideButtons();
    }

    public VolumeDialogPreference(Context context, AttributeSet attrs,
            int defStyle) {
        super(context, attrs, defStyle);
        hideButtons();
    }

    private void hideButtons() {
        this.setPositiveButtonText(null);
        this.setNegativeButtonText(null);
    }

    @Override
    protected View onCreateDialogView() {
        seekBar = new SeekBar(getContext());

        // 最大音量をシークバーの最大に設定
        am = (AudioManager) this.getContext().getSystemService(
                Context.AUDIO_SERVICE);
        int maxVolume = am.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
        currentVol = am.getStreamVolume(AudioManager.STREAM_MUSIC);
        seekBar.setMax(maxVolume);

        // 現在の設定の読み込み
        int volume = getPersistedInt(2);
        seekBar.setProgress(volume);

        seekBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
            public void onProgressChanged(SeekBar seekBar, int progress,
                    boolean fromUser) {
                // ツマミをドラッグしたときに呼ばれる
            }

            public void onStartTrackingTouch(SeekBar seekBar) {
                // ツマミに触れたときに呼ばれる
            }

            public void onStopTrackingTouch(SeekBar seekBar) {
                // ツマミを離したときに呼ばれる
                int vol = seekBar.getProgress();
                persistInt(vol);
                // Logger.d("VolumeDialogPreference.Volume: " +
                // seekBar.getProgress());
                am.setStreamVolume(AudioManager.STREAM_MUSIC, vol,
                        AudioManager.FLAG_PLAY_SOUND);
            }
        });

        return seekBar;
    }

    @Override
    protected void onDialogClosed(boolean positiveResult) {

        am.setStreamVolume(AudioManager.STREAM_MUSIC, currentVol, 0);

        super.onDialogClosed(positiveResult);
    }
}
