package jp.co.kayo.android.localplayer.util;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.consts.SystemConsts;

import org.json.JSONObject;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Build;
import android.os.Environment;
import android.text.format.Time;
import android.util.TimeFormatException;
import android.widget.Toast;

public class Funcs {
    public static int CACHE_STATE_OK = 1;
    public static int CACHE_STATE_NOT_LIMIT_SPACE = 0;
    public static int CACHE_STATE_NOUSE = -1;
    
    public static byte[] getBytes(String s){
        int slen = s.length();
        byte bytes[] = new byte[slen];
        for (int i = 0; i < slen; i++) {
            char c = s.charAt(i);
            if (c >= 0x100) {
                // Byte列になっていない
                return null;
            }
            bytes[i] = (byte) c;
        }
        return bytes;
    }
    
    public static String getEncodingString(String s, String enc) throws UnsupportedEncodingException{
        byte[] bufs = getBytes(s);
        if(bufs!=null){
            return new String(bufs, enc);
        }
        else{
            return s;
        }
    }
    
    /***
     * 1: fine 0: no limit space -1: cant use cache
     * 
     * @param totalsize
     * @return
     */
    public static int canUseCache(SharedPreferences mPref, long bytesize) {
        long maxCacheSize = Funcs.getCachSize(mPref);
        SdCardAccessHelper sdHelper = new SdCardAccessHelper();
        long cacheSize = sdHelper
                .calcDirSize(SdCardAccessHelper.cachedMusicDir);
        long totalsize = (bytesize+cacheSize)/1000000;

        if (totalsize <= maxCacheSize || maxCacheSize == -1) {
            return CACHE_STATE_OK;
        } else if (maxCacheSize > 0) {
            return CACHE_STATE_NOT_LIMIT_SPACE;
        } else {
            return CACHE_STATE_NOUSE;
        }
    }

    public static long makeSubstansId(String name) {
        return (name.hashCode() & 0xffffffffL) * -1;
    }

    public static void enableHttpResponseCache(Context context) {
        try {
            long httpCacheSize = 10 * 1024 * 1024; // 10 MiB
            File httpCacheDir = new File(context.getCacheDir(), "http");
            Class.forName("android.net.http.HttpResponseCache")
                    .getMethod("install", File.class, long.class)
                    .invoke(null, httpCacheDir, httpCacheSize);
        } catch (Exception httpResponseCacheNotAvailable) {
        }
    }

    public static void disableConnectionReuseIfNecessary() {
        // HTTP connection reuse which was buggy pre-froyo
        if (Build.VERSION.SDK_INT <= 8) {
            System.setProperty("http.keepAlive", "false");
        }
    }

    public static String trimString(String s) {
        if (s != null && s.length() > 0) {
            return s.trim();
        } else {
            return "";
        }
    }

    public static int getInt(String s) {
        if (s != null && s.length() > 0) {
            try {
                return Integer.parseInt(s);
            } catch (Exception e) {
            }
        }
        return 0;
    }

    public static boolean isMedia(String fname, String[] exts) {
        int s1 = fname.lastIndexOf('.');
        if (s1 > 0) {
            String ext = fname.substring(s1);
            for (int i = 0; i < exts.length; i++) {
                if (ext.toLowerCase().equals(exts[i])) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean isJPG(String fname) {
        int s1 = fname.lastIndexOf('.');
        if (s1 > 0) {
            String ext = fname.substring(s1);
            if (ext.toLowerCase().equals(".jpg")) {
                return true;
            }
        }
        return false;
    }

    // static Formatter _timefmt = new Formatter();
    /***
     * ミリ秒から0:00の形式で文字列を作成します
     * 
     * @param context
     * @param msecs
     * @return
     */
    public static String makeTimeString(long msecs) {
        if (msecs > 0) {
            long sec = msecs / 1000;
            // _timefmt.flush();
            return String.format("%1$2d:%2$02d",
                    new Object[] { sec / 60, sec % 60 }).toString();
        } else {
            return "0:00";
        }
    }

    public static Integer getAlbumKey(String album, String artist) {
        if (album != null && artist != null) {
            return (album.trim() + artist.trim()).hashCode();
        } else if (album != null) {
            return (album.trim()).hashCode();
        } else if (artist != null) {
            return (artist.trim()).hashCode();
        }
        return new Integer(0);
    }

    public static String getTrack(int i) {
        return String.format("%1$02d", i);
    }

    public static long parseLong(String text) {
        if (text != null && text.length() > 0) {
            try {
                return Long.parseLong(text);
            } catch (Exception e) {

            }
        }
        return 0;
    }

    public static int parseInt(String text) {
        if (text != null && text.length() > 0) {
            try {
                return Integer.parseInt(text);
            } catch (Exception e) {

            }
        }
        return 0;
    }

    static SimpleDateFormat _fmt = new SimpleDateFormat(
            "yyyy-MM-dd'T'HH:mm:ssZ");

    public static long parseDate(String text) {
        if (text != null && text.length() > 0) {
            if (false) {
                // SimpleDateFormatよりもandroid.text.format.Timeのほうが早いらしい
                try {
                    Time andTime = new Time();
                    andTime.parse3339(text);
                    long parsed = andTime.toMillis(true);
                    return parsed;
                } catch (TimeFormatException e) {
                    Logger.e("parseDate, text=" + text, e);
                }
                return -1;
            } else {
                try {
                    // 2011-08-01T16:03:16+09:00
                    Date date = _fmt.parse(text);
                    return date.getTime();
                } catch (Exception e) {
                }
            }
        }
        return 0;
    }

    static final String BITLYAPISHORTEN = "http://api.bit.ly/v3/shorten?apiKey=";
    static final String BITLYAPIEXPAND = "http://api.bit.ly/v3/expand?apiKey=";
    static final String BITLYLOGIN = "&login=";
    static final String BITLYLONGURL = "&longUrl=";
    static final String BITLYHASH = "&hash=";
    static final String BITLYSHORTURL = "&shortUrl=";
    static final String BITLYFORMAT = "&format=txt";
    static final String BITLYURL = "http://bit.ly/";

    public static String urlShorterBitly(String s) {
        try {
            String targetURL = BITLYAPISHORTEN
                    + "R_b73da8bcaa024e308246dd17b270d23a" + BITLYLOGIN
                    + "yokmama" + BITLYFORMAT + BITLYLONGURL
                    + URLEncoder.encode(s, "UTF-8");

            URL url = new URL(targetURL);
            HttpURLConnection urlconn = (HttpURLConnection) url
                    .openConnection();
            urlconn.setRequestMethod("GET");
            urlconn.setInstanceFollowRedirects(false);
            urlconn.setRequestProperty("Connection", "close");
            urlconn.connect();

            if (urlconn.getResponseCode() == 200) {
                BufferedReader bin;
                bin = new BufferedReader(new InputStreamReader(
                        urlconn.getInputStream()));
                String line;
                if ((line = bin.readLine()) != null) {
                    return line;
                }
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return "";
    }

    public static String urlShorterGoogle(String source) {
        String urlString = "https://www.googleapis.com/urlshortener/v1/url";
        try {
            URL url = new URL(urlString);
            URLConnection uc = url.openConnection();
            uc.setDoOutput(true); // POST可能にする

            uc.setRequestProperty("Content-Type", "application/json"); // ヘッダを設定
            OutputStream os = uc.getOutputStream();

            String postStr = "{\"longUrl\": \"" + source + "\"}"; // 短縮したいURLを設定
            PrintStream ps = new PrintStream(os);
            ps.print(postStr); // データをPOSTする
            ps.close();

            InputStream is = uc.getInputStream(); // POSTした結果を取得
            BufferedReader reader = new BufferedReader(
                    new InputStreamReader(is));
            StringBuilder json = new StringBuilder();
            String s;
            while ((s = reader.readLine()) != null) {
                json.append(s);
            }
            reader.close();
            if (json.length() > 0 && json.indexOf("id") != -1) {
                JSONObject obj = new JSONObject(json.toString());
                return obj.getString("id");
            }
        } catch (Exception e) {

        }

        return null;
    }

    public static void checkSDCard(Context context) {
        String status = Environment.getExternalStorageState();
        if (status.equalsIgnoreCase(Environment.MEDIA_MOUNTED)) {
            // 正常
            return;
        } else {
            Toast.makeText(context, context.getString(R.string.txt_sdcard_err),
                    Toast.LENGTH_LONG).show();
        }
    }

    public static long getHideTime(SharedPreferences pref) {
        int type = Integer.parseInt(pref.getString("key.control_hide", "1"));
        long time = 0L;
        if (type == 1) {
            time = 2000L;
        }
        if (type == 2) {
            time = 5000L;
        }
        if (type == 3) {
            time = 10000L;
        }
        return time;
    }

    /**
     * AndroidManifestからVersionNameを取得
     * 
     * @param prefix
     * @param context
     * @return
     */
    public static String getVersionNumber(String prefix, Context context) {
        String versionName = prefix;
        PackageManager pm = context.getPackageManager();
        try {
            PackageInfo info = pm.getPackageInfo(context.getPackageName(),
                    PackageManager.GET_META_DATA);
            versionName += info.versionName;
        } catch (NameNotFoundException e) {
            versionName += "0";
        }
        return versionName;
    }

    public static long getCachSize(SharedPreferences pref) {
        /*
         * <item>0</item> <item>104857600</item> <item>209715200</item>
         * <item>524288000</item> <item>1073741824</item>
         * <item>2147483648</item> <item>5368709120</item>
         * <item>10737418240</item> <item>21474836480</item> <item>-1</item>
         */
        int size = Funcs.parseInt(pref.getString(SystemConsts.KEY_CACHE_SIZE,
                "4"));
        if (size == 0) {
            return 0;
        } else if (size == 1) {
            return 100;
        } else if (size == 2) {
            return 200;
        } else if (size == 3) {
            return 500;
        } else if (size == 4) {
            return 1024;
        } else if (size == 5) {
            return 2048;
        } else if (size == 6) {
            return 5120;
        } else if (size == 7) {
            return 10240;
        } else if (size == 8) {
            return 20480;
        } else if (size == 9) {
            return -1;
        } else {
            return 2147;
        }
    }
}
