package jp.co.kayo.android.localplayer.util;
/***
 * Copyright (c) 2010-2012 yokmama. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.ArrayList;

import jp.co.kayo.android.localplayer.BaseActivity;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.service.IMediaPlayerService;
import jp.co.kayo.android.localplayer.service.MediaPlayerService;
import jp.co.kayo.android.localplayer.service.StreamCacherServer;
import jp.co.kayo.android.localplayer.ui.PlayOrderListViewAdapter;
import jp.co.kayo.android.localplayer.util.bean.OrderInfo;

import android.annotation.TargetApi;
import android.content.Intent;
import android.content.SharedPreferences;
import android.nfc.NdefMessage;
import android.nfc.NdefRecord;
import android.nfc.NfcAdapter;
import android.nfc.NfcEvent;
import android.os.Handler;
import android.os.Message;
import android.os.Parcelable;
import android.os.RemoteException;
import android.preference.PreferenceManager;

@TargetApi(14)
public class NfcHelper implements NfcAdapter.CreateNdefMessageCallback,
        NfcAdapter.OnNdefPushCompleteCallback {
    NfcAdapter mNfcAdapter;
    BaseActivity mActivity;
    Handler mHandler;
    SharedPreferences mPref;
    int mSendCount;
    boolean mCanSentList = false;

    public static class BeamMessage implements Serializable {
        public String title;
        public String artist;
        public String album;
        public long duration;
        public String url;

        public static ArrayList<BeamMessage> toObject(byte[] bufs) {
            ByteArrayInputStream bi = null;
            try {
                bi = new ByteArrayInputStream(bufs);
                ObjectInputStream oi = new ObjectInputStream(bi);
                ArrayList<BeamMessage> pp = (ArrayList<BeamMessage>) oi
                        .readObject();
                return pp;
            } catch (Exception e) {
                Logger.e(e.getMessage(), e);
            } finally {
                if (bi != null) {
                    try {
                        bi.close();
                    } catch (IOException e) {
                    }
                }
            }
            return null;
        }

        public static byte[] toBytes(ArrayList<BeamMessage> msg) {
            ByteArrayOutputStream bo = null;
            try {
                bo = new ByteArrayOutputStream();
                ObjectOutputStream oo = new ObjectOutputStream(bo);
                oo.writeObject(msg);
                oo.close();
                return bo.toByteArray();
            } catch (IOException e) {
                Logger.e(e.getMessage(), e);
            } finally {
                if (bo != null) {
                    try {
                        bo.close();
                    } catch (IOException e) {
                    }
                }
            }
            return null;
        }
    }

    public NfcHelper(BaseActivity activity, Handler handler) {
        mActivity = activity;
        mPref = PreferenceManager.getDefaultSharedPreferences(mActivity);
        mHandler = handler;
        try {
            mNfcAdapter = NfcAdapter.getDefaultAdapter(mActivity);
            if (mNfcAdapter != null) {
                // Register callback to set NDEF message
                mNfcAdapter.setNdefPushMessageCallback(this, mActivity);
                // Register callback to listen for message-sent success
                mNfcAdapter.setOnNdefPushCompleteCallback(this, mActivity);
            }
        } catch (Exception e) {

        }
    }

    public String[] getProjection() {
        return ContentsUtils.isSDCard(mPref) ? PlayOrderListViewAdapter.projection_sdcard
                : PlayOrderListViewAdapter.projection_nonsdcard;
    }

    @Override
    public NdefMessage createNdefMessage(NfcEvent event) {
        mSendCount = 0;
        boolean useNfcServer = false;
        mCanSentList = false;
        IMediaPlayerService binder = mActivity.getBinder();
        ArrayList<BeamMessage> list = new ArrayList<BeamMessage>();
        long[] ids;

        try {
            ids = binder.getList();
            String[] projection = getProjection();

            int pos = binder.getPosition();
            int max = getBeamSoungCount();
            int index = 0;
            while (index < ids.length && index < max) {
                OrderInfo inf = new OrderInfo();
                inf.id = ids[pos];
                inf.index = pos;
                if (inf.id != 0) {
                    inf.load(mActivity, projection);

                    BeamMessage msg = new BeamMessage();
                    msg.album = inf.album;
                    msg.title = inf.title;
                    msg.artist = inf.artist;
                    msg.duration = inf.duration;
                    String uri = StreamCacherServer.getContentUri(mActivity,
                            inf.data);
                    if (uri != null) {
                        if (uri.startsWith("http")) {
                            msg.url = uri;
                        } else {
                            msg.url = URLEncoder.encode(inf.data);
                            useNfcServer = true;
                            // NFC共有の通知を開始
                            Intent service = new Intent(mActivity,
                                    MediaPlayerService.class);
                            service.setAction(SystemConsts.CALL_NFC_SHARE);
                            mActivity.startService(service);
                        }
                        list.add(msg);
                    }
                }
                index++;
                pos++;
                if (pos >= ids.length) {
                    pos = 0;
                }
            }
        } catch (RemoteException e) {
        } catch (MalformedURLException e) {
        } finally {
            mSendCount = list.size();
        }

        if (useNfcServer) {
            if (binder != null) {
                try {
                    if (binder.startNfcServer() && binder.startBeam()) {
                        mCanSentList = true;
                        String ipadr = NetworkHelper.getIPAddress(mActivity
                                .getApplicationContext());
                        if (ipadr != null) {
                            int port = binder.getPort();
                            String host = "http://" + ipadr + ":" + port + "/";
                            for (BeamMessage msg : list) {
                                if (msg.url.startsWith("http") != true) {
                                    msg.url = host + msg.url;
                                }
                                Logger.d("nfc send url:" + msg.url);
                            }
                        }
                    }
                } catch (RemoteException e) {
                } catch (IOException e) {
                    mCanSentList = false;
                }
            }
        } else {
            if (binder != null) {
                try {
                    binder.stopNfcServer();
                } catch (RemoteException e) {
                }
            }
            mCanSentList = true;
        }

        byte[] bufs = BeamMessage.toBytes(list);
        if (bufs != null) {
            NdefMessage msg = new NdefMessage(
                    new NdefRecord[] { 
                            createMimeRecord("application/jp.co.kayo.android.localplayer", bufs)
                            ,NdefRecord.createApplicationRecord("jp.co.kayo.android.localplayer") 
                            });
            return msg;
        }

        bufs = BeamMessage.toBytes(new ArrayList<BeamMessage>());
        NdefMessage msg = new NdefMessage(new NdefRecord[] { 
                createMimeRecord("application/jp.co.kayo.android.localplayer", bufs)
                ,NdefRecord.createApplicationRecord("jp.co.kayo.android.localplayer")
                });
        return msg;
    }

    @Override
    public void onNdefPushComplete(NfcEvent event) {
        Message msg = mHandler.obtainMessage(SystemConsts.EVT_NDEFPUSHCOMPLETE);
        msg.obj = new Object[] { mCanSentList, mSendCount };
        mHandler.sendMessage(msg);
    }

    /*public boolean isBeamMessage(Intent intent){
        Parcelable[] msgs = intent
                .getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES);
        return msgs!=null && msgs.length>0;
    }*/
    
    public ArrayList<BeamMessage> getNdefMessage(Intent intent) {
        if (intent != null) {
            if (NfcAdapter.ACTION_NDEF_DISCOVERED.equals(intent.getAction())) {
                Parcelable[] msgs = intent
                        .getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES);
                if(msgs!=null && msgs.length>0){
                    NdefMessage msg = (NdefMessage) msgs[0];
                    byte[] bufs = msg.getRecords()[0].getPayload();
                    if (bufs != null) {
                        return BeamMessage.toObject(bufs);
                    }
                }
            }
        }
        return null;
    }

    private NdefRecord createMimeRecord(String mimeType, byte[] data) {
        byte[] mimeBytes = mimeType.getBytes(Charset.forName("US-ASCII"));
        NdefRecord mimeRecord = new NdefRecord(NdefRecord.TNF_MIME_MEDIA,
                mimeBytes, new byte[0], data);
        return mimeRecord;
    }

    private int getBeamSoungCount() {
        int type = Integer
                .parseInt(mPref.getString("key.beam_songnumber", "0"));
        switch (type) {
        case 0:
            return 5;
        case 1:
            return 10;
        case 2:
            return 15;
        case 3:
        default:
            return 20;
        }
    }
}
